@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: errors.c,v 1.7 2002/07/10 05:35:21 pmhahn Exp $
+ * $Id: errors.c,v 1.20 2003/04/16 14:32:36 f1rmb Exp $
  *
  * xine engine error handling/notification
  *
@@ -33,11 +33,7 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 
-#include "event.h"
-#include "i18n.h"
-#include "viewlog.h"
-
-#include "xitk.h"
+#include "common.h"
 
 extern gGui_t          *gGui;
 
@@ -55,16 +51,23 @@ static void _errors_display_log(xitk_widget_t *w, void *data, int state) {
 /*
  * Create the real window.
  */
-void errors_create_window(char *title, char *message) {
-  
+static void errors_create_window(char *title, char *message) {
+  xitk_window_t *xw;
+
   if((title == NULL) || (message == NULL))
     return;
   
-  xitk_window_dialog_two_buttons_with_width(gGui->imlib_data, title, 
-					    _("Done"), _("More..."),
-					    NULL, _errors_display_log, 
-					    NULL, 400, ALIGN_CENTER,
-					    message);
+  xw = xitk_window_dialog_two_buttons_with_width(gGui->imlib_data, title, 
+						 _("Done"), _("More..."),
+						 NULL, _errors_display_log, 
+						 NULL, 400, ALIGN_CENTER,
+						 message);
+  if(!gGui->use_root_window) {
+    XLockDisplay(gGui->display);
+    XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
+    XUnlockDisplay(gGui->display);
+  }
+  layer_above_video(xitk_window_get_window(xw));
 }
 
 /*
@@ -97,9 +100,18 @@ void xine_error(char *message, ...) {
   }
   
   {
+    xitk_window_t *xw;
     char buf2[(strlen(buf) * 2) + 1];
+
     xitk_subst_special_chars(buf, buf2);
-    xitk_window_dialog_error(gGui->imlib_data, buf2);
+    xw = xitk_window_dialog_error(gGui->imlib_data, buf2);
+
+    if(!gGui->use_root_window) {
+      XLockDisplay(gGui->display);
+      XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
+      XUnlockDisplay(gGui->display);
+    }
+    layer_above_video(xitk_window_get_window(xw));
   }
   free(buf);
 }
@@ -171,9 +183,18 @@ void xine_info(char *message, ...) {
   }
   
   {
-    char buf2[(strlen(buf) * 2) + 1];
+    xitk_window_t *xw;
+    char           buf2[(strlen(buf) * 2) + 1];
+
     xitk_subst_special_chars(buf, buf2);
-    xitk_window_dialog_info(gGui->imlib_data, buf2);
+    xw = xitk_window_dialog_info(gGui->imlib_data, buf2);
+
+    if(!gGui->use_root_window) {
+      XLockDisplay(gGui->display);
+      XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
+      XUnlockDisplay(gGui->display);
+    }
+    layer_above_video(xitk_window_get_window(xw));
   }
   free(buf);
 }
@@ -181,10 +202,14 @@ void xine_info(char *message, ...) {
 /*
  * Display an error window error from a xine engine error.
  */
-void gui_handle_xine_error(void) {
-  int err;
+void gui_handle_xine_error(xine_stream_t *stream, char *mrl) {
+  int   err;
+  char *_mrl = mrl;
 
-  err = xine_get_error(gGui->xine);
+  if(_mrl == NULL)
+    mrl = (stream == gGui->stream) ? gGui->mmk.mrl : gGui->visual_anim.mrls[gGui->visual_anim.current];
+
+  err = xine_get_error(stream);
 
   switch(err) {
 
@@ -194,23 +219,37 @@ void gui_handle_xine_error(void) {
     
   case XINE_ERROR_NO_INPUT_PLUGIN:
     xine_error_with_more(_("- xine engine error -\n\n"
-			   "There is no input plugin available to handle '%s'.\nMaybe MRL syntax is wrong or file/stream source doesn't exist."),
-			 gGui->filename);
+			   "There is no input plugin available to handle '%s'.\n"
+			   "Maybe MRL syntax is wrong or file/stream source doesn't exist."),
+			 _mrl);
     break;
     
-  case XINE_ERROR_NO_DEMUXER_PLUGIN:
+  case XINE_ERROR_NO_DEMUX_PLUGIN:
     xine_error_with_more(_("- xine engine error -\n\nThere is no demuxer plugin available "
-			   "to handle '%s'.\nUsually this means that the file format was not recognized."), gGui->filename);
+			   "to handle '%s'.\n"
+			   "Usually this means that the file format was not recognized."), 
+			 _mrl);
+    break;
+    
+  case XINE_ERROR_DEMUX_FAILED:
+    xine_error_with_more(_("- xine engine error -\n\nDemuxer failed. "
+			   "Maybe '%s' is a broken file?\n"), _mrl);
+    break;
+    
+  case XINE_ERROR_MALFORMED_MRL:
+    xine_error_with_more(_("- xine engine error -\n\nMalformed mrl. "
+			   "Mrl '%s' seems malformed/invalid.\n"), _mrl);
     break;
 
-  case XINE_ERROR_DEMUXER_FAILED:
-    xine_error_with_more(_("- xine engine error -\n\ndemuxer failed. Maybe '%s' is a broken file?\n"), 
-			 gGui->filename);
+  case XINE_ERROR_INPUT_FAILED:
+    xine_error_with_more(_("- xine engine error -\n\nInput plugin failed to open "
+			   "mrl '%s'\n"), _mrl);
     break;
     
   default:
     xine_error_with_more(_("- xine engine error -\n\n!! Unhandled error !!\n"));
     break;
   }
-
+  
+  gGui->new_pos = -1;
 }
