@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: event.c,v 1.111 2002/07/31 00:50:40 f1rmb Exp $
+ * $Id: event.c,v 1.213 2003/05/05 22:15:01 f1rmb Exp $
  *
  * gui initialization and top-level event handling stuff
  *
@@ -27,6 +27,7 @@
 #endif
 
 #include <stdio.h>
+#include <sys/time.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -37,32 +38,17 @@
 #include <signal.h>
 #include <limits.h>
 #include <zlib.h>
+#include <sys/wait.h>
+#include <time.h>
+#include <stdlib.h>
 
-#include "Imlib-light/Imlib.h"
-
-#include "event.h"
-#include "playlist.h"
-#include "control.h"
-#include "lirc.h"
-#include "videowin.h"
-#include "panel.h"
-#include "actions.h"
-#include <xine/video_out_x11.h>
-#include <xine.h>
-#include <xine/xineutils.h>
-#include "mrl_browser.h"
-#include "skins.h"
-#include "errors.h"
-#include "network.h"
-#include "i18n.h"
-
-#include "xitk.h"
+#include "common.h"
 
 #ifdef HAVE_LIRC
 extern int no_lirc;
 #endif
 
-extern int errno;
+#define STEP_SIZE 256
 
 /*
  * global variables
@@ -92,21 +78,116 @@ static unsigned char xine_bits[] = {
    0x00, 0x00, 0xf8, 0x1f, 0x00, 0x00, 0x00, 0xf8
 };
 
-static void skip_by_chapter_cb(void *data, cfg_entry_t *cfg) {
+static char *exp_levels[] = {
+  "Beginner",
+  "Advanced",
+  "Expert",
+  "Master of the known universe",
+#ifdef DEBUG
+  "Debugger",
+#else  
+  NULL,
+#endif
+  NULL
+};
+
+static char *visual_anim_style[] = {
+  "None",
+  "Post Plugin",
+  "Stream Animation",
+  NULL
+};
+
+static void auto_vo_visibility_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->auto_vo_visibility = cfg->num_value;
+}
+static void auto_panel_visibility_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->auto_panel_visibility = cfg->num_value;
+}
+static void skip_by_chapter_cb(void *data, xine_cfg_entry_t *cfg) {
   gGui->skip_by_chapter = cfg->num_value;
   panel_update_nextprev_tips();
 }
-static void ssaver_timeout_cb(void *data, cfg_entry_t *cfg) {
+static void ssaver_timeout_cb(void *data, xine_cfg_entry_t *cfg) {
   gGui->ssaver_timeout = cfg->num_value;
 }
 
+static void visual_anim_cb(void *data, xine_cfg_entry_t *cfg) {
+  
+  if(((gGui->visual_anim.enabled == 2) && (cfg->num_value != 2)) && gGui->visual_anim.running)
+     visual_anim_stop();
+  
+  gGui->visual_anim.enabled = cfg->num_value;
+}
+
+static void stream_info_auto_update_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->stream_info_auto_update = cfg->num_value;
+  stream_infos_toggle_auto_update();
+}
+
+/*
+ * Layer above callbacks
+ */
+static void layer_above_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->layer_above = cfg->num_value;
+}
+static void always_layer_above_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->always_layer_above = cfg->num_value;
+}
+
 /*
  * Callback for snapshots saving location.
  */
-static void snapshot_loc_cb(void *data, cfg_entry_t *cfg) {
+static void snapshot_loc_cb(void *data, xine_cfg_entry_t *cfg) {
   gGui->snapshot_location = cfg->str_value;
 }
 
+/*
+ * Callback for skin server
+ */
+static void skin_server_url_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->skin_server_url = cfg->str_value;
+}
+
+/*
+ * OSD cbs
+ */
+static void osd_enabled_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->osd.enabled = cfg->num_value;
+}
+static void osd_timeout_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->osd.timeout = cfg->num_value;
+}
+
+static void smart_mode_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->smart_mode = cfg->num_value;
+}
+
+static void play_anyway_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->play_anyway = cfg->num_value;
+}
+
+static void exp_level_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->experience_level = (cfg->num_value * 10);
+}
+static void amp_level_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->mixer.amp = cfg->num_value;
+  xine_set_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL, gGui->mixer.amp);
+  osd_draw_bar(_("Amplification Level"), 0, 200, gGui->mixer.amp, OSD_BAR_STEPPER);
+}
+
+
+int wm_not_ewmh_only(void) {
+  int wm_type = xitk_get_wm_type();
+  
+  if((wm_type & WM_TYPE_GNOME_COMP) && !(wm_type & WM_TYPE_EWMH_COMP))
+    return 0;
+  else if(wm_type & WM_TYPE_EWMH_COMP)
+    return 1;
+  
+  return 0;
+}
+
 int actions_on_start(action_id_t actions[], action_id_t a) {
   int i = 0, num = 0;
   while(actions[i] != ACTID_NOKEY) {
@@ -117,16 +198,6 @@ int actions_on_start(action_id_t actions[], action_id_t a) {
   return num;
 }
 
-void config_save(void) {
-
-  gGui->config->save(gGui->config);
-}
-
-void config_reset(void) {
-
-  gGui->config->read(gGui->config, gGui->configfile);
-}
-
 /*
  *
  */
@@ -171,7 +242,7 @@ static void gui_signal_handler (int sig, void *data) {
       }
       alarm(5);
       
-      xine_stop(gGui->xine);
+      xine_stop(gGui->stream);
     }
     break;
   }
@@ -185,7 +256,6 @@ void gui_execute_action_id(action_id_t action) {
   xine_event_t   xine_event;
   
   if(action & ACTID_IS_INPUT_EVENT) {
-    
     if((action >= ACTID_EVENT_NUMBER_0) && (action <= ACTID_EVENT_NUMBER_9)) {
 
       if(!gGui->numeric.set) {
@@ -221,7 +291,12 @@ void gui_execute_action_id(action_id_t action) {
 
     /* events for advanced input plugins. */
     xine_event.type = action & ~ACTID_IS_INPUT_EVENT;
-    xine_send_event(gGui->xine, &xine_event);
+    xine_event.data_length = 0;
+    xine_event.data = NULL;
+    xine_event.stream = gGui->stream;
+    gettimeofday(&xine_event.tv, NULL);
+    
+    xine_event_send(gGui->stream, &xine_event);
     return;
   }
 
@@ -232,20 +307,23 @@ void gui_execute_action_id(action_id_t action) {
     break;
 
   case ACTID_WINDOWENLARGE:
-    video_window_set_mag (1.2 * video_window_get_mag());
+    video_window_set_mag (1.2 * (video_window_get_mag() + 0.002));
     break;
 
   case ACTID_ZOOM_1_1:
   case ACTID_WINDOW100:
     video_window_set_mag (1.0);
+    osd_display_info(_("Zoom: 1:1"));
     break;
 
   case ACTID_WINDOW200:
     video_window_set_mag (2.0);
+    osd_display_info(_("Zoom: 200%%"));
     break;
 
   case ACTID_WINDOW50:
     video_window_set_mag (0.5);
+    osd_display_info(_("Zoom: 50%%"));
     break;
 
   case ACTID_SPU_NEXT:
@@ -300,8 +378,18 @@ void gui_execute_action_id(action_id_t action) {
     gui_set_fullscreen_mode(NULL, NULL);
     break;
 
+#ifdef HAVE_XINERAMA
+  case ACTID_TOGGLE_XINERAMA_FULLSCR:
+    gui_set_xinerama_fullscreen_mode(NULL, NULL);
+    break;
+#endif
+
   case ACTID_TOGGLE_ASPECT_RATIO:
-    gui_toggle_aspect();
+    gui_toggle_aspect(-1);
+    break;
+
+  case ACTID_STREAM_INFOS:
+    gui_stream_infos_show(NULL, NULL);
     break;
 
   case ACTID_TOGGLE_INTERLEAVE:
@@ -320,10 +408,10 @@ void gui_execute_action_id(action_id_t action) {
     gui_stop(NULL, NULL);
     break;
 
-  case ACTID_SETUP:
-    gui_setup_show(NULL, NULL);
+  case ACTID_EVENT_SENDER:
+    gui_event_sender_show(NULL, NULL);
     break;
-
+    
   case ACTID_MRL_NEXT:
     if(!gGui->numeric.set)
       gui_nextprev(NULL, (void*)GUI_NEXT);
@@ -338,6 +426,10 @@ void gui_execute_action_id(action_id_t action) {
       gui_direct_nextprev(NULL, (void*)GUI_PREV, gGui->numeric.arg);
     break;
       
+  case ACTID_SETUP:
+    gui_setup_show(NULL, NULL);
+    break;
+
   case ACTID_EJECT:
     gui_eject(NULL, NULL);
     break;
@@ -441,17 +533,48 @@ void gui_execute_action_id(action_id_t action) {
     break;
     
   case ACTID_AV_SYNC_m3600:
-    xine_set_av_offset (gGui->xine, xine_get_av_offset (gGui->xine) - 3600);
-    break;
+    {
+      int offset = (xine_get_param(gGui->stream, XINE_PARAM_AV_OFFSET) - 3600);
 
+      gGui->mmk.offset = gGui->playlist.mmk[gGui->playlist.cur]->offset = offset;
+      xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, offset);
+      osd_display_info(_("A/V offset: %d"), offset);
+    }
+    break;
+    
   case ACTID_AV_SYNC_p3600:
-    xine_set_av_offset (gGui->xine, xine_get_av_offset (gGui->xine) + 3600);
+    {
+      int offset = (xine_get_param(gGui->stream, XINE_PARAM_AV_OFFSET) + 3600);
+      
+      gGui->mmk.offset = gGui->playlist.mmk[gGui->playlist.cur]->offset = offset;
+      xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, offset);
+      osd_display_info(_("A/V offset: %d"), offset);
+    }
     break;
 
   case ACTID_AV_SYNC_RESET:
-    xine_set_av_offset (gGui->xine, 0);
+    xine_set_param(gGui->stream, XINE_PARAM_AV_OFFSET, 0);
+    osd_display_info(_("A/V Offset: reset."));
     break;
 
+  case ACTID_SV_SYNC_p:
+    {
+      int offset = xine_get_param(gGui->stream, XINE_PARAM_SPU_OFFSET) + 3600;
+      
+      xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, offset);
+      osd_display_info(_("SPU Offset: %d"), offset);
+    }
+    break;
+
+  case ACTID_SV_SYNC_m:
+    {
+      int offset = xine_get_param(gGui->stream, XINE_PARAM_SPU_OFFSET) - 3600;
+      
+      xine_set_param(gGui->stream, XINE_PARAM_SPU_OFFSET, offset);
+      osd_display_info(_("SPU Offset: %d"), offset);
+    }
+    break;
+    
   case ACTID_SPEED_FAST:
     gui_change_speed_playback(NULL, (void*)GUI_PREV);
     break;
@@ -472,32 +595,65 @@ void gui_execute_action_id(action_id_t action) {
     gui_decrease_audio_volume();
     break;
 
+  case ACTID_pAMP:
+    gui_increase_amp_level();
+    break;
+
+  case ACTID_mAMP:
+    gui_decrease_amp_level();
+    break;
+
+  case ACTID_AMP_RESET:
+    gui_reset_amp_level();
+    break;
+
   case ACTID_SNAPSHOT:
     panel_snapshot(NULL, NULL);
     break;
     
   case ACTID_ZOOM_IN:
-    gui_change_zoom(1);
+    gui_change_zoom(1,1);
     break;
 
   case ACTID_ZOOM_OUT:
-    gui_change_zoom(-1);
+    gui_change_zoom(-1,-1);
     break;
 
+  case ACTID_ZOOM_X_IN:
+    gui_change_zoom(1,0);
+    break;
+  
+  case ACTID_ZOOM_X_OUT:
+    gui_change_zoom(-1,0);
+    break;
+
+  case ACTID_ZOOM_Y_IN:
+    gui_change_zoom(0,1);
+    break;
+  
+  case ACTID_ZOOM_Y_OUT:
+    gui_change_zoom(0,-1);
+    break;
+  
   case ACTID_ZOOM_RESET:
     gui_reset_zoom();
     break;
     
   case ACTID_GRAB_POINTER:
     if(!gGui->cursor_grabbed) {
-      if(!panel_is_visible())
+      if(!panel_is_visible()) {
+	XLockDisplay(gGui->display);
 	XGrabPointer(gGui->display, gGui->video_window, 1, None, 
 		     GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
+	XUnlockDisplay(gGui->display);
+      }
       
       gGui->cursor_grabbed = 1;
     }
     else {
+      XLockDisplay(gGui->display);
       XUngrabPointer(gGui->display, CurrentTime);
+      XUnlockDisplay(gGui->display);
       gGui->cursor_grabbed = 0;
     }
     break;
@@ -514,6 +670,129 @@ void gui_execute_action_id(action_id_t action) {
     gui_kbedit_show(NULL, NULL);
     break;
 
+  case ACTID_DPMSSTANDBY:
+    xine_system(0, "xset dpms force standby");
+    break;
+
+  case ACTID_MRLIDENTTOGGLE:
+    gGui->is_display_mrl = !gGui->is_display_mrl;
+    panel_update_mrl_display();
+    playlist_mrlident_toggle();
+    break;
+    
+  case ACTID_SCANPLAYLIST:
+    playlist_scan_for_infos();
+    break;
+
+  case ACTID_MMKEDITOR:
+    playlist_mmk_editor();
+    break;
+
+  case ACTID_SUBSELECT:
+    gui_select_sub();
+    break;
+
+  case ACTID_LOOPMODE:
+    gGui->playlist.loop++;
+    if(gGui->playlist.loop == PLAYLIST_LOOP_MODES_NUM)
+      gGui->playlist.loop = PLAYLIST_LOOP_NO_LOOP;
+    
+    /* Temporary informations */    
+    switch(gGui->playlist.loop) {
+    case PLAYLIST_LOOP_NO_LOOP:
+      osd_display_info(_("Playlist: no loop."));
+      break;
+    case PLAYLIST_LOOP_LOOP:
+      osd_display_info(_("Playlist: loop."));
+      break;
+    case PLAYLIST_LOOP_REPEAT:
+      osd_display_info(_("Playlist: entry repeat."));
+      break;
+    case PLAYLIST_LOOP_SHUFFLE:
+      osd_display_info(_("Playlist: shuffle."));
+      break;
+    case PLAYLIST_LOOP_SHUF_PLUS:
+      osd_display_info(_("Playlist: shuffle forever."));
+      break;
+    }
+    break;
+    
+  case ACTID_ADDMEDIAMARK:
+    gui_add_mediamark();
+    break;
+
+  case ACTID_SKINDOWNLOAD:
+    download_skin(gGui->skin_server_url);
+    break;
+
+  case ACTID_OSD_SINFOS:
+    osd_stream_infos();
+    break;
+
+  case ACTID_FILESELECTOR:
+    gui_file_selector();
+    break;
+
+  case ACTID_HUECONTROLp:
+    if(gGui->video_settings.hue <= (65535 - STEP_SIZE)) {
+      config_update_num("gui.vo_hue", gGui->video_settings.hue + STEP_SIZE);
+      control_set_image_prop(XINE_PARAM_VO_HUE, gGui->video_settings.hue);
+      osd_draw_bar(_("Hue"), 0, 65535, gGui->video_settings.hue, OSD_BAR_STEPPER);
+    }
+    break;
+  case ACTID_HUECONTROLm:
+    if(gGui->video_settings.hue >= STEP_SIZE) {
+      config_update_num("gui.vo_hue", gGui->video_settings.hue - STEP_SIZE);
+      control_set_image_prop(XINE_PARAM_VO_HUE, gGui->video_settings.hue);
+      osd_draw_bar(_("Hue"), 0, 65535, gGui->video_settings.hue, OSD_BAR_STEPPER);
+    }
+    break;
+
+  case ACTID_SATURATIONCONTROLp:
+    if(gGui->video_settings.saturation <= (65535 - STEP_SIZE)) {
+      config_update_num("gui.vo_saturation", gGui->video_settings.saturation + STEP_SIZE);
+      control_set_image_prop(XINE_PARAM_VO_SATURATION, gGui->video_settings.saturation);
+      osd_draw_bar(_("Saturation"), 0, 65535, gGui->video_settings.saturation, OSD_BAR_STEPPER);
+    }
+    break;
+  case ACTID_SATURATIONCONTROLm:
+    if(gGui->video_settings.saturation >= STEP_SIZE) {
+      config_update_num("gui.vo_saturation", gGui->video_settings.saturation - STEP_SIZE);
+      control_set_image_prop(XINE_PARAM_VO_SATURATION, gGui->video_settings.saturation);
+      osd_draw_bar(_("Saturation"), 0, 65535, gGui->video_settings.saturation, OSD_BAR_STEPPER);
+    }
+    break;
+
+  case ACTID_BRIGHTNESSCONTROLp:
+    if(gGui->video_settings.brightness <= (65535 - STEP_SIZE)) {
+      config_update_num("gui.vo_brightness", gGui->video_settings.brightness + STEP_SIZE);
+      control_set_image_prop(XINE_PARAM_VO_BRIGHTNESS, gGui->video_settings.brightness);
+      osd_draw_bar(_("Brightness"), 0, 65535, gGui->video_settings.brightness, OSD_BAR_STEPPER);
+    }
+    break;
+  case ACTID_BRIGHTNESSCONTROLm:
+    if(gGui->video_settings.brightness >= STEP_SIZE) {
+      config_update_num("gui.vo_brightness", gGui->video_settings.brightness - STEP_SIZE);
+      control_set_image_prop(XINE_PARAM_VO_BRIGHTNESS, gGui->video_settings.brightness);
+      osd_draw_bar(_("Brightness"), 0, 65535, gGui->video_settings.brightness, OSD_BAR_STEPPER);
+    }
+    break;
+
+  case ACTID_CONTRASTCONTROLp:
+    if(gGui->video_settings.contrast <= (65535 - STEP_SIZE)) {
+      config_update_num("gui.vo_contrast", gGui->video_settings.contrast + STEP_SIZE);
+      control_set_image_prop(XINE_PARAM_VO_CONTRAST, gGui->video_settings.contrast);
+      osd_draw_bar(_("Contrast"), 0, 65535, gGui->video_settings.contrast, OSD_BAR_STEPPER);
+    }
+    break;
+  case ACTID_CONTRASTCONTROLm:
+    if(gGui->video_settings.contrast >= STEP_SIZE) {
+      config_update_num("gui.vo_contrast", gGui->video_settings.contrast - STEP_SIZE);
+      control_set_image_prop(XINE_PARAM_VO_CONTRAST, gGui->video_settings.contrast);
+      osd_draw_bar(_("Contrast"), 0, 65535, gGui->video_settings.contrast, OSD_BAR_STEPPER);
+    }
+    break;
+
   default:
     break;
   }
@@ -565,10 +844,6 @@ void gui_handle_event (XEvent *event, void *data) {
   case ConfigureNotify:
     break;
 
-  case ClientMessage:
-    xitk_process_client_dnd_message (&gGui->xdnd, event);
-    break;
-
     /*
       default:
       printf("Got event: %i\n", event->type);
@@ -578,60 +853,86 @@ void gui_handle_event (XEvent *event, void *data) {
 }
 
 /*
- * Callback functions called by Xine engine.
+ * Start playback to next entry in playlist (or stop the engine, then display logo).
  */
+void gui_playlist_start_next(void) {
 
-void gui_status_callback (int nStatus) {
-
-  if (gGui->ignore_status)
+  if (gGui->ignore_next)
     return;
 
-  /* printf ("gui status callback : %d\n", nStatus); */
+  panel_reset_slider ();
   
-  if (nStatus == XINE_STOP) {
-    gGui->playlist_cur++;
-    panel_reset_slider ();
-
-    if (gGui->playlist_cur < gGui->playlist_num) {
-      gui_set_current_mrl(gGui->playlist[gGui->playlist_cur]);
-      if(!xine_play (gGui->xine, gGui->filename, 0, 0 ))
-	gui_handle_xine_error();
-
-    } else {
-      
-      if(gGui->actions_on_start[0] == ACTID_QUIT)
-	gui_exit(NULL, NULL);
-      
-      gGui->playlist_cur--;
+  switch(gGui->playlist.loop) {
+    
+  case PLAYLIST_LOOP_NO_LOOP:
+  case PLAYLIST_LOOP_LOOP:
+    gGui->playlist.cur++;
+    
+    if(gGui->playlist.cur < gGui->playlist.num) {
+      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
+				 gGui->mmk.start, gGui->mmk.offset))
+	gui_display_logo();
     }
-  }
+    else {
+      
+      if(gGui->playlist.loop == PLAYLIST_LOOP_NO_LOOP) {
+	if(gGui->actions_on_start[0] == ACTID_QUIT)
+	  gui_exit(NULL, NULL);
 
-  if(is_playback_widgets_enabled() && (!gGui->playlist_num) && (!gGui->filename)) {
+	gGui->playlist.cur--;
+	mediamark_reset_played_state();
+	gui_display_logo();
+      }
+      else if(gGui->playlist.loop == PLAYLIST_LOOP_LOOP) {
+	gGui->playlist.cur = 0;
+	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
+				   gGui->mmk.start, gGui->mmk.offset))
+	  gui_display_logo();
+      }
+
+    }
+    break;
+    
+  case PLAYLIST_LOOP_REPEAT:
+    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+    if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
+			       gGui->mmk.start, gGui->mmk.offset))
+      gui_display_logo();
+    break;
+
+  case PLAYLIST_LOOP_SHUFFLE:
+  case PLAYLIST_LOOP_SHUF_PLUS:
+    if(!mediamark_all_played()) {
+      
+    __shuffle_restart:
+      gGui->playlist.cur = mediamark_get_shuffle_next();
+      
+      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      if(!gui_xine_open_and_play(gGui->mmk.mrl, gGui->mmk.sub, 0, 
+				 gGui->mmk.start, gGui->mmk.offset))
+	gui_display_logo();
+    }
+    else {
+      mediamark_reset_played_state();
+
+      if(gGui->playlist.loop == PLAYLIST_LOOP_SHUF_PLUS)
+	goto __shuffle_restart;
+
+      gui_display_logo();
+    }
+    break;
+        
+  }
+  
+  if(is_playback_widgets_enabled() && (!gGui->playlist.num)) {
     gui_set_current_mrl(NULL);
     enable_playback_controls(0);
   }
 
 }
 
-char *gui_next_mrl_callback (void) {
-
-  if (gGui->playlist_cur >= (gGui->playlist_num-1)) 
-    return NULL;
-
-  return gGui->playlist[gGui->playlist_cur+1];
-}
-
-void gui_branched_callback (void) {
-
-  if (gGui->playlist_cur < (gGui->playlist_num-1)) {
-  
-    gGui->playlist_cur++;
-    panel_reset_slider ();
-
-    gui_set_current_mrl(gGui->playlist[gGui->playlist_cur]);
-  }
-}
-
 static void gui_find_visual (Visual **visual_return, int *depth_return) {
   XWindowAttributes  attribs;
   XVisualInfo	    *vinfo;
@@ -719,27 +1020,40 @@ static void gui_find_visual (Visual **visual_return, int *depth_return) {
     *visual_return = visual;
 }
 
+void gui_deinit(void) {
+  xitk_unregister_event_handler(&gGui->widget_key);
+}
+
 /*
  * Initialize the GUI
  */
 void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attribute) {
-  int                   i;
-  char                 *display_name = ":0.0";
+  int    i;
+  char  *server;
 
   /*
    * init playlist
    */
+  for (i = 0; i < nfiles; i++) {
+    char *sub;
+    
+    if((sub = strstr(filenames[i], "::")) != NULL) {
+      *sub = 0;
+      sub += 2;
+    }
+    
+    mediamark_add_entry((const char *)filenames[i], (const char *)filenames[i], sub, 0, -1, 0);
+  }
+  
+  if((gGui->playlist.loop == PLAYLIST_LOOP_SHUFFLE) || 
+     (gGui->playlist.loop == PLAYLIST_LOOP_SHUF_PLUS))
+    gGui->playlist.cur = mediamark_get_shuffle_next();
+  
+  gGui->is_display_mrl = 0;
+  gGui->mrl_overrided = 0;
+  gGui->new_pos = -1;
 
-  for (i = 0; i < nfiles; i++)
-    gGui->playlist[i] = filenames[i];
-
-  gGui->playlist_num = nfiles; 
-  gGui->playlist_cur = 0;
-
-  if (nfiles)
-    strcpy(gGui->filename, gGui->playlist [gGui->playlist_cur]);
-  else 
-    panel_set_no_mrl();
+  gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
 
   /*
    * X / imlib stuff
@@ -751,41 +1065,182 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
     exit (1);
   } 
   
-  if(getenv("DISPLAY"))
-    display_name = getenv("DISPLAY");
-  
-  if((gGui->display = XOpenDisplay(display_name)) == NULL) {
+  if((gGui->display = XOpenDisplay((getenv("DISPLAY")))) == NULL) {
     fprintf(stderr, _("Cannot open display\n"));
     exit(1);
   }
 
-  if (gGui->config->register_bool (gGui->config, "gui.xsynchronize", 0,
-				   _("synchronized X protocol (debug)"), NULL, NULL, NULL)) {
+  if (xine_config_register_bool (gGui->xine, "gui.xsynchronize", 
+				 0,
+				 _("synchronized X protocol (debug)"), 
+				 CONFIG_NO_HELP,
+				 CONFIG_LEVEL_ADV,
+				 CONFIG_NO_CB,
+				 CONFIG_NO_DATA)) {
+
     XSynchronize (gGui->display, True);
     fprintf (stderr, _("Warning! Synchronized X activated - this is way slow...\n"));
   }
 
   gGui->layer_above = 
-    gGui->config->register_bool (gGui->config, "gui.layer_above", 1,
-				 _("use wm layer property to place window on top"), 
-				 NULL, NULL, NULL);
-  
+    xine_config_register_bool (gGui->xine, "gui.layer_above", 0,
+			       _("Windows stacking"),
+			       _("Use wm layer property to place window on top."), 
+			       CONFIG_LEVEL_ADV,
+			       layer_above_cb,
+			       CONFIG_NO_DATA);
+
+  gGui->always_layer_above = 
+    xine_config_register_bool (gGui->xine, "gui.always_layer_above", 0,
+			       _("Windows stacking (more)"),
+			       _("Use WM layer property to place windows on top."), 
+			       CONFIG_LEVEL_ADV,
+			       always_layer_above_cb,
+			       CONFIG_NO_DATA);
+
+  if(gGui->always_layer_above && (!gGui->layer_above))
+    config_update_num("gui.layer_above", 1);
+
   gGui->snapshot_location = 
-    gGui->config->register_string (gGui->config, "gui.snapshotdir", 
-				   (char *) (xine_get_homedir()),
-				   _("where snapshots will be saved"),
-				   NULL, snapshot_loc_cb, NULL);
+    (char *)xine_config_register_string (gGui->xine, "gui.snapshotdir", 
+					 (char *) (xine_get_homedir()),
+					 _("Snapshot location"),
+					 _("Where snapshots will be saved."),
+					 CONFIG_LEVEL_BEG,
+					 snapshot_loc_cb,
+					 CONFIG_NO_DATA);
   
   gGui->ssaver_timeout =
-    gGui->config->register_num (gGui->config, "gui.screensaver_timeout", 10,
-				_("time between two screensaver fake events, 0 to disable"),
-				NULL, ssaver_timeout_cb, NULL);
+    xine_config_register_num (gGui->xine, "gui.screensaver_timeout", 10,
+			      _("Screensaver wakeup"),
+			      _("Time between two screensaver fake events, 0 to disable."),
+			      CONFIG_LEVEL_ADV,
+			      ssaver_timeout_cb,
+			      CONFIG_NO_DATA);
   
   gGui->skip_by_chapter = 
-    gGui->config->register_bool (gGui->config, "gui.skip_by_chapter", 1,
-				 _("play next|previous chapter instead of mrl (dvdnav)"), 
-				 NULL, skip_by_chapter_cb, NULL);
+    xine_config_register_bool (gGui->xine, "gui.skip_by_chapter", 1,
+			       _("Chapter hopping"),
+			       _("Play next|previous chapter instead of mrl (dvdnav)"), 
+			       CONFIG_LEVEL_ADV,
+			       skip_by_chapter_cb, 
+			       CONFIG_NO_DATA);
   
+  gGui->auto_vo_visibility = 
+    xine_config_register_bool (gGui->xine, "gui.auto_video_output_visibility", 0,
+			       _("Visibility behavior of output window"),
+			       _("Hide video output window if there is no video in the stream"), 
+			       CONFIG_LEVEL_ADV,
+			       auto_vo_visibility_cb, 
+			       CONFIG_NO_DATA);
+
+  gGui->auto_panel_visibility = 
+    xine_config_register_bool (gGui->xine, "gui.auto_panel_visibility", 0,
+			       _("Visiblility behavior of panel"),
+			       _("Automatically show/hide panel window, according to auto_video_output_visibility"), 
+			       CONFIG_LEVEL_ADV,
+			       auto_panel_visibility_cb,
+			       CONFIG_NO_DATA); 
+ 
+  gGui->eventer_sticky = 
+    xine_config_register_bool(gGui->xine, "gui.eventer_sticky", 
+			      1,
+			      _("Event sender behavior"),
+			      _("Event sender window stick to main panel"), 
+			      CONFIG_LEVEL_ADV,
+			      event_sender_sticky_cb,
+			      CONFIG_NO_DATA);
+
+  gGui->visual_anim.enabled = 
+    xine_config_register_enum(gGui->xine, "gui.visual_anim", 
+			      1, /* Post plugin */
+			      visual_anim_style,
+			      _("Visual animation style"),
+			      _("Display some video animations when "
+				"current stream is audio only (eg: mp3)."), 
+			      CONFIG_LEVEL_BEG,
+			      visual_anim_cb,
+			      CONFIG_NO_DATA);
+
+  gGui->stream_info_auto_update = 
+    xine_config_register_bool(gGui->xine, "gui.sinfo_auto_update", 
+			      0,
+			      _("Stream information"),
+			      _("Update stream information (in stream info window) "
+				"each half seconds."), 
+			      CONFIG_LEVEL_ADV,
+			      stream_info_auto_update_cb,
+			      CONFIG_NO_DATA);
+  
+
+  server = 
+    (char *)xine_config_register_string (gGui->xine, "gui.skin_server_url", 
+					 SKIN_SERVER_URL,
+					 _("Skin Server Url"),
+					 _("From where we can get skins."),
+					 CONFIG_LEVEL_ADV,
+					 skin_server_url_cb,
+					 CONFIG_NO_DATA);
+  
+  config_update_string("gui.skin_server_url", 
+		       gGui->skin_server_url ? gGui->skin_server_url : server);
+  
+  gGui->osd.enabled = 
+    xine_config_register_bool(gGui->xine, "gui.osd_enabled", 
+			      1,
+			      _("Enable OSD support"),
+			      _("Enabling OSD permit to display some status/informations "
+				"in output window."), 
+			      CONFIG_LEVEL_BEG,
+			      osd_enabled_cb,
+			      CONFIG_NO_DATA);
+
+  gGui->osd.timeout = 
+    xine_config_register_num(gGui->xine, "gui.osd_timeout", 
+			      3,
+			      _("Dismiss OSD time (s)"),
+			      _("Persistence time of OSD visual, in seconds."),
+			      CONFIG_LEVEL_BEG,
+			      osd_timeout_cb,
+			      CONFIG_NO_DATA);
+
+  gGui->smart_mode = 
+    xine_config_register_bool(gGui->xine, "gui.smart_mode", 
+			      1,
+			      _("change xine's behavior for unexperienced user"), 
+			      _("In this mode, xine take some decisions to simplify user's life."),
+			      CONFIG_LEVEL_BEG,
+			      smart_mode_cb,
+			      CONFIG_NO_DATA);
+
+  gGui->play_anyway = 
+    xine_config_register_bool(gGui->xine, "gui.play_anyway", 
+			      0,
+			      _("Ask user for playback with unsupported codec"),
+			      _("If xine don't support audio or video codec of current stream "
+				"the user will be asked if the stream should be played or not"), 
+			      CONFIG_LEVEL_BEG,
+			      play_anyway_cb,
+			      CONFIG_NO_DATA);
+
+  gGui->experience_level =
+    (xine_config_register_enum(gGui->xine, "gui.experience_level", 
+			       0, exp_levels,
+			       _("configuration experience level"),
+			       _("Level of user's experience, this will show more or less "
+				 "configuration options."), 
+			       CONFIG_LEVEL_BEG,
+			       exp_level_cb, 
+			       CONFIG_NO_DATA)) * 10;
+
+  gGui->mixer.amp =  xine_config_register_range(gGui->xine, "gui.amp_level", 
+						100, 0, 200,
+						_("amplification level"),
+						NULL,
+						CONFIG_LEVEL_ADV,
+						amp_level_cb, 
+						CONFIG_NO_DATA);
+
   gGui->numeric.set = 0;
   gGui->numeric.arg = 0;
 
@@ -802,7 +1257,19 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
 	 , XScreenCount(gGui->display), gGui->screen);
   printf(_("        Depth: %d.\n"),
 	 XDisplayPlanes(gGui->display, gGui->screen));
-
+#ifdef HAVE_SHM
+  if(XShmQueryExtension(gGui->display)) {
+    int major, minor, ignore;
+    Bool pixmaps;
+    
+    if(XQueryExtension(gGui->display, "MIT-SHM", &ignore, &ignore, &ignore)) {
+      if(XShmQueryVersion(gGui->display, &major, &minor, &pixmaps ) == True) {
+	printf(_("        XShmQueryVersion: %d.%d.\n"), major, minor);
+      }
+    }
+  }
+#endif
+  
   gui_find_visual(&gGui->visual, &gGui->depth);
 
   gui_init_imlib (gGui->visual);
@@ -818,16 +1285,20 @@ void gui_init (int nfiles, char *filenames[], window_attributes_t *window_attrib
   
   init_skins_support();
 
-  gGui->kbindings = kbindings_init_kbinding();
-
+  gGui->on_quit = 0;
   gGui->running = 1;
-
+  
   video_window_init (window_attribute, 
 		     ((actions_on_start(gGui->actions_on_start, 
 				       ACTID_TOGGLE_WINOUT_VISIBLITY)) ? 1 : 0));
 
-  panel_init ();
+  /* kbinding might open an error dialog (double keymapping), which produces a segfault,
+   * when done before the video_window_init(). */
+  gGui->kbindings = kbindings_init_kbinding();
 
+  panel_init ();
+  
+  (void *) getcwd(&(gGui->curdir[0]), XITK_PATH_MAX);
 }
 
 void gui_init_imlib (Visual *vis) {
@@ -881,8 +1352,21 @@ void gui_init_imlib (Visual *vis) {
 /*
  *
  */
-void gui_run (void) {
-  int i;
+static void on_start(void *data) {
+  int start = (int) data;
+
+  gui_display_logo();
+  
+  do {
+    xine_usec_sleep(5000);
+  } while(gGui->logo_mode != 1);
+
+  if(start)
+    gui_execute_action_id(ACTID_PLAY);
+}
+
+void gui_run(void) {
+  int       i, auto_start = 0;
   
   video_window_change_skins();
   panel_add_autoplay_buttons();
@@ -891,75 +1375,38 @@ void gui_run (void) {
   panel_update_mrl_display ();
   panel_update_runtime_display();
 
+  /* Register config entries related to video control settings */
+  control_config_register();
+
   /* autoscan playlist  */
   if(gGui->autoscan_plugin != NULL) {
-    char **autoscan_plugins = xine_get_autoplay_input_plugin_ids(gGui->xine);
-    
-    int i;
+    const char *const *autoscan_plugins = xine_get_autoplay_input_plugin_ids(gGui->xine);
+    int         i;
 
     if(autoscan_plugins) {
 
       for(i=0; autoscan_plugins[i] != NULL; ++i) {
 
 	if(!strcasecmp(autoscan_plugins[i], gGui->autoscan_plugin)) {
-	  int num_mrls;
+	  int                num_mrls, j;
 	  char **autoplay_mrls = xine_get_autoplay_mrls (gGui->xine,
 							 gGui->autoscan_plugin,
 							 &num_mrls);
-	  int j;
 	  
 	  if(autoplay_mrls) {
 	    for (j = 0; j < num_mrls; j++)
-	      gGui->playlist[gGui->playlist_num + j] = autoplay_mrls[j];
+	      mediamark_add_entry((const char *)autoplay_mrls[j],
+				  (const char *)autoplay_mrls[j], NULL, 0, -1, 0);
 	   
-	    gGui->playlist_num += j;
-	    gGui->playlist_cur = 0;
-	    gui_set_current_mrl(gGui->playlist[gGui->playlist_cur]);
+	    gGui->playlist.cur = 0;
+	    gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
 	  }
 	}    
       }
     }
   }  
-
-  enable_playback_controls((gGui->playlist_num > 0));
   
-  if(gGui->actions_on_start[0] != ACTID_NOKEY) {
-
-    /* Popup setup window if there is no config file */
-    if(actions_on_start(gGui->actions_on_start, ACTID_SETUP))
-      gui_execute_action_id(ACTID_SETUP);
-    
-    /*  The user wants to hide control panel  */
-    if(panel_is_visible() && (actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_VISIBLITY)))
-      gui_execute_action_id(ACTID_TOGGLE_VISIBLITY);
-
-    /*  The user wants to hide video window  */
-    if(actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_WINOUT_VISIBLITY)) {
-      if(!panel_is_visible())
-	gui_execute_action_id(ACTID_TOGGLE_VISIBLITY);
-
-      gGui->vo_driver->gui_data_exchange (gGui->vo_driver, 
-					  GUI_DATA_EX_VIDEOWIN_VISIBLE, 
-					  (int *)0);
-      
-    }
-
-    /*  The user wants to see in fullscreen mode  */
-    for (i = actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_FULLSCREEN); i > 0; i--)
-      gui_execute_action_id(ACTID_TOGGLE_FULLSCREEN);
-    
-    /*  The user request "play on start" */
-    if(actions_on_start(gGui->actions_on_start, ACTID_PLAY))
-      
-      if(gGui->playlist[0] != NULL)
-	gui_execute_action_id(ACTID_PLAY);
-      
-    /* Flush actions on start */
-    if(actions_on_start(gGui->actions_on_start, ACTID_QUIT)) {
-      gGui->actions_on_start[0] = ACTID_QUIT;
-      gGui->actions_on_start[1] = ACTID_NOKEY;
-    }
-  }
+  enable_playback_controls((gGui->playlist.num > 0));
 
   /* We can't handle signals here, xitk handle this, so
    * give a function callback for this.
@@ -984,13 +1431,71 @@ void gui_run (void) {
 						 NULL, NULL);
   
   start_remote_server();
+  init_session();
 
   /* Need for tvmode */
   video_window_set_fullscreen_mode((video_window_get_fullscreen_mode()));
 
-  xitk_run();
+  if(gGui->actions_on_start[0] != ACTID_NOKEY) {
+
+    /* Popup setup window if there is no config file */
+    if(actions_on_start(gGui->actions_on_start, ACTID_SETUP)) {
+      config_save();
+      gui_execute_action_id(ACTID_SETUP);
+    }
+    
+    /*  The user wants to hide video window  */
+    if(actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_WINOUT_VISIBLITY)) {
+      if(!panel_is_visible())
+	gui_execute_action_id(ACTID_TOGGLE_VISIBLITY);
+      
+      /* DXR3 case */
+      if(video_window_is_visible())
+	video_window_set_visibility(!(video_window_is_visible()));
+      else
+	xine_gui_send_vo_data(gGui->stream,
+			      XINE_GUI_SEND_VIDEOWIN_VISIBLE,
+			      (int *)0);
+      
+    }
+
+    /* The user wants to see in fullscreen mode  */
+    for (i = actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_FULLSCREEN); i > 0; i--)
+      gui_execute_action_id(ACTID_TOGGLE_FULLSCREEN);
+
+    /* The user wants to see in xinerama fullscreen mode  */
+    for (i = actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_XINERAMA_FULLSCR); i > 0; i--)
+      gui_execute_action_id(ACTID_TOGGLE_XINERAMA_FULLSCR);
+
+    /* User load a playlist on startup */
+    if(actions_on_start(gGui->actions_on_start, ACTID_PLAYLIST)) {
+      mediamark_load_mediamarks(gGui->playlist.on_start);
+      gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+      if((!is_playback_widgets_enabled()) && gGui->playlist.num)
+	enable_playback_controls(1);
+    }
+
+    if(actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_INTERLEAVE))
+      gui_toggle_interlaced();
+    
+    /*  The user request "play on start" */
+    if(actions_on_start(gGui->actions_on_start, ACTID_PLAY)) {
+      if((mediamark_get_current_mrl()) != NULL) {
+	auto_start = 1;
+      }
+    }
+
+    /* Flush actions on start */
+    if(actions_on_start(gGui->actions_on_start, ACTID_QUIT)) {
+      gGui->actions_on_start[0] = ACTID_QUIT;
+      gGui->actions_on_start[1] = ACTID_NOKEY;
+    }
+  }
+
+  xitk_run(on_start, (void *)auto_start);
 
   gGui->running = 0;
+  deinit_session();
 
   kbindings_save_kbinding(gGui->kbindings);
   kbindings_free_kbinding(&gGui->kbindings);
