@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,22 +17,36 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: playlist.h,v 1.10 2002/06/14 09:32:37 f1rmb Exp $
+ * $Id: playlist.h,v 1.17 2003/03/02 00:54:45 f1rmb Exp $
  *
  */
 
 #ifndef PLAYLIST_H
 #define PLAYLIST_H
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include "xitk.h"
+
+void playlist_mmk_editor(void);
+void playlist_scan_for_infos(void);
+void playlist_show_tips(int enabled, unsigned long timeout);
+void playlist_mrlident_toggle(void);
+void playlist_update_playlist(void);
+void playlist_exit(xitk_widget_t *, void *);
+int playlist_is_running(void);
+int playlist_is_visible(void);
+void playlist_scan_input(xitk_widget_t *, void *);
+void playlist_raise_window(void);
+void playlist_toggle_visibility(xitk_widget_t *, void *);
+void playlist_update_focused_entry(void);
+void playlist_change_skins(void);
+void playlist_delete_all(xitk_widget_t *w, void *data);
+void playlist_deinit(void);
 void playlist_editor(void);
-void playlist_change_skins(void);
-void pl_exit(xitk_widget_t *, void *);
-int pl_is_running(void);
-int pl_is_visible(void);
-void pl_toggle_visibility(xitk_widget_t *, void *);
-void pl_raise_window(void);
-void pl_scan_input(xitk_widget_t *, void *);
-void pl_update_playlist(void);
-void playlist_show_tips(int enabled, unsigned long timeout);
-void pl_update_focused_entry(void);
+void playlist_load_playlist(xitk_widget_t *w, void *data);
+void playlist_save_playlist(xitk_widget_t *w, void *data);
+
 #endif
