@@ -1,4 +1,6 @@
 /*
+ * Copyright (C) 2000-2003 the xine project
+ *
  * This file is part of xine, a unix video player.
  * 
  * xine is free software; you can redistribute it and/or modify
@@ -15,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: snapshot.c,v 1.17 2002/06/10 23:27:56 f1rmb Exp $
+ * $Id: snapshot.c,v 1.26 2003/04/11 16:24:58 f1rmb Exp $
  *
  * Image snapshot related stuff
  *
@@ -53,10 +55,16 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+
 #ifdef HAVE_ALLOCA_H
 #include <alloca.h>
 #endif
+
 #include <pthread.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+#include <errno.h>
 
 #ifdef HAVE_PNG_H
 #include <png.h>
@@ -64,14 +72,7 @@
 #include <libpng/png.h>
 #endif
 
-#include <xine.h>
-#include <xine/xineutils.h>
-
-#include "event.h"
-#include "errors.h"
-#include "i18n.h"
-
-#include "xitk.h"
+#include "common.h"
 
 #define PIXSZ 3
 #define BIT_DEPTH 8
@@ -79,8 +80,7 @@
 extern gGui_t  *gGui;
 
 /* internal function use to scale yuv data */
-typedef void (*scale_line_func_t) (uint8_t *source, uint8_t *dest,
-                                   int width, int step);
+typedef void (*scale_line_func_t) (uint8_t *source, uint8_t *dest, int width, int step);
 
 /* Holdall structure */
 
@@ -90,6 +90,7 @@ struct prvt_image_s {
   int ratio_code;
   int format;
   uint8_t *y, *u, *v, *yuy2;
+  uint8_t *img;
 
   int u_width, v_width;
   int u_height, v_height;
@@ -105,6 +106,10 @@ struct prvt_image_s {
   png_infop info_ptr;
 };
 
+static snapshot_messenger_t error_msg_cb;
+static snapshot_messenger_t info_msg_cb;
+static void *msg_cb_data;
+
 /*
  *  This function was pinched from filter_yuy2tov12.c, part of
  *  transcode, a linux video stream processing tool
@@ -163,35 +168,6 @@ static void yuy2toyv12( struct prvt_image_s *image )
  *   from where it was pinched.
  */
 
-char* snap_filename(char *base, char *ext)
-{
-    static time_t last = 0;
-    static int count = 0;
-    static char *filename = NULL;
-    
-    time_t now;
-    struct tm* tm;
-    char timestamp[32];
-    
-    time(&now);
-    tm = localtime(&now);
-    
-    if (last != now)
-	count = 0;
-    last = now;
-    count++;
-    
-    if (filename != NULL)
-	free(filename);	
-
-    filename = (char *) xine_xmalloc(strlen(base) + strlen(ext) + 32);
-    
-    strftime(timestamp,31,"%Y%m%d-%H%M%S",tm);
-    sprintf(filename,"%s-%s-%d.%s",
-	    base,timestamp,count,ext);
-    return filename;
-}
-
 /*
  * Scale line with no horizontal scaling. For NTSC mpeg2 dvd input in
  * 4:3 output format (720x480 -> 720x540)
@@ -634,8 +610,6 @@ int scale_image( struct prvt_image_s *image )
 
 static int yuy2_fudge( struct prvt_image_s *image )
 {
-  image->yuy2 = image->y;
-
   image->y = png_malloc( image->struct_ptr, image->height*image->width );
   if ( image->y == NULL ) return( 0 );
   memset( image->y, 0, image->height*image->width );
@@ -696,19 +670,46 @@ static void rgb_free( struct prvt_image_s *image )
  *  Handler functions for image structure
  */
 
-static int prvt_image_alloc( struct prvt_image_s **image )
-{
-  char *filename;
+static char *snap_build_filename(const char *mrl) {
+  static char   buffer[XITK_NAME_MAX + XITK_PATH_MAX + 1];
+  char          basename[XITK_NAME_MAX + 1] = "xine_snapshot";
+  char         *p = strrchr(mrl, '/');
+  struct stat   sstat;
+  int           i;
+
+  if(p && (strlen(p) > 1)) {
+    char *ext;
+    
+    p++;
+    
+    sprintf(basename, "%s", p);
+    
+    if((ext = strrchr(basename, '.')) != NULL)
+      *ext = '\0';
+    
+  }
+
+  for(i = 1;; i++) {
+    sprintf(buffer, "%s/%s-%d.png", gGui->snapshot_location, basename, i);
+    if(((stat(buffer, &sstat)) == -1) && (errno == ENOENT))
+      break;
+  }
   
+  return &buffer[0];
+}
+
+static int prvt_image_alloc( struct prvt_image_s **image, int imgsize )
+{
   *image = (struct prvt_image_s*) xine_xmalloc( sizeof( struct prvt_image_s ) );
   
   if (*image == NULL) 
     return 0;
-  
-  filename = (char *) alloca(strlen(gGui->snapshot_location) + 10);
-  sprintf(filename, "%s/%s", gGui->snapshot_location, "xinesnap");
-  
-  (*image)->file_name = snap_filename( filename, "png" );
+    
+  (*image)->img = malloc( imgsize );
+  if ((*image)->img == NULL) {
+    free(*image);
+    return 0;
+  }
   
   return( 1 );
 }
@@ -721,12 +722,15 @@ static void prvt_image_free( struct prvt_image_s **image )
    * Do not free image_p->file_name
    *   It looks after itself
    */
-
+  if (image_p->img)
+    free(image_p->img);
+   
   rgb_free ( image_p );
 
   if (image_p->info_ptr)   png_destroy_info_struct (  image_p->struct_ptr, &image_p->info_ptr );
   if (image_p->struct_ptr) png_destroy_write_struct( &image_p->struct_ptr, (png_infopp)NULL );
   if (image_p->fp)         fclose( image_p->fp );
+  free(image_p);
 }
 
 static int clip_8_bit( int val )
@@ -813,12 +817,25 @@ static void yv12_2_rgb( struct prvt_image_s *image )
 
 static void user_error_fn(png_structp png_ptr, png_const_charp error_msg)
 {
-  xine_error(_("Error: %s\n"), error_msg);
+
+  if(error_msg_cb) {
+    char uerror[4096]; 
+
+    memset(&uerror, 0, sizeof(uerror));
+    sprintf(uerror, _("Error: %s\n"), error_msg);
+    error_msg_cb(msg_cb_data, uerror);
+  }
 }
 
 static void user_warning_fn(png_structp png_ptr, png_const_charp warning_msg)
 {
-  xine_error(_("Warning: %s\n"), warning_msg);
+  if(error_msg_cb) {
+    char uerror[4096];
+    
+    memset(&uerror, 0, sizeof(uerror));
+    sprintf(uerror, _("Error: %s\n"), warning_msg);
+    error_msg_cb(msg_cb_data, uerror);
+  }
 }
 
 /*
@@ -832,11 +849,13 @@ static void write_row_callback( png_structp png_ptr, png_uint_32 row, int pass)
  *  External function
  */
 
-void create_snapshot (void)
+void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb,
+		      snapshot_messenger_t info_mcb, void *mcb_data)
 {
   int err = 0;
   struct prvt_image_s *image;
-
+  int width, height, ratio_code, format;
+  
 #ifdef DEBUG
   static int	   prof_scale_image = -1;
   static int	   prof_yuv2rgb     = -1;
@@ -850,16 +869,40 @@ void create_snapshot (void)
     prof_png = xine_profiler_allocate_slot ("snapshot convert to png");
 #endif /* DEBUG */
 
-  if ( ! prvt_image_alloc( &image ) )
+  error_msg_cb = error_mcb;
+  info_msg_cb = info_mcb;
+  msg_cb_data = mcb_data;
+
+  err = xine_get_current_frame(gGui->stream, &width, &height, &ratio_code, &format, NULL);
+  
+  if (err == 0) {
+    printf("   xine_get_current_frame() failed\n");
+    return;
+  }
+
+  if((!width) || (!height)) {
+    if(error_msg_cb) {
+      char umessage[4096];
+      
+      memset(&umessage, 0, sizeof(umessage));
+      sprintf(umessage, _("Wrong image size: %dx%d. Snapshot aborted.\n"), width, height);
+      error_msg_cb(msg_cb_data, umessage);
+    }
+    return;
+  }
+  
+  printf("snapshot.c: allocating space for a %d x %d image\n", width, height );
+  
+  if ( ! prvt_image_alloc( &image, width*height*2 ) )
   {
     printf("  prvt_image_alloc failed\n");
     return;
   }
 
-  err = xine_get_current_frame( gGui->xine,
-          &image->width, &image->height, &image->ratio_code,
-          &image->format, &image->y, &image->u, &image->v);
-
+  err = xine_get_current_frame(gGui->stream,
+			       &image->width, &image->height, &image->ratio_code,
+			       &image->format, image->img);
+  
   if (err == 0) {
     printf("   Framegrabber failed\n");
     prvt_image_free( &image );
@@ -867,7 +910,7 @@ void create_snapshot (void)
   }
 
   /* the dxr3 driver does not allocate yuv buffers */
-  if (! image->y) { /* image->u and image->v are always 0 for YUY2 */
+  if (! image->img) { /* image->u and image->v are always 0 for YUY2 */
     printf(_("snapshot.c: not supported for this video out driver\n"));
     prvt_image_free( &image );
     return;
@@ -878,31 +921,31 @@ void create_snapshot (void)
   printf("  ratio:  " );
 
   switch ( image->ratio_code ) {
-    case XINE_ASPECT_RATIO_SQUARE:
+    case XINE_VO_ASPECT_SQUARE:
       printf( "XINE_ASPECT_RATIO_SQUARE\n" ); 
       image->scale_line = scale_line_1_1;
       image->scale_factor = ( 32768 * 1 ) / 1;
       break;
 
-    case XINE_ASPECT_RATIO_4_3:
+    case XINE_VO_ASPECT_4_3:
       printf( "XINE_ASPECT_RATIO_4_3\n" ); 
       image->scale_line = scale_line_15_16;
       image->scale_factor = ( 32768 * 16 ) / 15;
       break;
 
-    case XINE_ASPECT_RATIO_ANAMORPHIC: 
+    case XINE_VO_ASPECT_ANAMORPHIC: 
       printf( "XINE_ASPECT_RATIO_ANAMORPHIC\n" ); 
       image->scale_line = scale_line_45_64;
       image->scale_factor = ( 32768 * 64 ) / 45;
       break;
 
-    case XINE_ASPECT_RATIO_211_1:      
+    case XINE_VO_ASPECT_DVB:      
       printf( "XINE_ASPECT_RATIO_211_1\n" ); 
       image->scale_line = scale_line_45_64;
       image->scale_factor = ( 32768 * 64 ) / 45;
       break;
 
-    case XINE_ASPECT_RATIO_DONT_TOUCH: 
+    case XINE_VO_ASPECT_DONT_TOUCH: 
       printf( "XINE_ASPECT_RATIO_DONT_TOUCH\n" ); 
       image->scale_line = scale_line_1_1;
       image->scale_factor = ( 32768 * 1 ) / 1;
@@ -920,6 +963,9 @@ void create_snapshot (void)
   switch ( image->format ) {
     case XINE_IMGFMT_YV12:
       printf( "XINE_IMGFMT_YV12\n" ); 
+      image->y        = image->img;
+      image->u        = image->img + (image->width*image->height);
+      image->v        = image->img + (image->width*image->height)+(image->width*image->height)/4;
       image->u_width  = ((image->width+1)/2);
       image->v_width  = ((image->width+1)/2);
       image->u_height = ((image->height+1)/2);
@@ -928,6 +974,7 @@ void create_snapshot (void)
 
     case XINE_IMGFMT_YUY2: 
       printf( "XINE_IMGFMT_YUY2\n" );
+      image->yuy2     = image->img;
       image->u_width  = ((image->width+1)/2);
       image->v_width  = ((image->width+1)/2);
       image->u_height = ((image->height+1)/2);
@@ -943,8 +990,17 @@ void create_snapshot (void)
 
   /**/
 
+  image->file_name = snap_build_filename(mrl);
+  
   if ( (image->fp = fopen(image->file_name, "wb")) == NULL ) {
-    xine_error(_("File open failed (%s)\n"), image->file_name);
+    
+    if(error_msg_cb) {
+      char umessage[4096];
+      
+      memset(&umessage, 0, sizeof(umessage));
+      sprintf(umessage, _("File open failed (%s)\n"), image->file_name);
+      error_msg_cb(msg_cb_data, umessage);
+    }
     prvt_image_free( &image );
     return;
   }
@@ -1092,7 +1148,12 @@ void create_snapshot (void)
 #endif
 
   /**/
-  xine_info(_("File '%s' written.\n"), image->file_name);
+  if(info_msg_cb) {
+    char umessage[4096];
+    memset(&umessage, 0, sizeof(umessage));
+    sprintf(umessage, _("File '%s' written.\n"), image->file_name);
+    info_msg_cb(msg_cb_data, umessage);
+  }
 
   printf("  prvt_image_free\n" );
   prvt_image_free( &image );
