@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: snapshot.h,v 1.4 2002/04/27 13:26:41 f1rmb Exp $
+ * $Id: snapshot.h,v 1.7 2003/02/21 16:21:17 f1rmb Exp $
  *
  * Image snapshot related stuff
  *
@@ -25,6 +25,9 @@
 #ifndef SNAPSHOT_H
 #define SNAPSHOT_H
 
-void create_snapshot (void);
+typedef void (*snapshot_messenger_t)(void *data, char *message);
+
+void create_snapshot (const char *mrl, snapshot_messenger_t error_mcb, 
+		      snapshot_messenger_t info_mcb, void *mcb_data);
 
 #endif
