@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: videowin.h,v 1.24 2002/07/13 10:32:32 f1rmb Exp $
+ * $Id: videowin.h,v 1.30 2003/01/05 00:24:34 f1rmb Exp $
  *
  * video window handling functions
  */
@@ -40,19 +40,27 @@ void video_window_exit (void);
 void video_window_select_visual (void);
 
 void video_window_frame_output_cb (void *this,
-				   int video_width, int video_height, 
-				   int *dest_x, int *dest_y,
+				   int video_width, int video_height,
+				   double video_pixel_aspect,
+				   int *dest_x, int *dest_y, 
 				   int *dest_width, int *dest_height,
+				   double *dest_pixel_aspect,
 				   int *win_x, int *win_y);
 
 void video_window_dest_size_cb (void *this,
 				int video_width, int video_height,
-				int *dest_width, int *dest_height);
+				double video_pixel_aspect,
+				int *dest_width, int *dest_height,
+				double *dest_pixel_aspect);
 
 /* set/check fullscreen mode */
 void video_window_set_fullscreen_mode (int req_fullscreen);
 int video_window_get_fullscreen_mode (void);
 
+/* set/check xinerama fullscreen mode */
+void video_window_set_xinerama_fullscreen_mode (int req_fullscreen);
+int video_window_get_xinerama_fullscreen_mode (void);
+
 /* hide/show cursor in video window*/
 void video_window_set_cursor_visibility(int show_cursor);
 /* Get cursor visiblity (boolean) */
@@ -65,8 +73,10 @@ int video_window_is_visible (void);
 void video_window_set_mag (float mag);
 float video_window_get_mag (void);
 
+void video_window_update_logo(void);
 void video_window_change_skins(void);
 
 void video_window_reset_ssaver(void);
 
+void video_window_get_frame_size(int *w, int *h);
 #endif
