@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,13 +17,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: viewlog.h,v 1.5 2002/06/04 20:11:48 f1rmb Exp $
+ * $Id: viewlog.h,v 1.7 2003/01/05 00:24:34 f1rmb Exp $
  *
  */
 
 #ifndef VIEWLOG_H
 #define VIEWLOG_H
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include "xitk.h"
+
 void viewlog_window(void);
 void viewlog_exit(xitk_widget_t *, void *);
 int viewlog_is_visible(void);
