@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: checkbox.c,v 1.23 2002/07/17 20:29:31 f1rmb Exp $
+ * $Id: checkbox.c,v 1.33 2003/03/27 20:52:03 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -26,20 +26,15 @@
 
 #include <stdio.h>
 
-#include "Imlib-light/Imlib.h"
-#include "widget.h"
-#include "image.h"
-#include "checkbox.h"
-#include "widget_types.h"
 #include "_xitk.h"
 
 /*
  *
  */
-static void notify_destroy(xitk_widget_t *w, void *data) {
+static void notify_destroy(xitk_widget_t *w) {
   checkbox_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
     private_data = (checkbox_private_data_t *) w->private_data;
 
     XITK_FREE(private_data->skin_element_name);
@@ -54,7 +49,7 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
 static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
   checkbox_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
     private_data = (checkbox_private_data_t *) w->private_data;
 
     if(sk == FOREGROUND_SKIN && private_data->skin) {
@@ -71,13 +66,14 @@ static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
 static int notify_inside(xitk_widget_t *w, int x, int y) {
   checkbox_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
     private_data = (checkbox_private_data_t *) w->private_data;
 
     if((w->visible == 1)) {
       xitk_image_t *skin = private_data->skin;
       
-      return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, w, skin->mask, x, y);
+      if(skin->mask)
+	return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, w, skin->mask->pixmap, x, y);
     }
     else
       return 0;
@@ -89,52 +85,59 @@ static int notify_inside(xitk_widget_t *w, int x, int y) {
 /*
  *
  */
-static void paint_checkbox (xitk_widget_t *w, Window win, GC gc) {
+static void paint_checkbox (xitk_widget_t *w) {
   checkbox_private_data_t *private_data;
   GC                       lgc;
   int                      checkbox_width;
   xitk_image_t            *skin;
   
-  if(w && (((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX) && (w->visible == 1))) {
+  if(w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX) && (w->visible == 1))) {
     private_data = (checkbox_private_data_t *) w->private_data;
 
     skin           = private_data->skin;
     checkbox_width = skin->width / 3;
     
     XLOCK (private_data->imlibdata->x.disp);
+    lgc = XCreateGC(private_data->imlibdata->x.disp, w->wl->win, None, None);
+    XCopyGC(private_data->imlibdata->x.disp, w->wl->gc, (1 << GCLastBit) - 1, lgc);
+    XUNLOCK (private_data->imlibdata->x.disp);
     
-    lgc = XCreateGC(private_data->imlibdata->x.disp, win, None, None);
-    XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, lgc);
-
     if (skin->mask) {
+      XLOCK (private_data->imlibdata->x.disp);
       XSetClipOrigin(private_data->imlibdata->x.disp, lgc, w->x, w->y);
-      XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask);
+      XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask->pixmap);
+      XUNLOCK (private_data->imlibdata->x.disp);
     }
 
+    XLOCK (private_data->imlibdata->x.disp);
     if ((private_data->focus == FOCUS_RECEIVED) || (private_data->focus == FOCUS_MOUSE_IN)) {
       if (private_data->cClicked) { //click
-	XCopyArea (private_data->imlibdata->x.disp, skin->image, 
-		   win, lgc, 2*checkbox_width, 0,
+	XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap, 
+		   w->wl->win, lgc, 2*checkbox_width, 0,
 		   checkbox_width, skin->height, w->x, w->y);
       }
       else {
 	if(!private_data->cState) //focus
-	  XCopyArea (private_data->imlibdata->x.disp, skin->image, 
-		     win, lgc, checkbox_width, 0,
+	  XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap, 
+		     w->wl->win, lgc, checkbox_width, 0,
 		     checkbox_width, skin->height, w->x, w->y);
       }
-    } else {
-      if(private_data->cState) //click
-	XCopyArea (private_data->imlibdata->x.disp, skin->image, 
-		   win, lgc, 2*checkbox_width, 0,
+    } 
+    else {
+      if(private_data->cState) { //click
+	XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap, 
+		   w->wl->win, lgc, 2*checkbox_width, 0,
 		   checkbox_width, skin->height, w->x, w->y);
-      else  //normal
-	XCopyArea (private_data->imlibdata->x.disp, skin->image, win, lgc, 0, 0,
+      }
+      else { //normal
+	XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap, w->wl->win, lgc, 0, 0,
 		   checkbox_width, skin->height, w->x, w->y);
+      }
     }
+    XUNLOCK (private_data->imlibdata->x.disp);
 
+    XLOCK (private_data->imlibdata->x.disp);
     XFreeGC(private_data->imlibdata->x.disp, lgc);
-
     XUNLOCK(private_data->imlibdata->x.disp);
   }
 
@@ -143,38 +146,40 @@ static void paint_checkbox (xitk_widget_t *w, Window win, GC gc) {
 /*
  *
  */
-static int notify_click_checkbox (xitk_widget_list_t *wl, 
-				  xitk_widget_t *w, int cUp, int x, int y) {
+static int notify_click_checkbox (xitk_widget_t *w, int button, int cUp, int x, int y) {
   checkbox_private_data_t *private_data;
+  int                      ret = 0;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
-    
-    private_data = (checkbox_private_data_t *) w->private_data;
-    
-    private_data->cClicked = !cUp;
-    if (cUp && (private_data->focus == FOCUS_RECEIVED)) {
-      private_data->cState = !private_data->cState;
-
-      if(private_data->callback) {
-	private_data->callback(private_data->cWidget, 
-			       private_data->userdata,
-			       private_data->cState);
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
+    if(button == Button1) {
+      private_data = (checkbox_private_data_t *) w->private_data;
+      
+      private_data->cClicked = !cUp;
+      if (cUp && (private_data->focus == FOCUS_RECEIVED)) {
+	private_data->cState = !private_data->cState;
+	
+	if(private_data->callback) {
+	  private_data->callback(private_data->cWidget, 
+				 private_data->userdata,
+				 private_data->cState);
+	}
       }
+      
+      paint_checkbox(w);
+      ret = 1;
     }
-    
-    paint_checkbox(w, wl->win, wl->gc);
   }
 
-  return 1;
+  return ret;
 }
 
 /*
  *
  */
-static int notify_focus_checkbox (xitk_widget_list_t *wl, xitk_widget_t *w, int focus) {
+static int notify_focus_checkbox (xitk_widget_t *w, int focus) {
   checkbox_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
     private_data = (checkbox_private_data_t *) w->private_data;
     private_data->focus = focus;
   }
@@ -184,16 +189,15 @@ static int notify_focus_checkbox (xitk_widget_list_t *wl, xitk_widget_t *w, int
 /*
  *
  */
-static void notify_change_skin(xitk_widget_list_t *wl, 
-			       xitk_widget_t *w, xitk_skin_config_t *skonfig) {
+static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
   checkbox_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
     private_data = (checkbox_private_data_t *) w->private_data;
     
     if(private_data->skin_element_name) {
       xitk_skin_lock(skonfig);
-      XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->skin);
+      xitk_image_free_image(private_data->imlibdata, &private_data->skin);
       private_data->skin = xitk_image_load_image(private_data->imlibdata,
 						 xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
       w->x               = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
@@ -210,13 +214,63 @@ static void notify_change_skin(xitk_widget_list_t *wl,
   }
 }
 
+static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_result_t *result) {
+  int retval = 0;
+  
+  switch(event->type) {
+  case WIDGET_EVENT_PAINT:
+    paint_checkbox(w);
+    break;
+  case WIDGET_EVENT_CLICK:
+    result->value = notify_click_checkbox(w, event->button,
+					  event->button_pressed, event->x, event->y);
+    retval = 1;
+    break;
+  case WIDGET_EVENT_FOCUS:
+    notify_focus_checkbox(w, event->focus);
+    break;
+  case WIDGET_EVENT_INSIDE:
+    result->value = notify_inside(w, event->x, event->y);
+    retval = 1;
+    break;
+  case WIDGET_EVENT_CHANGE_SKIN:
+    notify_change_skin(w, event->skonfig);
+    break;
+  case WIDGET_EVENT_DESTROY:
+    notify_destroy(w);
+    break;
+  case WIDGET_EVENT_GET_SKIN:
+    if(result) {
+      result->image = get_skin(w, event->skin_layer);
+      retval = 1;
+    }
+    break;
+  }
+  
+  return retval;
+}
+
+void xitk_checkbox_callback_exec(xitk_widget_t *w) {
+  checkbox_private_data_t *private_data;
+
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
+    private_data = (checkbox_private_data_t *) w->private_data;
+
+    if(private_data->callback) {
+      private_data->callback(private_data->cWidget, 
+			     private_data->userdata,
+			     private_data->cState);
+    }
+  }
+}
+
 /*
  *
  */
 int xitk_checkbox_get_state(xitk_widget_t *w) {
   checkbox_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
     private_data = (checkbox_private_data_t *) w->private_data;
     return private_data->cState;
   }
@@ -227,11 +281,11 @@ int xitk_checkbox_get_state(xitk_widget_t *w) {
 /*
  *
  */
-void xitk_checkbox_set_state(xitk_widget_t *w, int state, Window win, GC gc) {
+void xitk_checkbox_set_state(xitk_widget_t *w, int state) {
   checkbox_private_data_t *private_data;
   int                      clk, focus;
 
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
     private_data = (checkbox_private_data_t *) w->private_data;
 
     if(xitk_checkbox_get_state(w) != state) {
@@ -243,12 +297,12 @@ void xitk_checkbox_set_state(xitk_widget_t *w, int state, Window win, GC gc) {
       private_data->cClicked = 1;
       private_data->cState = state;
 
-      paint_checkbox(w, win, gc);
+      paint_checkbox(w);
 
       private_data->focus = focus;
       private_data->cClicked = clk;
 
-      paint_checkbox(w, win, gc);
+      paint_checkbox(w);
     }
   }
 
@@ -284,7 +338,7 @@ static xitk_widget_t *_xitk_checkbox_create(xitk_widget_list_t *wl,
 
   mywidget->private_data          = private_data;
 
-  mywidget->widget_list           = wl;
+  mywidget->wl                    = wl;
 
   mywidget->enable                = enable;
   mywidget->running               = 1;
@@ -295,16 +349,8 @@ static xitk_widget_t *_xitk_checkbox_create(xitk_widget_list_t *wl,
   mywidget->y                     = y;
   mywidget->width                 = private_data->skin->width/3;
   mywidget->height                = private_data->skin->height;
-  mywidget->widget_type           = WIDGET_TYPE_CHECKBOX;
-  mywidget->paint                 = paint_checkbox;
-  mywidget->notify_click          = notify_click_checkbox;
-  mywidget->notify_focus          = notify_focus_checkbox;
-  mywidget->notify_keyevent       = NULL;
-  mywidget->notify_inside         = notify_inside;
-  mywidget->notify_change_skin    = (skin_element_name == NULL) ? NULL : notify_change_skin;
-  mywidget->notify_destroy        = notify_destroy;
-  mywidget->get_skin              = get_skin;
-
+  mywidget->type                  = WIDGET_TYPE_CHECKBOX | WIDGET_CLICKABLE | WIDGET_FOCUSABLE | WIDGET_KEYABLE;
+  mywidget->event                 = notify_event;
   mywidget->tips_timeout          = 0;
   mywidget->tips_string           = NULL;
   
