@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,15 +17,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: checkbox.h,v 1.8 2002/07/17 20:29:31 f1rmb Exp $
+ * $Id: checkbox.h,v 1.11 2003/01/05 00:24:35 f1rmb Exp $
  *
  */
 #ifndef HAVE_XITK_CHECKBOX_H
 #define HAVE_XITK_CHECKBOX_H
 
-#include <X11/Xlib.h>
-#include "Imlib-light/Imlib.h"
-#include "widget.h"
 #include "_xitk.h"
 
 typedef struct {
@@ -43,29 +40,4 @@ typedef struct {
 
 } checkbox_private_data_t;
 
-/* ****************************************************************** */
-
-/**
- * Create a checkbox.
- */
-xitk_widget_t *xitk_checkbox_create (xitk_widget_list_t *wl,
-				     xitk_skin_config_t *skonfig, xitk_checkbox_widget_t *cb);
-
-/*
- * Same as above, without skinable feature.
- */
-xitk_widget_t *xitk_noskin_checkbox_create(xitk_widget_list_t *wl,
-					   xitk_checkbox_widget_t *cb,
-					   int x, int y, int width, int height);
-
-/**
- * get state of checkbox "widget".
- */
-int xitk_checkbox_get_state(xitk_widget_t *);
-
-/**
- * Set state of checkbox .
- */
-void xitk_checkbox_set_state(xitk_widget_t *, int, Window, GC);
-
 #endif
