@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: combo.c,v 1.22 2002/07/17 20:29:31 f1rmb Exp $
+ * $Id: combo.c,v 1.33 2003/01/05 00:24:35 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -28,15 +28,6 @@
 #include <X11/Xutil.h>
 #include <stdio.h>
 
-#include "combo.h"
-#include "browser.h"
-#include "image.h"
-#include "label.h"
-#include "button.h"
-#include "labelbutton.h"
-#include "checkbox.h"
-#include "font.h"
-#include "window.h"
 #include "_xitk.h"
 
 static void _combo_rollunroll(xitk_widget_t *w, void *data, int state);
@@ -44,16 +35,43 @@ static void _combo_rollunroll(xitk_widget_t *w, void *data, int state);
 /*
  *
  */
-static void notify_destroy(xitk_widget_t *w, void *data) {
+static void enability(xitk_widget_t *w) {
   combo_private_data_t *private_data;
   
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
+
+    private_data = (combo_private_data_t *) w->private_data;
+
+    if(w->enable == WIDGET_ENABLE) {
+      xitk_enable_widget(private_data->label_widget);
+      xitk_enable_widget(private_data->button_widget);
+    }
+    else {
+
+      if(private_data->visible) {
+	xitk_checkbox_set_state(private_data->button_widget, 0);
+	_combo_rollunroll(private_data->button_widget, (void *)w, 0);
+      }
+
+
+      xitk_disable_widget(private_data->label_widget);
+      xitk_disable_widget(private_data->button_widget);
+    }
+
+  }
+}
+
+static void notify_destroy(xitk_widget_t *w) {
+  combo_private_data_t *private_data;
+  
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
 
     private_data = (combo_private_data_t *) w->private_data;
     
     if(private_data->visible)
-      _combo_rollunroll(NULL, (void *)w, 0);
+      _combo_rollunroll(private_data->button_widget, (void *)w, 0);
 
     xitk_destroy_widgets(private_data->widget_list);
     xitk_list_free(private_data->widget_list->l);
@@ -75,18 +93,17 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
 /*
  *
  */
-static void paint(xitk_widget_t *w, Window win, GC gc) {
+static void paint(xitk_widget_t *w) {
   combo_private_data_t *private_data;
 
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
 
     private_data = (combo_private_data_t *) w->private_data;
 
     if(private_data->visible == 1 && (w->visible < 1)) {
-      xitk_checkbox_set_state(private_data->button_widget, 0,
-			      private_data->parent_wlist->win, private_data->parent_wlist->gc);
-      _combo_rollunroll(NULL, (void *)w, 0);
+      xitk_checkbox_set_state(private_data->button_widget, 0);
+      _combo_rollunroll(private_data->button_widget, (void *)w, 0);
     }
     if((w->visible == 1)) {
       int bx, lw;
@@ -96,12 +113,12 @@ static void paint(xitk_widget_t *w, Window win, GC gc) {
       bx = w->x + lw;
       xitk_set_widget_pos(private_data->button_widget, bx, w->y);
 
-      xitk_show_widget(private_data->parent_wlist, private_data->label_widget);
-      xitk_show_widget(private_data->parent_wlist, private_data->button_widget);
+      xitk_show_widget(private_data->label_widget);
+      xitk_show_widget(private_data->button_widget);
     }
     else {
-      xitk_hide_widget(private_data->parent_wlist, private_data->label_widget);
-      xitk_hide_widget(private_data->parent_wlist, private_data->button_widget);
+      xitk_hide_widget(private_data->label_widget);
+      xitk_hide_widget(private_data->button_widget);
     }
   }
 }
@@ -109,20 +126,17 @@ static void paint(xitk_widget_t *w, Window win, GC gc) {
 /*
  * Called on select action.
  */
-static void combo_select(xitk_widget_t *w, void *data) {
+static void combo_select(xitk_widget_t *w, void *data, int selected) {
   combo_private_data_t *private_data;
   xitk_widget_t        *c;
-  int                   selected;
 
-  if(w && ((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER)) {
+  if(w && ((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER)) {
     c = (xitk_widget_t *) ((browser_private_data_t *)w->private_data)->userdata;
     private_data = (combo_private_data_t *)c->private_data;
-    selected = (int)data;
     
     private_data->selected = selected;
     
-    xitk_label_change_label(private_data->parent_wlist, 
-			    private_data->label_widget, private_data->entries[selected]);
+    xitk_label_change_label(private_data->label_widget, private_data->entries[selected]);
     
     XLOCK(private_data->imlibdata->x.disp);
     XUnmapWindow(private_data->imlibdata->x.disp, (xitk_window_get_window(private_data->xwin)));
@@ -132,10 +146,10 @@ static void combo_select(xitk_widget_t *w, void *data) {
     
     xitk_browser_release_all_buttons(private_data->browser_widget);
     xitk_browser_update_list(private_data->browser_widget, 
-			     private_data->entries, private_data->num_entries, 0);
+			     (const char* const*)private_data->entries, 
+			     private_data->num_entries, 0);
     
-    xitk_checkbox_set_state(private_data->button_widget, 0, 
-			    private_data->widget_list->win, private_data->widget_list->gc);
+    xitk_checkbox_set_state(private_data->button_widget, 0);
 
     if(private_data->callback)
       private_data->callback(private_data->combo_widget, private_data->userdata, selected);
@@ -150,32 +164,31 @@ static void _combo_handle_event(XEvent *event, void *data) {
   combo_private_data_t *private_data = (combo_private_data_t *)data;
 
   switch(event->type) {
-
+    
   case ButtonRelease:
     /*
      * If we try to move the combo window,
      * move it back to right position (under label*
      */
     if(private_data->visible) { 
-      if(!xitk_is_inside_widget(private_data->browser_widget, 
-				event->xbutton.x, event->xbutton.y)) {
+      int  x, y;
+      xitk_window_get_window_position(private_data->combo_widget->imlibdata, 
+				      private_data->xwin, &x, &y, NULL, NULL);
+      if((x != private_data->win_x) || (y != private_data->win_y))
 	xitk_combo_update_pos(private_data->combo_widget);
-      }
     }
     break;
-  }
-    
+  }    
 }
 
 /*
  *
  */
-static void notify_change_skin(xitk_widget_list_t *wl, 
-			       xitk_widget_t *w, xitk_skin_config_t *skonfig) {
+static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
   combo_private_data_t *private_data;
   
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
     private_data = (combo_private_data_t *) w->private_data;
 
     if(private_data->skin_element_name) {
@@ -201,6 +214,27 @@ static void notify_change_skin(xitk_widget_list_t *wl,
   }
 }
 
+static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_result_t *result) {
+  int retval = 0;
+  
+  switch(event->type) {
+  case WIDGET_EVENT_PAINT:
+    paint(w);
+    break;
+  case WIDGET_EVENT_CHANGE_SKIN:
+    notify_change_skin(w, event->skonfig);
+    break;
+  case WIDGET_EVENT_DESTROY:
+    notify_destroy(w);
+    break;
+  case WIDGET_EVENT_ENABLE:
+    enability(w);
+    break;
+  }
+  
+  return retval;
+}
+
 /*
  *
  */
@@ -208,8 +242,8 @@ static void _combo_rollunroll(xitk_widget_t *w, void *data, int state) {
   xitk_widget_t        *combo = (xitk_widget_t *)data;
   combo_private_data_t *private_data = (combo_private_data_t *)combo->private_data;
   
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
-	   ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX))) {
     
     if(state && private_data->visible == 0) {
       private_data->visible = 1;
@@ -232,16 +266,16 @@ static void _combo_rollunroll_from_lbl(xitk_widget_t *w, void *data) {
   xitk_widget_t        *combo = (xitk_widget_t *)data;
   combo_private_data_t *private_data = (combo_private_data_t *)combo->private_data;
   
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
-	   ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL))) {
     int state;
     
     state = !xitk_checkbox_get_state(private_data->button_widget);
     
-    xitk_checkbox_set_state(private_data->button_widget, state, 
-			    private_data->parent_wlist->win, private_data->parent_wlist->gc);
-    
+    xitk_checkbox_set_state(private_data->button_widget, state);
+
     if(state && private_data->visible == 0) {
+      w->wl->widget_focused = private_data->button_widget;
       private_data->visible = 1;
       xitk_combo_update_pos(combo);
     }
@@ -252,7 +286,7 @@ static void _combo_rollunroll_from_lbl(xitk_widget_t *w, void *data) {
       XUnmapWindow(private_data->imlibdata->x.disp, (xitk_window_get_window(private_data->xwin)));
       XUNLOCK(private_data->imlibdata->x.disp);
     }
-
+    
   }      
 }
 
@@ -260,20 +294,86 @@ static void _combo_rollunroll_from_lbl(xitk_widget_t *w, void *data) {
  * ************************* END OF PRIVATES *******************************
  */
 
+int xitk_combo_is_same_parent(xitk_widget_t *w1, xitk_widget_t *w2) {
+
+  if((w1 && w2) && ((w1->type & WIDGET_GROUP_COMBO) && (w1->type & WIDGET_GROUP_COMBO))) {
+    
+    if(w1 == w2)
+      return 1;
+    
+    if(w1->wl == w2->wl) {
+      xitk_widget_list_t  *wl = w1->wl;
+      xitk_widget_t       *w, *wt;
+      
+      w = (xitk_widget_t *) xitk_list_first_content(wl->l);
+      while(w && ((w != w1) && (w != w2))) {
+	w = (xitk_widget_t *) xitk_list_next_content(wl->l);
+      }
+
+      if(w) {
+
+	wt = (w == w1) ? w2 : w1;
+	
+	if((w->type & WIDGET_GROUP_COMBO) && 
+	   ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
+	  w = (xitk_widget_t *) xitk_list_next_content(wl->l);
+	  w = (xitk_widget_t *) xitk_list_next_content(wl->l);
+	}
+	else if((w->type & WIDGET_GROUP_COMBO) && 
+		((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) {
+	  w = (xitk_widget_t *) xitk_list_next_content(wl->l);
+	}
+	
+	if(((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET)) {
+	  combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
+	
+	  if((wt == private_data->label_widget) || (wt == private_data->button_widget))
+	    return 1;
+	  
+	}
+
+      }
+    }
+  }
+  return 0;  
+}
+
 /*
  *
  */
-void xitk_combo_set_select(xitk_widget_list_t *wl, xitk_widget_t *w, int select) {
+void xitk_combo_rollunroll(xitk_widget_t *w) {
+  
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO))) {
+
+    if((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX) {
+      int state = !xitk_checkbox_get_state(w);
+      xitk_checkbox_callback_exec(w);
+      xitk_checkbox_set_state(w, state);
+    }
+  }
+  else if(w->type & WIDGET_GROUP_WIDGET) {
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
+    int state = !xitk_checkbox_get_state(private_data->button_widget);
+      xitk_checkbox_callback_exec(private_data->button_widget);
+      xitk_checkbox_set_state(private_data->button_widget, state);
+  }
+}
+
+  /*
+   *
+   */
+void xitk_combo_set_select(xitk_widget_t *w, int select) {
   combo_private_data_t *private_data;
 
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
     
     private_data = (combo_private_data_t *)w->private_data;
     
     if(private_data->entries && private_data->entries[select]) {
       private_data->selected = select;
-      xitk_label_change_label(wl, private_data->label_widget, private_data->entries[select]);
+      xitk_label_change_label(private_data->label_widget, private_data->entries[select]);
     }
 
   }
@@ -284,30 +384,39 @@ void xitk_combo_set_select(xitk_widget_list_t *wl, xitk_widget_t *w, int select)
  */
 void xitk_combo_update_pos(xitk_widget_t *w) {
   combo_private_data_t  *private_data;
-  int                    x = 0, xx = 0, y = 0, yy = 0;
+  int                    xx = 0, yy = 0;
   window_info_t          wi;
-  
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  XSizeHints             hint;
+
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
     
     private_data = (combo_private_data_t *)w->private_data;
     
     if(private_data->visible) {
       if((xitk_get_window_info(*(private_data->parent_wkey), &wi))) {
-	x = wi.x;
-	y = wi.y;
+	private_data->win_x = wi.x;
+	private_data->win_y = wi.y;
 	WINDOW_INFO_ZERO(&wi);
       }
       
       xitk_get_widget_pos(private_data->label_widget, &xx, &yy);
       
       yy += xitk_get_widget_height(private_data->label_widget);
-      x += xx;
-      y += yy;
+      private_data->win_x += xx;
+      private_data->win_y += yy;
       
+      hint.x = private_data->win_x;
+      hint.y = private_data->win_y;
+      hint.flags = PPosition;
+
       XLOCK(private_data->imlibdata->x.disp);
+      XSetWMNormalHints (private_data->imlibdata->x.disp,
+			 xitk_window_get_window(private_data->xwin),
+			 &hint);
       XMoveWindow(private_data->imlibdata->x.disp, 
-		  (xitk_window_get_window(private_data->xwin)), x, y);
+		  (xitk_window_get_window(private_data->xwin)), 
+		  private_data->win_x, private_data->win_y);
       XMapRaised(private_data->imlibdata->x.disp, (xitk_window_get_window(private_data->xwin)));
       XSync(private_data->imlibdata->x.disp, False);
       XUNLOCK(private_data->imlibdata->x.disp);
@@ -316,8 +425,10 @@ void xitk_combo_update_pos(xitk_widget_t *w) {
 				    (xitk_window_get_window(private_data->xwin))))
 	xitk_usec_sleep(5000);
       
+      XLOCK(private_data->imlibdata->x.disp);
       XSetInputFocus(private_data->imlibdata->x.disp, 
 		     (xitk_window_get_window(private_data->xwin)), RevertToParent, CurrentTime);
+      XUNLOCK(private_data->imlibdata->x.disp);
       
       /* No widget focused, give focus to the first one */
       if(private_data->widget_list->widget_focused == NULL)
@@ -333,8 +444,8 @@ void xitk_combo_update_pos(xitk_widget_t *w) {
 int xitk_combo_get_current_selected(xitk_widget_t *w) {
   combo_private_data_t *private_data;
   
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
     private_data = (combo_private_data_t *) w->private_data;
     return private_data->selected;
   }
@@ -348,8 +459,8 @@ int xitk_combo_get_current_selected(xitk_widget_t *w) {
 char *xitk_combo_get_current_entry_selected(xitk_widget_t *w) {
   combo_private_data_t *private_data;
   
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
     private_data = (combo_private_data_t *) w->private_data;
 
     if(private_data->entries && private_data->selected >= 0)
@@ -365,8 +476,8 @@ char *xitk_combo_get_current_entry_selected(xitk_widget_t *w) {
 void xitk_combo_update_list(xitk_widget_t *w, char **list, int len) {
   combo_private_data_t *private_data;
   
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
  
     private_data              = (combo_private_data_t *) w->private_data;
     private_data->entries     = list;
@@ -374,7 +485,8 @@ void xitk_combo_update_list(xitk_widget_t *w, char **list, int len) {
     private_data->selected    = -1;
     
     xitk_browser_update_list(private_data->browser_widget, 
-			     private_data->entries, private_data->num_entries, 0);
+			     (const char* const*)private_data->entries, 
+			     private_data->num_entries, 0);
   }
 }
 
@@ -422,12 +534,12 @@ static xitk_widget_t *_xitk_combo_create(xitk_widget_list_t *wl,
   }
   
   if(private_data->num_entries) {
-    xitk_label_change_label(c->parent_wlist, private_data->label_widget, entries[0]);
+    xitk_label_change_label(private_data->label_widget, entries[0]);
     private_data->selected = 0;
   }
   
   private_data->xwin = xitk_window_create_simple_window(c->imlibdata, 0, 0,
-							(itemw + 2), (itemh * 5) + 2);
+							(itemw + 2), (itemh * 5) + slidw + 2);
   XLOCK(c->imlibdata->x.disp);
 
   if(c->layer_above) {
@@ -439,7 +551,7 @@ static xitk_widget_t *_xitk_combo_create(xitk_widget_list_t *wl,
 		    XA_CARDINAL, 32, PropModeReplace, (unsigned char *)data,
 		    1);
   }
-  
+
   XSetTransientForHint (c->imlibdata->x.disp,
 			(xitk_window_get_window(private_data->xwin)), private_data->parent_wlist->win);
   
@@ -471,7 +583,7 @@ static xitk_widget_t *_xitk_combo_create(xitk_widget_list_t *wl,
   browser.browser.skin_element_name     = NULL;
   browser.browser.max_displayed_entries = 5;
   browser.browser.num_entries           = private_data->num_entries;
-  browser.browser.entries               = private_data->entries;
+  browser.browser.entries               = (const char* const*)private_data->entries;
   browser.callback                      = combo_select;
   browser.dbl_click_callback            = NULL;
   browser.parent_wlist                  = private_data->widget_list;
@@ -482,10 +594,11 @@ static xitk_widget_t *_xitk_combo_create(xitk_widget_list_t *wl,
 							private_data->gc, 1, 1, 
 							(itemw - slidw), itemh, slidw,
 							DEFAULT_FONT_10)));
-  private_data->browser_widget->widget_type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
+  private_data->browser_widget->type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
   
   xitk_browser_update_list(private_data->browser_widget, 
-			   private_data->entries, private_data->num_entries, 0);
+			   (const char* const*)private_data->entries, 
+			   private_data->num_entries, 0);
   
   private_data->widget_key = 
     xitk_register_event_handler("xitk combo",
@@ -496,32 +609,21 @@ static xitk_widget_t *_xitk_combo_create(xitk_widget_list_t *wl,
 				private_data->widget_list,
 				(void *) private_data);
 
-  private_data->visible                  = 0;
+  private_data->visible  = 0;
 
-  mywidget->private_data                 = private_data;
-  mywidget->widget_list                  = wl;
+  mywidget->private_data = private_data;
+  mywidget->wl           = wl;
+  mywidget->enable       = enable;
+  mywidget->running      = 1;
+  mywidget->visible      = visible;
+  mywidget->have_focus   = FOCUS_LOST;
 
-  mywidget->enable                       = enable;
-  mywidget->running                      = 1;
-  mywidget->visible                      = visible;
-  mywidget->have_focus                   = FOCUS_LOST;
-
-
-  mywidget->imlibdata                    = private_data->imlibdata;
+  mywidget->imlibdata    = private_data->imlibdata;
   //  mywidget->x = mywidget->y = mywidget->width = mywidget->height = 0;
-  
-  mywidget->widget_type                  = WIDGET_GROUP | WIDGET_GROUP_WIDGET | WIDGET_GROUP_COMBO;
-  mywidget->paint                        = paint;
-  mywidget->notify_click                 = NULL;
-  mywidget->notify_focus                 = NULL;
-  mywidget->notify_keyevent              = NULL;
-  mywidget->notify_inside                = NULL;
-  mywidget->notify_change_skin           = (skin_element_name == NULL) ? NULL : notify_change_skin;
-  mywidget->notify_destroy               = notify_destroy;
-  mywidget->get_skin                     = NULL;
-
-  mywidget->tips_timeout                 = 0;
-  mywidget->tips_string                  = NULL;
+  mywidget->type         = WIDGET_GROUP | WIDGET_GROUP_WIDGET | WIDGET_GROUP_COMBO;
+  mywidget->event        = notify_event;
+  mywidget->tips_timeout = 0;
+  mywidget->tips_string  = NULL;
 
   return mywidget;
 }
@@ -556,7 +658,7 @@ xitk_widget_t *xitk_combo_create(xitk_widget_list_t *wl,
   xitk_list_append_content(c->parent_wlist->l,
 			   (private_data->label_widget = 
 			    xitk_label_create(c->parent_wlist, skonfig, &lbl)));
-  private_data->label_widget->widget_type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
+  private_data->label_widget->type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
 
   cb.skin_element_name = c->skin_element_name;
   cb.callback          = _combo_rollunroll;
@@ -564,7 +666,7 @@ xitk_widget_t *xitk_combo_create(xitk_widget_list_t *wl,
   xitk_list_append_content(c->parent_wlist->l, 
 			   (private_data->button_widget = 
 			    xitk_checkbox_create(c->parent_wlist, skonfig, &cb)));
-  private_data->button_widget->widget_type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
+  private_data->button_widget->type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
 
   if(lw)
     *lw = private_data->label_widget;
@@ -632,7 +734,7 @@ xitk_widget_t *xitk_noskin_combo_create(xitk_widget_list_t *wl,
 			     (private_data->label_widget = 
 			      xitk_noskin_label_create(c->parent_wlist, &lbl,
 						       x, y, (width - height), (height + 4), DEFAULT_FONT_10)));
-    private_data->label_widget->widget_type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
+    private_data->label_widget->type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
 
     cb.skin_element_name = NULL;
     cb.callback          = _combo_rollunroll;
@@ -643,7 +745,7 @@ xitk_widget_t *xitk_noskin_combo_create(xitk_widget_list_t *wl,
 			      xitk_noskin_checkbox_create(c->parent_wlist, &cb,
 							  x + (width - height), y,
 							  (height + 4), (height + 4))));
-    private_data->button_widget->widget_type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
+    private_data->button_widget->type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
   
     if(lw)
       *lw = private_data->label_widget;
