@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: dnd.c,v 1.18 2002/06/04 15:35:50 f1rmb Exp $
+ * $Id: dnd.c,v 1.21 2003/01/05 00:24:35 f1rmb Exp $
  *
  *
  * Thanks to Paul Sheer for his nice xdnd implementation in cooledit.
@@ -32,11 +32,8 @@
 #include <string.h>
 #include <signal.h>
 
-#include "widget.h"
-#include "dnd.h"
 #include "_xitk.h"
 
-
 #define XDND_VERSION 3
 
 #undef DEBUG_DND
@@ -338,7 +335,7 @@ int xitk_make_window_dnd_aware(xitk_dnd_t *xdnd, Window window) {
   
   if((status == BadAlloc) || (status == BadAtom) || 
      (status == BadMatch) || (status == BadValue) || (status == BadWindow)) {
-    XITK_WARNING("%s()@%d: XChangeProperty() failed.\n", __FUNCTION__, __LINE__);
+    XITK_WARNING("XChangeProperty() failed.\n");
     return 0;
   }
   
@@ -349,7 +346,7 @@ int xitk_make_window_dnd_aware(xitk_dnd_t *xdnd, Window window) {
   
   if((status == BadAlloc) || (status == BadAtom) || 
      (status == BadMatch) || (status == BadValue) || (status == BadWindow)) {
-    XITK_WARNING("%s()@%d: XChangeProperty() failed.\n", __FUNCTION__, __LINE__);
+    XITK_WARNING("XChangeProperty() failed.\n");
     return 0;
   }
 
