@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,20 +17,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: filebrowser.h,v 1.14 2002/07/17 20:29:31 f1rmb Exp $
+ * $Id: filebrowser.h,v 1.17 2003/01/05 00:24:35 f1rmb Exp $
  *
  */
 
 #ifndef HAVE_XITK_FILEBROWSER_H
 #define HAVE_XITK_FILEBROWSER_H
 
-#include <limits.h>
-#include <X11/Xlib.h>
-
-#include "Imlib-light/Imlib.h"
-#include "widget.h"
-#include "dnd.h"
-#include "browser.h"
 #include "_xitk.h"
 
 #define MAXFILES      65535
@@ -85,16 +78,4 @@ typedef struct {
 
 } filebrowser_private_data_t;
 
-xitk_widget_t *xitk_filebrowser_create(xitk_widget_list_t *wl,
-				       xitk_skin_config_t *skonfig, xitk_filebrowser_widget_t *fb);
-
-int xitk_filebrowser_is_running(xitk_widget_t *w);
-int xitk_filebrowser_is_visible(xitk_widget_t *w);
-void xitk_filebrowser_hide(xitk_widget_t *w);
-void xitk_filebrowser_show(xitk_widget_t *w);
-void xitk_filebrowser_set_transient(xitk_widget_t *w, Window window);
-void xitk_filebrowser_destroy(xitk_widget_t *w);
-char *xitk_filebrowser_get_current_dir(xitk_widget_t *w);
-int xitk_filebrowser_get_window_info(xitk_widget_t *w, window_info_t *inf);
-
 #endif
