@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: inputtext.c,v 1.35 2002/07/31 00:50:41 f1rmb Exp $
+ * $Id: inputtext.c,v 1.52 2003/03/27 20:52:03 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -31,13 +31,7 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
-
-#include "Imlib-light/Imlib.h"
-#include "widget.h"
-#include "image.h"
-#include "inputtext.h"
-#include "font.h"
-#include "widget_types.h"
+#include <X11/cursorfont.h>
 
 #include "_xitk.h"
 
@@ -46,15 +40,39 @@
 
 unsigned int mods_used = (ShiftMask | ControlMask | Mod1Mask |
 			  Mod2Mask| Mod3Mask| Mod4Mask| Mod5Mask);
+
 /*
  *
  */
-static void notify_destroy(xitk_widget_t *w, void *data) {
+static void _cursor_focus(inputtext_private_data_t *private_data, Window win, int focus) {
+
+  private_data->cursor_focus = focus;
+  
+  XLOCK(private_data->imlibdata->x.disp);
+
+  if(focus)
+    XDefineCursor(private_data->imlibdata->x.disp, win, private_data->cursor[1]);
+  else
+    XDefineCursor(private_data->imlibdata->x.disp, win, private_data->cursor[0]);
+  
+  XSync(private_data->imlibdata->x.disp, False);
+  XUNLOCK(private_data->imlibdata->x.disp);
+}
+
+/*
+ *
+ */
+static void notify_destroy(xitk_widget_t *w) {
   inputtext_private_data_t *private_data;
 
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     private_data = (inputtext_private_data_t *) w->private_data;
-    
+  
+    XLOCK(private_data->imlibdata->x.disp);
+    XFreeCursor(private_data->imlibdata->x.disp, private_data->cursor[0]);
+    XFreeCursor(private_data->imlibdata->x.disp, private_data->cursor[1]);
+    XUNLOCK(private_data->imlibdata->x.disp);
+
     XITK_FREE(private_data->skin_element_name);
     xitk_image_free_image(private_data->imlibdata, &private_data->skin);
     XITK_FREE(private_data->text);
@@ -71,7 +89,7 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
 static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     private_data = (inputtext_private_data_t *) w->private_data;
     if(sk == BACKGROUND_SKIN && private_data->skin) {
       return private_data->skin;
@@ -87,12 +105,13 @@ static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
 static int notify_inside(xitk_widget_t *w, int x, int y) {
   inputtext_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     private_data = (inputtext_private_data_t *) w->private_data;
     if((w->visible == 1)) {
       xitk_image_t *skin = private_data->skin;
       
-      return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, w, skin->mask, x, y);
+      if(skin->mask)
+	return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, w, skin->mask->pixmap, x, y);
     }
     else
       return 0;
@@ -116,6 +135,11 @@ int xitk_get_key_modifier(XEvent *xev, int *modifier) {
     state = xev->xbutton.state;
   else if ((xev->type == KeyPress) || (xev->type == KeyRelease))
     state = xev->xkey.state;
+  else
+    state = xev->xkey.state;
+
+  if(state & XK_Multi_key)
+    state = (state | XK_Multi_key) & 0xFF;
   
   if(state & ShiftMask)
     *modifier |= MODIFIER_SHIFT;
@@ -140,8 +164,8 @@ int xitk_get_key_modifier(XEvent *xev, int *modifier) {
 /*
  * Recalculate display offsets.
  */
-static void inputtext_recalc_offsets(xitk_widget_list_t *wl, 
-				     inputtext_private_data_t *private_data) {
+static void inputtext_recalc_offsets(xitk_widget_t *w) {
+  inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   char         *label = private_data->text;
   char         *p = private_data->text;
   xitk_font_t  *fs = NULL;
@@ -159,7 +183,7 @@ static void inputtext_recalc_offsets(xitk_widget_list_t *wl,
     if(fs == NULL)
       XITK_DIE("%s()@%d: xitk_font_load_font() failed. Exiting\n", __FUNCTION__, __LINE__);
 
-    xitk_font_set_font(fs, wl->gc);
+    xitk_font_set_font(fs, w->wl->gc);
 
     private_data->disp_offset = private_data->pos_in_pos = 0;
     
@@ -296,39 +320,45 @@ static void create_labelofinputtext(xitk_widget_t *w,
     plabel += private_data->disp_offset;
 
   /*  Put text in the right place */
-  XLOCK(private_data->imlibdata->x.disp);
   if(private_data->skin_element_name) {
+    XLOCK(private_data->imlibdata->x.disp);
     XDrawString(private_data->imlibdata->x.disp, pix, gc, 
 		2, ((ysize+asc+des+yoff)>>1)-des, 
 		plabel, strlen(plabel));
+    XUNLOCK(private_data->imlibdata->x.disp);
   }
   else {
-    XWindowAttributes attr;
-    Pixmap            tpix;
-    GC                lgc;
+    XWindowAttributes  attr;
+    xitk_pixmap_t     *tpix;
+    GC                 lgc;
     
+    XLOCK(private_data->imlibdata->x.disp);
     XGetWindowAttributes(private_data->imlibdata->x.disp, win, &attr);
     
     lgc = XCreateGC(private_data->imlibdata->x.disp, win, None, None);
     XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, lgc);
+    XUNLOCK(private_data->imlibdata->x.disp);
     
-    tpix = XCreatePixmap(private_data->imlibdata->x.disp, win,
-			 xsize, ysize, attr.depth);
+    tpix = xitk_image_create_xitk_pixmap(private_data->imlibdata, xsize, ysize);
     
-    XCopyArea (private_data->imlibdata->x.disp, pix, tpix, lgc, 0, 0,
+    XLOCK(private_data->imlibdata->x.disp);
+    XCopyArea (private_data->imlibdata->x.disp, pix, tpix->pixmap, lgc, 0, 0,
 	       xsize, ysize, 0, 0);
     
-    XDrawString(private_data->imlibdata->x.disp, tpix, lgc, 
+    XDrawString(private_data->imlibdata->x.disp, tpix->pixmap, lgc, 
 		2, ((ysize+asc+des+yoff)>>1)-des, 
 		plabel, strlen(plabel));
     
-    XCopyArea (private_data->imlibdata->x.disp, tpix, pix, lgc, 0, 0,
+    XCopyArea (private_data->imlibdata->x.disp, tpix->pixmap, pix, lgc, 0, 0,
 	       xsize - 1, ysize, 0, 0);
+    XUNLOCK(private_data->imlibdata->x.disp);
 
-    XFreePixmap(private_data->imlibdata->x.disp, tpix);
+    xitk_image_destroy_xitk_pixmap(tpix);
+
+    XLOCK(private_data->imlibdata->x.disp);
     XFreeGC(private_data->imlibdata->x.disp, lgc);
+    XUNLOCK(private_data->imlibdata->x.disp);
   }
-  XUNLOCK(private_data->imlibdata->x.disp);
 
   width = xitk_font_get_text_width(fs, plabel,
 				   (private_data->cursor_pos - private_data->disp_offset));
@@ -357,79 +387,100 @@ static void create_labelofinputtext(xitk_widget_t *w,
 /*
  * Paint the input text box.
  */
-static void paint_inputtext(xitk_widget_t *w, Window win, GC gc) {
+static void paint_inputtext(xitk_widget_t *w) {
   inputtext_private_data_t *private_data;
   int                       button_width, state = 0;
   xitk_image_t             *skin;
-  Pixmap                    btn;
+  xitk_pixmap_t            *btn;
   GC                        lgc;
   XWindowAttributes         attr;
 
-  if(w && (((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT) && (w->visible == 1))) {
-    
+  if(w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT))) {
     private_data = (inputtext_private_data_t *) w->private_data;
-    
-    XLOCK(private_data->imlibdata->x.disp);
-    XGetWindowAttributes(private_data->imlibdata->x.disp, win, &attr);
-    
-    skin = private_data->skin;
-    
-    lgc = XCreateGC(private_data->imlibdata->x.disp, win, None, None);
-    XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, lgc);
 
-    if (skin->mask) {
-      XSetClipOrigin(private_data->imlibdata->x.disp, lgc, w->x, w->y);
-      XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask);
-    }
+    if(w->visible == 1) {
+    
+      
+      if(w->enable && (!private_data->cursor_focus) 
+	 && (xitk_is_mouse_over_widget(private_data->imlibdata->x.disp, w->wl->win, w)))
+	_cursor_focus(private_data, w->wl->win, 1);
 
-    button_width = skin->width / 2;
-    
-    btn = XCreatePixmap(private_data->imlibdata->x.disp, skin->image,
-			button_width, skin->height, attr.depth);
-    
-    if((w->have_focus == FOCUS_RECEIVED) || (private_data->have_focus == FOCUS_MOUSE_IN)) {
-      state = FOCUS;
-      XCopyArea (private_data->imlibdata->x.disp, skin->image,
-		 btn, gc, button_width, 0,
-		 button_width, skin->height, 0, 0);
+      XLOCK(private_data->imlibdata->x.disp);
+      XGetWindowAttributes(private_data->imlibdata->x.disp, w->wl->win, &attr);
+      XUNLOCK(private_data->imlibdata->x.disp);
+
+      skin = private_data->skin;
+      
+      XLOCK(private_data->imlibdata->x.disp);
+      lgc = XCreateGC(private_data->imlibdata->x.disp, w->wl->win, None, None);
+      XCopyGC(private_data->imlibdata->x.disp, w->wl->gc, (1 << GCLastBit) - 1, lgc);
+      XUNLOCK(private_data->imlibdata->x.disp);
+      
+      if (skin->mask) {
+	XLOCK(private_data->imlibdata->x.disp);
+	XSetClipOrigin(private_data->imlibdata->x.disp, lgc, w->x, w->y);
+	XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask->pixmap);
+	XUNLOCK(private_data->imlibdata->x.disp);
+      }
+      
+      button_width = skin->width / 2;
+      
+      btn = xitk_image_create_xitk_pixmap(private_data->imlibdata, button_width, skin->height);
+      
+      XLOCK(private_data->imlibdata->x.disp);
+      if((w->have_focus == FOCUS_RECEIVED) || (private_data->have_focus == FOCUS_MOUSE_IN)) {
+	state = FOCUS;
+	XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap,
+		   btn->pixmap, w->wl->gc, button_width, 0,
+		   button_width, skin->height, 0, 0);
+      }
+      else {
+	state = NORMAL;
+	XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap,
+		   btn->pixmap, w->wl->gc, 0, 0,
+		   button_width, skin->height, 0, 0);
+      }
+      XUNLOCK(private_data->imlibdata->x.disp);
+      
+      create_labelofinputtext(w, w->wl->win, w->wl->gc, btn->pixmap, 
+			      button_width, skin->height, private_data->text, state);
+      
+      XLOCK(private_data->imlibdata->x.disp);
+      XCopyArea (private_data->imlibdata->x.disp, btn->pixmap, w->wl->win, lgc, 0, 0,
+		 button_width, skin->height, w->x, w->y);
+      XUNLOCK(private_data->imlibdata->x.disp);
+
+      xitk_image_destroy_xitk_pixmap(btn);
+
+      XLOCK(private_data->imlibdata->x.disp);
+      XFreeGC(private_data->imlibdata->x.disp, lgc);
+      XUNLOCK(private_data->imlibdata->x.disp);
     }
     else {
-      state = NORMAL;
-      XCopyArea (private_data->imlibdata->x.disp, skin->image,
-		 btn, gc, 0, 0,
-		 button_width, skin->height, 0, 0);
+      if(private_data->cursor_focus)
+	_cursor_focus(private_data, w->wl->win, 0);
     }
-    
-    XUNLOCK(private_data->imlibdata->x.disp);
-    
-    create_labelofinputtext(w, win, gc, btn, 
-			    button_width, skin->height, private_data->text, state);
-    
-    XLOCK(private_data->imlibdata->x.disp);
-    XCopyArea (private_data->imlibdata->x.disp, btn, win, lgc, 0, 0,
-	       button_width, skin->height, w->x, w->y);
-    XFreePixmap(private_data->imlibdata->x.disp, btn);
-    XFreeGC(private_data->imlibdata->x.disp, lgc);
-    XUNLOCK(private_data->imlibdata->x.disp);
-  }
+ }
 
 }
 
 /*
  * Handle click events.
  */
-static int notify_click_inputtext(xitk_widget_list_t *wl, 
-				  xitk_widget_t *w, int bUp, int x, int y) {
+static int notify_click_inputtext(xitk_widget_t *w, int button, int bUp, int x, int y) {
   inputtext_private_data_t *private_data;
   int pos;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     private_data = (inputtext_private_data_t *) w->private_data;
 
-    if(w->have_focus == FOCUS_LOST) {
+    if(w->have_focus == FOCUS_LOST)
       w->have_focus = private_data->have_focus = FOCUS_RECEIVED;
-    }
-
+    
+    if(w->enable && (!private_data->cursor_focus)
+       && (xitk_is_mouse_over_widget(private_data->imlibdata->x.disp, w->wl->win, w)))
+      _cursor_focus(private_data, w->wl->win, 1);
+    
     pos = x - w->x;
 
     {
@@ -448,7 +499,7 @@ static int notify_click_inputtext(xitk_widget_list_t *wl,
       if(fs == NULL)
 	XITK_DIE("%s()@%d: xitk_font_load_font() failed. Exiting\n", __FUNCTION__, __LINE__);
       
-      xitk_font_set_font(fs, wl->gc);
+      xitk_font_set_font(fs, w->wl->gc);
       
       while(width < pos) {
 	
@@ -468,7 +519,7 @@ static int notify_click_inputtext(xitk_widget_list_t *wl,
 
     private_data->cursor_pos = (pos < 0) ? 0 : pos;
 
-    paint_inputtext(w, wl->win, wl->gc);
+    paint_inputtext(w);
 
   }
 
@@ -478,15 +529,20 @@ static int notify_click_inputtext(xitk_widget_list_t *wl,
 /*
  * Handle motion on input text box.
  */
-static int notify_focus_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w, int focus) {
+static int notify_focus_inputtext(xitk_widget_t *w, int focus) {
   inputtext_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     private_data = (inputtext_private_data_t *) w->private_data;
     
     if((private_data->have_focus = focus) == FOCUS_LOST)
       private_data->cursor_pos = -1;
-    
+  
+    if((focus == FOCUS_MOUSE_OUT) || (focus == FOCUS_LOST))
+      _cursor_focus(private_data, w->wl->win, 0);
+    else if(w->enable && (focus == FOCUS_MOUSE_IN))
+      _cursor_focus(private_data, w->wl->win, 1);
+
   }
   
   return 1;
@@ -495,11 +551,10 @@ static int notify_focus_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w, int
 /*
  *
  */
-static void notify_change_skin(xitk_widget_list_t *wl, 
-			       xitk_widget_t *w, xitk_skin_config_t *skonfig) {
+static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
   inputtext_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     private_data = (inputtext_private_data_t *) w->private_data;
 
     if(private_data->skin_element_name) {
@@ -509,7 +564,7 @@ static void notify_change_skin(xitk_widget_list_t *wl,
       XITK_FREE(private_data->fontname);
       private_data->fontname      = strdup(xitk_skin_get_label_fontname(skonfig, private_data->skin_element_name));
       
-      XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->skin);
+      xitk_image_free_image(private_data->imlibdata, &private_data->skin);
       private_data->skin          = xitk_image_load_image(private_data->imlibdata, 
 							  xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
       
@@ -537,7 +592,7 @@ static void notify_change_skin(xitk_widget_list_t *wl,
 /*
  * Erase one char from right of cursor.
  */
-static void inputtext_erase_with_delete(xitk_widget_list_t *wl, xitk_widget_t *w) {
+static void inputtext_erase_with_delete(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   char                      *oldtext, *newtext;
   char                      *p, *pp;
@@ -574,7 +629,7 @@ static void inputtext_erase_with_delete(xitk_widget_list_t *wl, xitk_widget_t *w
       XITK_FREE(private_data->text);
       private_data->text = strdup((newtext != NULL) ? newtext : "");
       
-      paint_inputtext(w, wl->win, wl->gc);
+      paint_inputtext(w);
       
       XITK_FREE(oldtext);
       XITK_FREE(newtext);
@@ -582,14 +637,14 @@ static void inputtext_erase_with_delete(xitk_widget_list_t *wl, xitk_widget_t *w
   }
   else {
     sprintf(private_data->text, "%s", "");
-    paint_inputtext(w, wl->win, wl->gc);
+    paint_inputtext(w);
   }
 }
 
 /*
  * Erase one char from left of cursor.
  */
-static void inputtext_erase_with_backspace(xitk_widget_list_t *wl, xitk_widget_t *w) {
+static void inputtext_erase_with_backspace(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   char                     *oldtext, *newtext;
   char                     *p, *pp;
@@ -626,7 +681,7 @@ static void inputtext_erase_with_backspace(xitk_widget_list_t *wl, xitk_widget_t
       XITK_FREE(private_data->text);
       private_data->text = strdup(newtext);
       
-      paint_inputtext(w, wl->win, wl->gc);
+      paint_inputtext(w);
       
       XITK_FREE(oldtext);
       XITK_FREE(newtext);
@@ -635,14 +690,14 @@ static void inputtext_erase_with_backspace(xitk_widget_list_t *wl, xitk_widget_t
   else {
     sprintf(private_data->text, "%s", "");
     private_data->cursor_pos = 0;
-    paint_inputtext(w, wl->win, wl->gc);
+    paint_inputtext(w);
   }
 }
 
 /*
  * Erase chars from cursor pos to EOL.
  */
-static void inputtext_kill_line(xitk_widget_list_t *wl, xitk_widget_t *w) {
+static void inputtext_kill_line(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   char                      *oldtext, *newtext;
   
@@ -657,7 +712,7 @@ static void inputtext_kill_line(xitk_widget_list_t *wl, xitk_widget_t *w) {
       XITK_FREE(private_data->text);
       private_data->text = strdup(newtext);
       
-      paint_inputtext(w, wl->win, wl->gc);
+      paint_inputtext(w);
       
       XITK_FREE(oldtext);
       XITK_FREE(newtext);
@@ -668,38 +723,39 @@ static void inputtext_kill_line(xitk_widget_list_t *wl, xitk_widget_t *w) {
 /*
  * Move cursor pos to left.
  */
-static void inputtext_move_left(xitk_widget_list_t *wl, xitk_widget_t *w) {
+static void inputtext_move_left(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   
   if(private_data->cursor_pos > 0) {
     private_data->cursor_pos--;
-    paint_inputtext(w, wl->win, wl->gc);
+    paint_inputtext(w);
   }
 }
 
 /*
  * Move cursor pos to right.
  */
-static void inputtext_move_right(xitk_widget_list_t *wl, xitk_widget_t *w) {
+static void inputtext_move_right(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
 
   if(private_data->cursor_pos < strlen(private_data->text)) {
     private_data->cursor_pos++;
-    paint_inputtext(w, wl->win, wl->gc);
+    paint_inputtext(w);
   }
 }
 
 /*
  * Remove focus of widget, then call callback function.
  */
-static void inputtext_exec_return(xitk_widget_list_t *wl, xitk_widget_t *w) {
+static void inputtext_exec_return(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
 
   private_data->cursor_pos   = -1;
   w->have_focus              = 
     private_data->have_focus = FOCUS_LOST;
   //  wl->widget_focused = NULL;
-  paint_inputtext(w, wl->win, wl->gc);
+  _cursor_focus(private_data, w->wl->win, 0);
+  paint_inputtext(w);
   
   if(strlen(private_data->text) > 0) {
     if(private_data->callback)
@@ -710,46 +766,47 @@ static void inputtext_exec_return(xitk_widget_list_t *wl, xitk_widget_t *w) {
 /*
  * Remove focus of widget.
  */
-static void inputtext_exec_escape(xitk_widget_list_t *wl, xitk_widget_t *w) {
+static void inputtext_exec_escape(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   
   private_data->cursor_pos = -1;
   w->have_focus = private_data->have_focus = FOCUS_LOST;
-  wl->widget_focused = NULL;
-  paint_inputtext(w, wl->win, wl->gc);
+  w->wl->widget_focused = NULL;
+  _cursor_focus(private_data, w->wl->win, 0);
+  paint_inputtext(w);
 }
 
 /*
  *
  */
-static void inputtext_move_bol(xitk_widget_list_t *wl, xitk_widget_t *w) {
+static void inputtext_move_bol(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
 
   if(private_data->text) {
     private_data->cursor_pos = private_data->disp_offset = 0;
-    inputtext_recalc_offsets(wl, private_data);
-    paint_inputtext(w, wl->win, wl->gc);
+    inputtext_recalc_offsets(w);
+    paint_inputtext(w);
   }
 }
 
 /*
  *
  */
-static void inputtext_move_eol(xitk_widget_list_t *wl, xitk_widget_t *w) {
+static void inputtext_move_eol(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
 
   if(private_data->text) {
     private_data->cursor_pos = strlen(private_data->text);
-    inputtext_recalc_offsets(wl, private_data);
+    inputtext_recalc_offsets(w);
     
-    paint_inputtext(w, wl->win, wl->gc);
+    paint_inputtext(w);
   }
 }
 
 /*
  * Transpose two characters.
  */
-static void inputtext_transpose_chars(xitk_widget_list_t *wl, xitk_widget_t *w) {
+static void inputtext_transpose_chars(xitk_widget_t *w) {
   inputtext_private_data_t *private_data = (inputtext_private_data_t *) w->private_data;
   
   if(private_data->text && (strlen(private_data->text) >= 2)) {
@@ -759,7 +816,7 @@ static void inputtext_transpose_chars(xitk_widget_list_t *wl, xitk_widget_t *w)
       private_data->text[private_data->cursor_pos - 2] = private_data->text[private_data->cursor_pos - 1];
       private_data->text[private_data->cursor_pos - 1] = c;
       
-      paint_inputtext(w, wl->win, wl->gc);
+      paint_inputtext(w);
     }
   }
 }
@@ -767,10 +824,10 @@ static void inputtext_transpose_chars(xitk_widget_list_t *wl, xitk_widget_t *w)
 /*
  * Handle keyboard event in input text box.
  */
-static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w, XEvent *xev) {
+static void notify_keyevent_inputtext(xitk_widget_t *w, XEvent *xev) {
   inputtext_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     XKeyEvent   keyevent = xev->xkey;
     KeySym      key;
     char        buf[256];
@@ -803,7 +860,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w,
       case XK_a:
       case XK_A:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_move_bol(wl, w);
+	  inputtext_move_bol(w);
 	}
 	break;
 	
@@ -811,7 +868,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w,
       case XK_b:
       case XK_B:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_move_left(wl, w);
+	  inputtext_move_left(w);
 	}
 	break;
 	
@@ -819,7 +876,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w,
       case XK_c:
       case XK_C:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_exec_escape(wl, w);
+	  inputtext_exec_escape(w);
 	}
 	break;
 	
@@ -827,7 +884,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w,
       case XK_d:
       case XK_D:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_erase_with_delete(wl, w);
+	  inputtext_erase_with_delete(w);
 	}
 	break;
 	
@@ -835,7 +892,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w,
       case XK_e:
       case XK_E:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_move_eol(wl, w);
+	  inputtext_move_eol(w);
 	}
 	break;
 
@@ -843,7 +900,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w,
       case XK_f:
       case XK_F:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_move_right(wl, w);
+	  inputtext_move_right(w);
 	}
 	break;
 
@@ -851,7 +908,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w,
       case XK_k:
       case XK_K:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_kill_line(wl, w);
+	  inputtext_kill_line(w);
 	}
 	break;
 
@@ -859,7 +916,7 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w,
       case XK_m:
       case XK_M:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_exec_return(wl, w);
+	  inputtext_exec_return(w);
 	}
 	break;
 
@@ -867,15 +924,21 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w,
       case XK_t:
       case XK_T:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_transpose_chars(wl, w);
+	  inputtext_transpose_chars(w);
 	}
 	break;
 
       case XK_question:
 	if(modifier & MODIFIER_CTRL) {
-	  inputtext_erase_with_backspace(wl, w);
+	  inputtext_erase_with_backspace(w);
 	}
 	break;
+	
+      default:
+	if((!(modifier & MODIFIER_CTRL)) &&
+	   (!(modifier & MODIFIER_META)))
+	  goto __store_as_is;
+	break;
 
       }
     }
@@ -883,54 +946,56 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w,
       return;
     }
     else if(key == XK_Delete) {
-      inputtext_erase_with_delete(wl, w);
+      inputtext_erase_with_delete(w);
     }
     else if(key == XK_BackSpace) {
-      inputtext_erase_with_backspace(wl, w);
+      inputtext_erase_with_backspace(w);
     }
     else if(key == XK_Left) {
-      inputtext_move_left(wl, w);
+      inputtext_move_left(w);
     }
     else if(key == XK_Right) {
-      inputtext_move_right(wl, w);
+      inputtext_move_right(w);
     }
     else if(key == XK_Home) {
-      inputtext_move_bol(wl, w);
+      inputtext_move_bol(w);
     }
     else if(key == XK_End) {
-      inputtext_move_eol(wl, w);
+      inputtext_move_eol(w);
     }
     else if((key == XK_Return) || (key == XK_KP_Enter)) {
-      inputtext_exec_return(wl, w);
+      inputtext_exec_return(w);
     }
     else if((key == XK_Escape) || (key == XK_Tab)) {
-      inputtext_exec_escape(wl, w);
+      inputtext_exec_escape(w);
     }
     else if((buf[0] != 0) && (buf[1] == 0)) {
-      char *oldtext;
-      int pos;
-      
-      if((private_data->text != NULL) 
-	 && (strlen(private_data->text) < private_data->max_length)) {
-
-	oldtext = strdup(private_data->text);
+    __store_as_is:
+      {
+	char *oldtext;
+	int pos;
 	
-	if(strlen(private_data->text) <= private_data->cursor_pos)
-	  pos = strlen(private_data->text);
-	else
-	  pos = private_data->cursor_pos;
-
-	XITK_FREE(private_data->text);
-	private_data->text = (char *) xitk_xmalloc(strlen(oldtext) + 3);
-
-	sprintf(private_data->text, "%s", oldtext);
-	sprintf(&private_data->text[pos], "%c%s%c", buf[0], &oldtext[pos], 0);
-
-	private_data->cursor_pos++;
-	paint_inputtext(w, wl->win, wl->gc);
-	XITK_FREE(oldtext);
+	if((private_data->text != NULL) 
+	   && (strlen(private_data->text) < private_data->max_length)) {
+	  
+	  oldtext = strdup(private_data->text);
+	  
+	  if(strlen(private_data->text) <= private_data->cursor_pos)
+	    pos = strlen(private_data->text);
+	  else
+	    pos = private_data->cursor_pos;
+	  
+	  XITK_FREE(private_data->text);
+	  private_data->text = (char *) xitk_xmalloc(strlen(oldtext) + 3);
+	  
+	  sprintf(private_data->text, "%s", oldtext);
+	  sprintf(&private_data->text[pos], "%c%s%c", buf[0], &oldtext[pos], 0);
+	  
+	  private_data->cursor_pos++;
+	  paint_inputtext(w);
+	  XITK_FREE(oldtext);
+	}
       }
-      
     }
 #if 0
     else {
@@ -941,13 +1006,52 @@ static void notify_keyevent_inputtext(xitk_widget_list_t *wl, xitk_widget_t *w,
 
 }
 
+static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_result_t *result) {
+  int retval = 0;
+
+  switch(event->type) {
+  case WIDGET_EVENT_PAINT:
+    paint_inputtext(w);
+    break;
+  case WIDGET_EVENT_CLICK:
+    result->value = notify_click_inputtext(w, event->button,
+					   event->button_pressed, event->x, event->y);
+    retval = 1;
+    break;
+  case WIDGET_EVENT_FOCUS:
+    notify_focus_inputtext(w, event->focus);
+    break;
+  case WIDGET_EVENT_KEY_EVENT:
+    notify_keyevent_inputtext(w, event->xevent);
+    break;
+  case WIDGET_EVENT_INSIDE:
+    result->value = notify_inside(w, event->x, event->y);
+    retval = 1;
+    break;
+  case WIDGET_EVENT_CHANGE_SKIN:
+    notify_change_skin(w, event->skonfig);
+    break;
+  case WIDGET_EVENT_DESTROY:
+    notify_destroy(w);
+    break;
+  case WIDGET_EVENT_GET_SKIN:
+    if(result) {
+      result->image = get_skin(w, event->skin_layer);
+      retval = 1;
+    }
+    break;
+  }
+  
+  return retval;
+}
+
 /*
  * Return the text of widget.
  */
 char *xitk_inputtext_get_text(xitk_widget_t *w) {
   inputtext_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     private_data = (inputtext_private_data_t *) w->private_data;
     return private_data->text;
   }
@@ -958,16 +1062,16 @@ char *xitk_inputtext_get_text(xitk_widget_t *w) {
 /*
  * Change and redisplay the text of widget.
  */
-void xitk_inputtext_change_text(xitk_widget_list_t *wl, xitk_widget_t *w, char *text) {
+void xitk_inputtext_change_text(xitk_widget_t *w, char *text) {
   inputtext_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)) {
     private_data = (inputtext_private_data_t *) w->private_data;
     XITK_FREE(private_data->text);
     private_data->text = strdup((text != NULL) ? text : "");
     private_data->disp_offset = 0;
     private_data->cursor_pos = -1;
-    paint_inputtext(w, wl->win, wl->gc);
+    paint_inputtext(w);
   }
 
 }
@@ -1009,7 +1113,13 @@ static xitk_widget_t *_xitk_inputtext_create (xitk_widget_list_t *wl,
     private_data->max_visible -= 2;
 
   private_data->disp_offset       = 0;
-  
+
+  XLOCK(private_data->imlibdata->x.disp);
+  private_data->cursor[0] = XCreateFontCursor(private_data->imlibdata->x.disp, XC_left_ptr);
+  private_data->cursor[1] = XCreateFontCursor(private_data->imlibdata->x.disp, XC_xterm);
+  XUNLOCK(private_data->imlibdata->x.disp);
+  private_data->cursor_focus      = 0;
+
   private_data->callback          = it->callback;
   private_data->userdata          = it->userdata;
   
@@ -1021,7 +1131,7 @@ static xitk_widget_t *_xitk_inputtext_create (xitk_widget_list_t *wl,
 
   mywidget->private_data          = private_data;
 
-  mywidget->widget_list           = wl;
+  mywidget->wl                    = wl;
 
   mywidget->enable                = enable;
   mywidget->running               = 1;
@@ -1032,17 +1142,8 @@ static xitk_widget_t *_xitk_inputtext_create (xitk_widget_list_t *wl,
   mywidget->y                     = y;
   mywidget->width                 = private_data->skin->width/2;
   mywidget->height                = private_data->skin->height;
-  mywidget->widget_type           = WIDGET_TYPE_INPUTTEXT;
-
-  mywidget->paint                 = paint_inputtext;
-  mywidget->notify_click          = notify_click_inputtext;
-  mywidget->notify_focus          = notify_focus_inputtext;
-  mywidget->notify_keyevent       = notify_keyevent_inputtext;
-  mywidget->notify_inside         = notify_inside;
-  mywidget->notify_change_skin    = (skin_element_name == NULL) ? NULL : notify_change_skin;
-  mywidget->notify_destroy        = notify_destroy;
-  mywidget->get_skin              = get_skin;
-
+  mywidget->type                  = WIDGET_TYPE_INPUTTEXT | WIDGET_FOCUSABLE | WIDGET_CLICKABLE | WIDGET_KEYABLE;
+  mywidget->event                 = notify_event;
   mywidget->tips_timeout          = 0;
   mywidget->tips_string           = NULL;
 
