@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,15 +17,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: intbox.h,v 1.4 2002/07/17 20:29:31 f1rmb Exp $
+ * $Id: intbox.h,v 1.7 2003/01/05 00:24:35 f1rmb Exp $
  *
  */
 #ifndef HAVE_XITK_INTBOX_H
 #define HAVE_XITK_INTBOX_H
 
-#include <X11/Xlib.h>
-#include "Imlib-light/Imlib.h"
-#include "widget.h"
 #include "_xitk.h"
 
 typedef struct {
@@ -47,18 +44,4 @@ typedef struct {
 
 } intbox_private_data_t;
 
-/*
- * Create a combo box.
- */
-/*
- * Same as above, without skinable feature.
- */
-xitk_widget_t *xitk_noskin_intbox_create(xitk_widget_list_t *wl,
-					 xitk_intbox_widget_t *ib,
-					 int x, int y, int width, int height, 
-					 xitk_widget_t **iw, xitk_widget_t **mw, xitk_widget_t **lw);
-
-void xitk_intbox_set_value(xitk_widget_t *, int);
-int xitk_intbox_get_value(xitk_widget_t *);
-
 #endif
