@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: labelbutton.c,v 1.37 2002/07/17 20:29:31 f1rmb Exp $
+ * $Id: labelbutton.c,v 1.50 2003/03/27 20:52:05 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -28,13 +28,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "Imlib-light/Imlib.h"
-#include "widget.h"
-#include "image.h"
-#include "labelbutton.h"
-#include "font.h"
-#include "widget_types.h"
-
 #include "_xitk.h"
 
 #define CLICK     1
@@ -44,10 +37,10 @@
 /*
  *
  */
-static void notify_destroy(xitk_widget_t *w, void *data) {
+static void notify_destroy(xitk_widget_t *w) {
   lbutton_private_data_t *private_data;
 
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
     XITK_FREE(private_data->label);
     XITK_FREE(private_data->skin_element_name);
@@ -66,7 +59,7 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
 static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
   lbutton_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
 
     if(sk == FOREGROUND_SKIN && private_data->skin) {
@@ -83,13 +76,14 @@ static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
 static int notify_inside(xitk_widget_t *w, int x, int y) {
   lbutton_private_data_t *private_data;
   
-  if (w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
 
     if(w->visible == 1) {
       xitk_image_t *skin = private_data->skin;
       
-      return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, w, skin->mask, x, y);
+      if(skin->mask)
+	return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, w, skin->mask->pixmap, x, y);
     }
     else 
       return 0;
@@ -189,19 +183,21 @@ static void create_labelofbutton(xitk_widget_t *lb,
   XSetForeground(private_data->imlibdata->x.disp, gc, fg);
   
   /*  Put text in the right place */
-  if(private_data->align == LABEL_ALIGN_CENTER) {
+  if(private_data->align == ALIGN_CENTER) {
     XDrawString(private_data->imlibdata->x.disp, pix, gc, 
-		(xsize-(width+xoff))>>1, ((ysize+asc+des+yoff)>>1)-des, 
+		((xsize-(width+xoff))>>1) + private_data->label_offset, 
+		((ysize+asc+des+yoff)>>1)-des, 
 		label, strlen(label));
   }
-  else if(private_data->align == LABEL_ALIGN_LEFT) {
+  else if(private_data->align == ALIGN_LEFT) {
     XDrawString(private_data->imlibdata->x.disp, pix, gc, 
-		((state != CLICK) ? 1 : 5), ((ysize+asc+des+yoff)>>1)-des, 
+		(((state != CLICK) ? 1 : 5)) + private_data->label_offset, 
+		((ysize+asc+des+yoff)>>1)-des, 
 		label, strlen(label));
   }
-  else if(private_data->align == LABEL_ALIGN_RIGHT) {
+  else if(private_data->align == ALIGN_RIGHT) {
     XDrawString(private_data->imlibdata->x.disp, pix, gc, 
-    		xsize - (width + ((state != CLICK) ? 5 : 1)),
+    		(xsize - (width + ((state != CLICK) ? 5 : 1))) + private_data->label_offset,
     		((ysize+asc+des+yoff)>>1)-des, 
     		label, strlen(label));
   }
@@ -217,96 +213,108 @@ static void create_labelofbutton(xitk_widget_t *lb,
 /*
  * Paint the button with correct background pixmap
  */
-static void paint_labelbutton (xitk_widget_t *w, Window win, GC gc) {
+static void paint_labelbutton (xitk_widget_t *w) {
   lbutton_private_data_t *private_data;
   int               button_width, state = 0;
   GC                lgc;
   xitk_image_t     *skin;
-  Pixmap            btn;
   XWindowAttributes attr;
+  xitk_pixmap_t    *btn;
 
-  if (w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
 
     private_data = (lbutton_private_data_t *) w->private_data;
 
     if(w->visible == 1) {
       
       XLOCK(private_data->imlibdata->x.disp);
-      
-      XGetWindowAttributes(private_data->imlibdata->x.disp, win, &attr);
+      XGetWindowAttributes(private_data->imlibdata->x.disp, w->wl->win, &attr);
+      XUNLOCK(private_data->imlibdata->x.disp);
       
       skin = private_data->skin;
       
-      lgc = XCreateGC(private_data->imlibdata->x.disp, win, None, None);
-      XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, lgc);
-      
+      XLOCK(private_data->imlibdata->x.disp);
+      lgc = XCreateGC(private_data->imlibdata->x.disp, w->wl->win, None, None);
+      XCopyGC(private_data->imlibdata->x.disp, w->wl->gc, (1 << GCLastBit) - 1, lgc);
+      XUNLOCK(private_data->imlibdata->x.disp);
+	    
       if (skin->mask) {
+	XLOCK(private_data->imlibdata->x.disp);
 	XSetClipOrigin(private_data->imlibdata->x.disp, lgc, w->x, w->y);
-	XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask);
+	XSetClipMask(private_data->imlibdata->x.disp, lgc, skin->mask->pixmap);
+	XUNLOCK(private_data->imlibdata->x.disp);
       }
       
       button_width = skin->width / 3;
-      btn = XCreatePixmap(private_data->imlibdata->x.disp, skin->image,
-			  button_width, skin->height, attr.depth);
+      btn = xitk_image_create_xitk_pixmap(private_data->imlibdata, button_width, skin->height);
       
       if ((private_data->focus == FOCUS_RECEIVED) || (private_data->focus == FOCUS_MOUSE_IN)) {
 	if (private_data->bClicked) {
 	  state = CLICK;
-	  XCopyArea (private_data->imlibdata->x.disp, skin->image, 
-		     btn, gc, 2*button_width, 0,
+	  XLOCK(private_data->imlibdata->x.disp);
+	  XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap, 
+		     btn->pixmap, w->wl->gc, 2*button_width, 0,
 		     button_width, skin->height, 0, 0);
+	  XUNLOCK(private_data->imlibdata->x.disp);
 	}
 	else {
 	  if(!private_data->bState || private_data->bType == CLICK_BUTTON) {
 	    state = FOCUS;
-	    XCopyArea (private_data->imlibdata->x.disp, skin->image,
-		       btn, gc, button_width, 0,
+	    XLOCK(private_data->imlibdata->x.disp);
+	    XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap,
+		       btn->pixmap, w->wl->gc, button_width, 0,
 		       button_width, skin->height, 0, 0);
+	    XUNLOCK(private_data->imlibdata->x.disp);
 	  }
 	  else {
 	    if(private_data->bType == RADIO_BUTTON) {
 	      state = CLICK;
-	      XCopyArea (private_data->imlibdata->x.disp, skin->image,
-			 btn, gc, 2*button_width, 0,
+	      XLOCK(private_data->imlibdata->x.disp);
+	      XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap,
+			 btn->pixmap, w->wl->gc, 2*button_width, 0,
 			 button_width, skin->height, 0, 0);
+	      XUNLOCK(private_data->imlibdata->x.disp);
 	    }
 	  }
 	}
       }
       else {
+	XLOCK(private_data->imlibdata->x.disp);
 	if(private_data->bState && private_data->bType == RADIO_BUTTON) {
 	  if(private_data->bOldState == 1 && private_data->bClicked == 1) {
 	    state = NORMAL;
-	    XCopyArea (private_data->imlibdata->x.disp, skin->image, btn, gc, 0, 0,
-		       button_width, skin->height, 0, 0);
+	    XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap, btn->pixmap, 
+		       w->wl->gc, 0, 0, button_width, skin->height, 0, 0);
 	  }
 	  else {
 	    state = CLICK;
-	    XCopyArea (private_data->imlibdata->x.disp, skin->image, 
-		       btn, gc, 2*button_width, 0,
+	    XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap, 
+		       btn->pixmap, w->wl->gc, 2*button_width, 0,
 		       button_width, skin->height, 0, 0);
 	  }
 	}
 	else {
 	  state = NORMAL;
-	  XCopyArea (private_data->imlibdata->x.disp, skin->image, btn, gc, 0, 0,
-		     button_width, skin->height, 0, 0);
+	  XCopyArea (private_data->imlibdata->x.disp, skin->image->pixmap, btn->pixmap, 
+		     w->wl->gc, 0, 0, button_width, skin->height, 0, 0);
 	}
+	XUNLOCK(private_data->imlibdata->x.disp);
       }
       
-      /* Only draw text of label is visible */
-      XUNLOCK(private_data->imlibdata->x.disp);
-      
       if(private_data->label_visible) {
-	create_labelofbutton(w, win, gc, btn,
+	create_labelofbutton(w, w->wl->win, w->wl->gc, btn->pixmap,
 			     button_width, skin->height, 
 			     private_data->label, state);
       }
       
       XLOCK(private_data->imlibdata->x.disp);
-      XCopyArea (private_data->imlibdata->x.disp, btn, win, lgc, 0, 0,
+      XCopyArea (private_data->imlibdata->x.disp, btn->pixmap, w->wl->win, lgc, 0, 0,
 		 button_width, skin->height, w->x, w->y);
-      XFreePixmap(private_data->imlibdata->x.disp, btn);
+      XUNLOCK(private_data->imlibdata->x.disp);
+
+      xitk_image_destroy_xitk_pixmap(btn);
+
+      XLOCK(private_data->imlibdata->x.disp);
       XFreeGC(private_data->imlibdata->x.disp, lgc);
       XUNLOCK(private_data->imlibdata->x.disp);
     }
@@ -316,55 +324,57 @@ static void paint_labelbutton (xitk_widget_t *w, Window win, GC gc) {
 /*
  * Handle click events
  */
-static int notify_click_labelbutton (xitk_widget_list_t *wl, 
-				     xitk_widget_t *w, int bUp, int x, int y) {
+static int notify_click_labelbutton (xitk_widget_t *w, int button, int bUp, int x, int y) {
   lbutton_private_data_t *private_data;
+  int                     ret = 0;
   
-  if (w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
-    private_data = (lbutton_private_data_t *) w->private_data;
-    
-    private_data->bClicked = !bUp;
-    private_data->bOldState = private_data->bState;
-    
-    if (bUp && (private_data->focus == FOCUS_RECEIVED)) {
-      private_data->bState = !private_data->bState;
-      paint_labelbutton(w, wl->win, wl->gc);
-      if(private_data->bType == RADIO_BUTTON) {
-	if(private_data->state_callback) {
-	  private_data->state_callback(private_data->bWidget, 
-				       private_data->userdata,
-				       private_data->bState);
+  if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+    if(button == Button1) {
+      private_data = (lbutton_private_data_t *) w->private_data;
+      
+      private_data->bClicked = !bUp;
+      private_data->bOldState = private_data->bState;
+      
+      if (bUp && (private_data->focus == FOCUS_RECEIVED)) {
+	private_data->bState = !private_data->bState;
+	paint_labelbutton(w);
+	if(private_data->bType == RADIO_BUTTON) {
+	  if(private_data->state_callback) {
+	    private_data->state_callback(private_data->bWidget, 
+					 private_data->userdata,
+					 private_data->bState);
+	  }
 	}
-      }
-      else if(private_data->bType == CLICK_BUTTON) {
-	if(private_data->callback) {
-	  private_data->callback(private_data->bWidget, 
-				 private_data->userdata);
+	else if(private_data->bType == CLICK_BUTTON) {
+	  if(private_data->callback) {
+	    private_data->callback(private_data->bWidget, 
+				   private_data->userdata);
+	  }
 	}
       }
+      else
+	paint_labelbutton(w);
+      
+      ret = 1;
     }
-    else
-      paint_labelbutton(w, wl->win, wl->gc);
-
   }
 
-  return 1;
+  return ret;
 }
 
 /*
  * Changing button caption
  */
-int xitk_labelbutton_change_label(xitk_widget_list_t *wl, 
-				  xitk_widget_t *w, char *newlabel) {
+int xitk_labelbutton_change_label(xitk_widget_t *w, char *newlabel) {
   lbutton_private_data_t *private_data;
   
-  if (w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
 
     if((private_data->label = (char *) realloc(private_data->label, strlen(newlabel)+1)) != NULL)
       strcpy(private_data->label, newlabel);
     
-    paint_labelbutton(w, wl->win, wl->gc);
+    paint_labelbutton(w);
     
     return 1;
   }
@@ -378,7 +388,7 @@ int xitk_labelbutton_change_label(xitk_widget_list_t *wl,
 char *xitk_labelbutton_get_label(xitk_widget_t *w) {
   lbutton_private_data_t *private_data;
 
-  if (w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
     return private_data->label;
   }
@@ -389,10 +399,10 @@ char *xitk_labelbutton_get_label(xitk_widget_t *w) {
 /*
  * Handle motion on button
  */
-static int notify_focus_labelbutton (xitk_widget_list_t *wl, xitk_widget_t *w, int focus) {
+static int notify_focus_labelbutton (xitk_widget_t *w, int focus) {
   lbutton_private_data_t *private_data;
   
-  if (w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
     private_data->focus = focus;
   }
@@ -403,18 +413,17 @@ static int notify_focus_labelbutton (xitk_widget_list_t *wl, xitk_widget_t *w, i
 /*
  *
  */
-static void notify_change_skin(xitk_widget_list_t *wl, 
-			       xitk_widget_t *w, xitk_skin_config_t *skonfig) {
+static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
   lbutton_private_data_t *private_data;
   
-  if (w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
     
     if(private_data->skin_element_name) {
       
       xitk_skin_lock(skonfig);
       
-      XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->skin);
+      xitk_image_free_image(private_data->imlibdata, &private_data->skin);
       private_data->skin          = xitk_image_load_image(private_data->imlibdata, 
 		       xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
       XITK_FREE(private_data->normcolor);
@@ -449,13 +458,54 @@ static void notify_change_skin(xitk_widget_list_t *wl,
   }
 }
 
+
+static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_result_t *result) {
+  int retval = 0;
+  
+  switch(event->type) {
+  case WIDGET_EVENT_PAINT:
+    paint_labelbutton(w);
+    break;
+  case WIDGET_EVENT_CLICK:
+    result->value = notify_click_labelbutton(w, event->button,
+					     event->button_pressed, event->x, event->y);
+    retval = 1;
+    break;
+  case WIDGET_EVENT_FOCUS:
+    notify_focus_labelbutton(w, event->focus);
+    if((w->type & WIDGET_GROUP_MENU) && 
+       (event->type == WIDGET_EVENT_FOCUS) && (event->focus == FOCUS_MOUSE_IN)) {
+      menu_auto_pop(w);
+    }
+    break;
+  case WIDGET_EVENT_INSIDE:
+    result->value = notify_inside(w, event->x, event->y);
+    retval = 1;
+    break;
+  case WIDGET_EVENT_CHANGE_SKIN:
+    notify_change_skin(w, event->skonfig);
+    break;
+  case WIDGET_EVENT_DESTROY:
+    notify_destroy(w);
+    break;
+  case WIDGET_EVENT_GET_SKIN:
+    if(result) {
+      result->image = get_skin(w, event->skin_layer);
+      retval = 1;
+    }
+    break;
+  }
+  
+  return retval;
+}
+
 /*
  * Return state (ON/OFF) if button is radio button, otherwise -1
  */
 int xitk_labelbutton_get_state(xitk_widget_t *w) {
   lbutton_private_data_t *private_data;
   
-  if (w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
 
     if(private_data->bType == RADIO_BUTTON)
@@ -471,7 +521,7 @@ int xitk_labelbutton_get_state(xitk_widget_t *w) {
 void xitk_labelbutton_set_alignment(xitk_widget_t *w, int align) {
   lbutton_private_data_t *private_data;
 
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
     private_data->align = align;
   }
@@ -483,7 +533,7 @@ void xitk_labelbutton_set_alignment(xitk_widget_t *w, int align) {
 int xitk_labelbutton_get_alignment(xitk_widget_t *w) {
   lbutton_private_data_t *private_data;
 
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     private_data = (lbutton_private_data_t *) w->private_data;
     return private_data->align;
   }
@@ -491,14 +541,51 @@ int xitk_labelbutton_get_alignment(xitk_widget_t *w) {
   return -1;
 }
 
+/*
+ * Return used font name
+ */
+char *xitk_labelbutton_get_fontname(xitk_widget_t *w) {
+  lbutton_private_data_t *private_data;
+  
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+    private_data = (lbutton_private_data_t *) w->private_data;
+    if(private_data->fontname == NULL)
+      return NULL;
+    return (strdup(private_data->fontname));
+  }
+  
+  return NULL;
+}
+
+/*
+ *
+ */
+void xitk_labelbutton_set_label_offset(xitk_widget_t *w, int offset) {
+  lbutton_private_data_t *private_data;
+  
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+    private_data = (lbutton_private_data_t *) w->private_data;
+    private_data->label_offset = offset;
+  }
+}
+int xitk_labelbutton_get_label_offset(xitk_widget_t *w) {
+  lbutton_private_data_t *private_data;
+  
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+    private_data = (lbutton_private_data_t *) w->private_data;
+    return private_data->label_offset;
+  }
+  return 0;
+}
+
 /*
  * Set radio button to state 'state'
  */
-void xitk_labelbutton_set_state(xitk_widget_t *w, int state, Window win, GC gc) {
+void xitk_labelbutton_set_state(xitk_widget_t *w, int state) {
   lbutton_private_data_t *private_data;
   int                     clk, focus;
   
-  if (w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+  if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
     
     private_data = (lbutton_private_data_t *) w->private_data;
     
@@ -512,18 +599,48 @@ void xitk_labelbutton_set_state(xitk_widget_t *w, int state, Window win, GC gc)
 	private_data->bOldState = private_data->bState;
 	private_data->bState = state;
 
-	paint_labelbutton(w, win, gc);
+	paint_labelbutton(w);
 	
 	private_data->focus = focus;
 	private_data->bClicked = clk;
 	
-	paint_labelbutton(w, win, gc);
+	paint_labelbutton(w);
       }
     }
   }
 
 }
 
+void *labelbutton_get_user_data(xitk_widget_t *w) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+    lbutton_private_data_t *private_data = (lbutton_private_data_t *) w->private_data;
+    
+    return private_data->userdata;
+  }
+
+  return NULL;
+}
+void xitk_labelbutton_callback_exec(xitk_widget_t *w) {
+
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON)) {
+    lbutton_private_data_t *private_data = (lbutton_private_data_t *) w->private_data;
+    
+    if(private_data->bType == RADIO_BUTTON) {
+      if(private_data->state_callback) {
+	private_data->state_callback(private_data->bWidget, 
+				     private_data->userdata,
+				     private_data->bState);
+      }
+    }
+    else if(private_data->bType == CLICK_BUTTON) {
+      if(private_data->callback) {
+	private_data->callback(private_data->bWidget, 
+			       private_data->userdata);
+      }
+    }
+  }
+}
+
 /*
  * Create the labeled button
  */
@@ -566,11 +683,13 @@ static xitk_widget_t *_xitk_labelbutton_create (xitk_widget_list_t *wl,
   private_data->clickcolor        = strdup(ccolor);
   private_data->fontname          = strdup(fontname);
 
+  private_data->label_offset      = 0;
+
   private_data->align             = skin_element_name ? (xitk_skin_get_label_alignment(skonfig, skin_element_name)) : b->align;
 
   mywidget->private_data          = private_data;
 
-  mywidget->widget_list           = wl;
+  mywidget->wl                    = wl;
 
   mywidget->enable                = enable;
   mywidget->running               = 1;
@@ -581,16 +700,8 @@ static xitk_widget_t *_xitk_labelbutton_create (xitk_widget_list_t *wl,
   mywidget->y                     = y;
   mywidget->width                 = private_data->skin->width/3;
   mywidget->height                = private_data->skin->height;
-  mywidget->widget_type           = WIDGET_TYPE_LABELBUTTON;
-  mywidget->paint                 = paint_labelbutton;
-  mywidget->notify_click          = notify_click_labelbutton;
-  mywidget->notify_focus          = notify_focus_labelbutton;
-  mywidget->notify_keyevent       = NULL;
-  mywidget->notify_inside         = notify_inside;
-  mywidget->notify_change_skin    = (skin_element_name == NULL) ? NULL : notify_change_skin;
-  mywidget->notify_destroy        = notify_destroy;
-  mywidget->get_skin              = get_skin;
-
+  mywidget->type                  = WIDGET_TYPE_LABELBUTTON | WIDGET_FOCUSABLE | WIDGET_CLICKABLE | WIDGET_KEYABLE;
+  mywidget->event                 = notify_event;
   mywidget->tips_timeout          = 0;
   mywidget->tips_string           = NULL;
 
