@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: list.c,v 1.9 2002/05/29 21:50:52 f1rmb Exp $
+ * $Id: list.c,v 1.16 2003/03/15 13:15:28 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -29,8 +29,6 @@
 #include <sys/types.h>
 #include <inttypes.h>
 
-#include "widget.h"
-#include "list.h"
 #include "_xitk.h"
 
 /*
@@ -55,14 +53,12 @@ void xitk_list_free(xitk_list_t *l) {
   xitk_node_t *node;
 
   if (!l) {
-    XITK_WARNING("%s(): No list.\n", __FUNCTION__);
+    XITK_WARNING("No list.\n");
     return;
   }
  
-  if(!l->first) {
-    XITK_WARNING("%s(): No node.\n", __FUNCTION__);
+  if(!l->first)
     return;
-  }
 
   node = l->first;
   
@@ -104,10 +100,9 @@ void *xitk_list_next_content (xitk_list_t *l) {
       return NULL;
     
   } 
-  else {
-    XITK_WARNING("%s(): passed end of list", __FUNCTION__);
+  else
     return NULL;
-  }    
+
 }
 
 /*
@@ -128,7 +123,6 @@ void *xitk_list_last_content (xitk_list_t *l) {
     return l->last->content;
   } 
   else {
-    XITK_WARNING("xitk_list : wanted last of empty list");
     return NULL;
   }    
 }
@@ -146,10 +140,9 @@ void *xitk_list_prev_content (xitk_list_t *l) {
     else
       return NULL;
   } 
-  else {
-    XITK_WARNING("xitk_list : passed begin of list");
+  else
     return NULL;
-  }    
+
 }
 
 /*
@@ -218,8 +211,7 @@ void xitk_list_delete_current (xitk_list_t *l) {
     l->cur = node_cur->next;
   }
   else { /* last entry in the list */
-    l->last = node_cur->prev;
-    l->cur = node_cur->prev;
+    l->last = l->cur = node_cur->prev;
   }
 
   //  XITK_FREE(node_cur->content);
