@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrlbrowser.c,v 1.47 2002/07/17 20:29:32 f1rmb Exp $
+ * $Id: mrlbrowser.c,v 1.68 2003/01/05 00:24:35 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -40,26 +40,10 @@
 #include <xine.h>
 
 #include "_xitk.h"
-#include "mrlbrowser.h"
-#include "browser.h"
-#include "label.h"
-#include "button.h"
-#include "labelbutton.h"
-#include "inputtext.h"
-#include "dnd.h"
-#include "window.h"
-#include "combo.h"
-#include "widget.h"
-#include "widget_types.h"
 
 #undef DEBUG_MRLB
 
-typedef struct {
-  char *name;
-  char *ending;
-} mrl_filter_t;
-
-static mrl_filter_t mrl_filters[] = {
+static xitk_mrlbrowser_filter_t __mrl_filters[] = {
   { "All"                 , "*"            },
   { "*.vob"               , "vob"          }, /* mpeg block */
   { "*.mpv"               , "mpv"          }, /* elementary */
@@ -76,19 +60,88 @@ static mrl_filter_t mrl_filters[] = {
   { NULL                  , NULL           }
 };
 
+/*
+ *
+ */
+static void _duplicate_mrl_filters(mrlbrowser_private_data_t *private_data, 
+				   xitk_mrlbrowser_filter_t **mrl_f) {
+  xitk_mrlbrowser_filter_t **mrl_filters = mrl_f;
+  int                        i = 0;
+  
+  private_data->filters_num = 0;
+
+  if(mrl_f == NULL) {
+    int len = sizeof(__mrl_filters) / sizeof(xitk_mrlbrowser_filter_t);
+
+    private_data->mrl_filters = (xitk_mrlbrowser_filter_t **) 
+      xitk_xmalloc(sizeof(xitk_mrlbrowser_filter_t *) * len);
+
+    for(i = 0; i < (len - 1); i++) {
+      
+      private_data->mrl_filters[i]         = (xitk_mrlbrowser_filter_t *) 
+	xitk_xmalloc(sizeof(xitk_mrlbrowser_filter_t));
+      private_data->mrl_filters[i]->name   = strdup(__mrl_filters[i].name);
+      private_data->mrl_filters[i]->ending = strdup(__mrl_filters[i].ending);
+
+    }
+  }
+  else {
+    
+    private_data->mrl_filters = (xitk_mrlbrowser_filter_t **) 
+      xitk_xmalloc(sizeof(xitk_mrlbrowser_filter_t *));
+    
+    for(i = 0; 
+	mrl_filters[i] && (mrl_filters[i]->name && mrl_filters[i]->ending); i++) {
+
+      private_data->mrl_filters            = (xitk_mrlbrowser_filter_t **)
+	realloc(private_data->mrl_filters, sizeof(xitk_mrlbrowser_filter_t *) * (i + 2));
+      private_data->mrl_filters[i]         = (xitk_mrlbrowser_filter_t *) 
+	xitk_xmalloc(sizeof(xitk_mrlbrowser_filter_t));
+      private_data->mrl_filters[i]->name   = strdup(mrl_filters[i]->name);
+      private_data->mrl_filters[i]->ending = strdup(mrl_filters[i]->ending);
+    }
+  }
+  
+  private_data->filters_num            = i;
+  private_data->mrl_filters[i]         = (xitk_mrlbrowser_filter_t *) 
+    xitk_xmalloc(sizeof(xitk_mrlbrowser_filter_t));
+  private_data->mrl_filters[i]->name   = NULL;
+  private_data->mrl_filters[i]->ending = NULL;
+  
+  private_data->filters = (char **) xitk_xmalloc(sizeof(char *) * (private_data->filters_num + 1));
+  
+  for(i = 0; private_data->mrl_filters[i]->name != NULL; i++)
+    private_data->filters[i] = private_data->mrl_filters[i]->name;
+  
+  private_data->filters[i] = NULL;
+  private_data->filter_selected = 0;
+}
 
 /*
  *
  */
-static void notify_destroy(xitk_widget_t *w, void *data) {
+static void notify_destroy(xitk_widget_t *w) {
 
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
     
     xitk_mrlbrowser_destroy(w);
   }
 }
 
+
+static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_result_t *result) {
+  int retval = 0;
+  
+  switch(event->type) {
+  case WIDGET_EVENT_DESTROY:
+    notify_destroy(w);
+    break;
+  }
+  
+  return retval;
+}
+
 /*
  * Redisplay the displayed current origin (useful for file plugin).
  */
@@ -100,27 +153,26 @@ static void update_current_origin(mrlbrowser_private_data_t *private_data) {
   else
     sprintf(private_data->current_origin, "%s", "");
   
-  xitk_label_change_label (private_data->widget_list, 
-			   private_data->widget_origin, 
+  xitk_label_change_label (private_data->widget_origin, 
 			   private_data->current_origin);
   
 }
 
 /*
- * Return the *marker* of the giver file *fully named*
+ * Return the *marker* of a given file *fully named*
  */
-static int get_mrl_marker(mrl_t *mrl) {
+static int get_mrl_marker(xine_mrl_t *mrl) {
   
-  if(mrl->type & mrl_file_symlink)
+  if(mrl->type & XINE_MRL_TYPE_file_symlink)
     return '@';
-  else if(mrl->type & mrl_file_directory)
+  else if(mrl->type & XINE_MRL_TYPE_file_directory)
     return '/';
-  else if(mrl->type & mrl_file_fifo)
+  else if(mrl->type & XINE_MRL_TYPE_file_fifo)
     return '|';
-  else if(mrl->type & mrl_file_sock)
+  else if(mrl->type & XINE_MRL_TYPE_file_sock)
     return '=';
-  else if(mrl->type & mrl_file_normal) {
-    if(mrl->type & mrl_file_exec)
+  else if(mrl->type & XINE_MRL_TYPE_file_normal) {
+    if(mrl->type & XINE_MRL_TYPE_file_exec)
       return '*';
   }
 
@@ -142,15 +194,15 @@ static void mrlbrowser_filter_mrls(mrlbrowser_private_data_t *private_data) {
 
     private_data->mc->mrls_to_disp = 0;
 
-    xitk_strdupa(filter_ends, mrl_filters[private_data->filter_selected].ending);
-    p = filter_ends;
+    xitk_strdupa(filter_ends, private_data->mrl_filters[private_data->filter_selected]->ending);
+    xitk_strdupa(p, filter_ends);
 
     for(i = 0; i < private_data->mrls_num; i++) {
 
-      if(private_data->mc->mrls[i]->type & mrl_file_directory) {
+      if(private_data->mc->mrls[i]->type & XINE_MRL_TYPE_file_directory) {
 
 	if(private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp] == NULL)
-	  private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp] = (mrl_t *) xitk_xmalloc(sizeof(mrl_t));
+	  private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp] = (xine_mrl_t *) xitk_xmalloc(sizeof(xine_mrl_t));
 
 	MRL_DUPLICATE(private_data->mc->mrls[i], private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp]);
 	private_data->mc->mrls_to_disp++;
@@ -159,21 +211,18 @@ static void mrlbrowser_filter_mrls(mrlbrowser_private_data_t *private_data) {
 	ending = strrchr(private_data->mc->mrls[i]->mrl, '.');
 	
 	if(ending) {
+
 	  while((m = xitk_strsep(&filter_ends, ",")) != NULL) {
 	    
-	    while(*m == ' ' || *m == '\t') m++;
-	    
 	    if(!strcasecmp((ending + 1), m)) {
-
 	      if(private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp] == NULL)
-		private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp] = (mrl_t *) xitk_xmalloc(sizeof(mrl_t));
+		private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp] = (xine_mrl_t *) xitk_xmalloc(sizeof(xine_mrl_t));
 	      
 	      MRL_DUPLICATE(private_data->mc->mrls[i], private_data->mc->filtered_mrls[private_data->mc->mrls_to_disp]);
 	      private_data->mc->mrls_to_disp++;
 	    }
 	  }
-	  
-	  filter_ends = p;
+	  xitk_strdupa(filter_ends, p);
 	}
       }    
     }
@@ -183,7 +232,7 @@ static void mrlbrowser_filter_mrls(mrlbrowser_private_data_t *private_data) {
     for(i = 0; i < private_data->mrls_num; i++) {
       
       if(private_data->mc->filtered_mrls[i] == NULL)
-	private_data->mc->filtered_mrls[i] = (mrl_t *) xitk_xmalloc(sizeof(mrl_t));
+	private_data->mc->filtered_mrls[i] = (xine_mrl_t *) xitk_xmalloc(sizeof(xine_mrl_t));
       
       MRL_DUPLICATE(private_data->mc->mrls[i], private_data->mc->filtered_mrls[i]);
     }
@@ -220,7 +269,7 @@ static void mrlbrowser_create_enlighted_entries(mrlbrowser_private_data_t *priva
 	p++;
     }
     
-    if(private_data->mc->filtered_mrls[i]->type & mrl_file_symlink) {
+    if(private_data->mc->filtered_mrls[i]->type & XINE_MRL_TYPE_file_symlink) {
 
       if(private_data->mc->mrls_disp[i])
 	private_data->mc->mrls_disp[i] = (char *) 
@@ -256,14 +305,13 @@ static void mrlbrowser_create_enlighted_entries(mrlbrowser_private_data_t *priva
  * directly with these ones.
  */
 static void mrlbrowser_duplicate_mrls(mrlbrowser_private_data_t *private_data,
-				      mrl_t **mtmp, int num_mrls) {
+				      xine_mrl_t **mtmp, int num_mrls) {
   int i;
   int old_mrls_num = private_data->mrls_num;
 
   for (i = 0; i < num_mrls; i++) {
-
     if(private_data->mc->mrls[i] == NULL)
-      private_data->mc->mrls[i] = (mrl_t *) xitk_xmalloc(sizeof(mrl_t));
+      private_data->mc->mrls[i] = (xine_mrl_t *) xitk_xmalloc(sizeof(xine_mrl_t));
 
     MRL_DUPLICATE(mtmp[i], private_data->mc->mrls[i]);
   }
@@ -302,24 +350,23 @@ static void mrlbrowser_grab_mrls(xitk_widget_t *w, void *data) {
 
     {
       int num_mrls;
-      mrl_t **mtmp = xine_get_browse_mrls(private_data->xine, 
-					  private_data->last_mrl_source, NULL,
-					  &num_mrls);
-      
+      xine_mrl_t **mtmp = xine_get_browse_mrls(private_data->xine, 
+					       private_data->last_mrl_source, 
+					       NULL, &num_mrls);
       if(!mtmp) {
 	private_data->last_mrl_source = (char *)
 	  realloc(private_data->last_mrl_source, strlen(old_old_src) + 1);
 	sprintf(private_data->last_mrl_source, "%s", old_old_src);
 	return;
       }
-
+      
       mrlbrowser_duplicate_mrls(private_data, mtmp, num_mrls);
     }
     
     update_current_origin(private_data);
     mrlbrowser_create_enlighted_entries(private_data);
     xitk_browser_update_list(private_data->mrlb_list, 
-			     private_data->mc->mrls_disp, 
+			     (const char* const*)private_data->mc->mrls_disp, 
 			     private_data->mc->mrls_to_disp, 0);
   }
 }
@@ -333,61 +380,61 @@ static void mrlbrowser_dumpmrl(xitk_widget_t *w, void *data) {
   int j = -1;
   
   if((j = xitk_browser_get_current_selected(private_data->mrlb_list)) >= 0) {
-    mrl_t *ms = private_data->mc->mrls[j];
+    xine_mrl_t *ms = private_data->mc->mrls[j];
     
     printf("mrl '%s'\n\t+", ms->mrl);
 
-    if(ms->type & mrl_file_symlink)
+    if(ms->type & XINE_MRL_TYPE_file_symlink)
       printf(" symlink to '%s' \n\t+ ", ms->link);
 
     printf("[");
 
-    if(ms->type & mrl_unknown)
+    if(ms->type & XINE_MRL_TYPE_unknown)
       printf(" unknown ");
     
-      if(ms->type & mrl_dvd)
+      if(ms->type & XINE_MRL_TYPE_dvd)
 	printf(" dvd ");
       
-      if(ms->type & mrl_vcd)
+      if(ms->type & XINE_MRL_TYPE_vcd)
 	    printf(" vcd ");
       
-      if(ms->type & mrl_net)
+      if(ms->type & XINE_MRL_TYPE_net)
 	printf(" net ");
       
-      if(ms->type & mrl_rtp)
+      if(ms->type & XINE_MRL_TYPE_rtp)
 	printf(" rtp ");
       
-      if(ms->type & mrl_stdin)
+      if(ms->type & XINE_MRL_TYPE_stdin)
 	printf(" stdin ");
       
-      if(ms->type & mrl_file)
+      if(ms->type & XINE_MRL_TYPE_file)
 	printf(" file ");
       
-      if(ms->type & mrl_file_fifo)
+      if(ms->type & XINE_MRL_TYPE_file_fifo)
 	printf(" fifo ");
       
-      if(ms->type & mrl_file_chardev)
+      if(ms->type & XINE_MRL_TYPE_file_chardev)
 	printf(" chardev ");
       
-      if(ms->type & mrl_file_directory)
+      if(ms->type & XINE_MRL_TYPE_file_directory)
 	printf(" directory ");
       
-      if(ms->type & mrl_file_blockdev)
+      if(ms->type & XINE_MRL_TYPE_file_blockdev)
 	printf(" blockdev ");
       
-      if(ms->type & mrl_file_normal)
+      if(ms->type & XINE_MRL_TYPE_file_normal)
 	printf(" normal ");
       
-      if(ms->type & mrl_file_sock)
+      if(ms->type & XINE_MRL_TYPE_file_sock)
 	printf(" sock ");
       
-      if(ms->type & mrl_file_exec)
+      if(ms->type & XINE_MRL_TYPE_file_exec)
 	printf(" exec ");
       
-      if(ms->type & mrl_file_backup)
+      if(ms->type & XINE_MRL_TYPE_file_backup)
 	printf(" backup ");
       
-      if(ms->type & mrl_file_hidden)
+      if(ms->type & XINE_MRL_TYPE_file_hidden)
 	printf(" hidden ");
       
       printf("] (%Ld byte%c)\n", ms->size, (ms->size > 0) ?'s':'\0');
@@ -404,8 +451,8 @@ static void mrlbrowser_dumpmrl(xitk_widget_t *w, void *data) {
 void xitk_mrlbrowser_set_tips_timeout(xitk_widget_t *w, int enabled, unsigned long timeout) {
   mrlbrowser_private_data_t *private_data;
   
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
 
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     
@@ -422,8 +469,8 @@ void xitk_mrlbrowser_set_tips_timeout(xitk_widget_t *w, int enabled, unsigned lo
 Window xitk_mrlbrowser_get_window_id(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
   
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
 
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     return private_data->window;
@@ -439,8 +486,8 @@ Window xitk_mrlbrowser_get_window_id(xitk_widget_t *w) {
 int xitk_mrlbrowser_get_window_info(xitk_widget_t *w, window_info_t *inf) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
 
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     
@@ -456,8 +503,8 @@ int xitk_mrlbrowser_get_window_info(xitk_widget_t *w, window_info_t *inf) {
 int xitk_mrlbrowser_is_running(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
  
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
 
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     return (private_data->running);
@@ -472,8 +519,8 @@ int xitk_mrlbrowser_is_running(xitk_widget_t *w) {
 int xitk_mrlbrowser_is_visible(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
 
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     return (private_data->visible);
@@ -488,19 +535,17 @@ int xitk_mrlbrowser_is_visible(xitk_widget_t *w) {
 void xitk_mrlbrowser_hide(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
 
     private_data = (mrlbrowser_private_data_t *)w->private_data;
 
     if(private_data->visible) {
-
+      private_data->visible = 0;
+      xitk_hide_widgets(private_data->widget_list);
       XLOCK(private_data->imlibdata->x.disp);
       XUnmapWindow(private_data->imlibdata->x.disp, private_data->window);
       XUNLOCK(private_data->imlibdata->x.disp);
-
-      xitk_hide_widgets(private_data->widget_list);
-      private_data->visible = 0;
     }
   }
 }
@@ -511,18 +556,16 @@ void xitk_mrlbrowser_hide(xitk_widget_t *w) {
 void xitk_mrlbrowser_show(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
-
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
     private_data = (mrlbrowser_private_data_t *)w->private_data;
-
-    xitk_show_widgets(private_data->widget_list);
-
-    XLOCK(private_data->imlibdata->x.disp);
-    XMapRaised(private_data->imlibdata->x.disp, private_data->window); 
-    XUNLOCK(private_data->imlibdata->x.disp);
-
+    
     private_data->visible = 1;
+    xitk_show_widgets(private_data->widget_list);
+    XLOCK(private_data->imlibdata->x.disp);
+    XRaiseWindow(private_data->imlibdata->x.disp, private_data->window); 
+    XMapWindow(private_data->imlibdata->x.disp, private_data->window); 
+    XUNLOCK(private_data->imlibdata->x.disp);
   }
 }
 
@@ -532,8 +575,8 @@ void xitk_mrlbrowser_show(xitk_widget_t *w) {
 void xitk_mrlbrowser_set_transient(xitk_widget_t *w, Window window) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET)) && (window != None)) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->type & WIDGET_GROUP_WIDGET)) && (window != None)) {
     private_data = (mrlbrowser_private_data_t *)w->private_data;
 
     if(private_data->visible) {
@@ -552,8 +595,8 @@ void xitk_mrlbrowser_set_transient(xitk_widget_t *w, Window window) {
 void xitk_mrlbrowser_destroy(xitk_widget_t *w) {
   mrlbrowser_private_data_t *private_data;
 
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) && 
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) && 
+	   (w->type & WIDGET_GROUP_WIDGET))) {
     private_data = (mrlbrowser_private_data_t *)w->private_data;
 
     private_data->running = 0;
@@ -583,7 +626,15 @@ void xitk_mrlbrowser_destroy(xitk_widget_t *w) {
     
     {
       int i;
-      for(i = private_data->mrls_num; i > 0; i--) {
+      
+      for(i = 0; i <= private_data->filters_num; i++) {
+	XITK_FREE(private_data->mrl_filters[i]->name);
+	XITK_FREE(private_data->mrl_filters[i]->ending);
+	XITK_FREE(private_data->mrl_filters[i]);
+      }
+      XITK_FREE(private_data->mrl_filters);
+      
+      for(i = 0; i < private_data->mrls_num; i++) {
 	MRL_ZERO(private_data->mc->mrls[i]);
 	XITK_FREE(private_data->mc->mrls[i]);
 	MRL_ZERO(private_data->mc->filtered_mrls[i]);
@@ -591,9 +642,6 @@ void xitk_mrlbrowser_destroy(xitk_widget_t *w) {
 	XITK_FREE(private_data->mc->mrls_disp[i]);
       }
       
-      for(i = 0; private_data->filters[i] != NULL; i++)
-	free(private_data->filters[i]);
-
       free(private_data->filters);
     }
     
@@ -626,8 +674,8 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
   mrlbrowser_private_data_t *private_data;
   XSizeHints                 hint;
 
-  if(w && (((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
-	   (w->widget_type & WIDGET_GROUP_WIDGET))) {
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
     private_data = (mrlbrowser_private_data_t *)w->private_data;
     
     xitk_skin_lock(skonfig);
@@ -640,10 +688,13 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
 							private_data->skin_element_name)))) {
       XITK_DIE("%s(): couldn't find image for background\n", __FUNCTION__);
     }
-    
+    XUNLOCK(private_data->imlibdata->x.disp);
+
     hint.width  = new_img->rgb_width;
     hint.height = new_img->rgb_height;
-    hint.flags  = PSize;
+    hint.flags  = PPosition | PSize;
+
+    XLOCK(private_data->imlibdata->x.disp);
     XSetWMNormalHints(private_data->imlibdata->x.disp, private_data->window, &hint);
 
     XResizeWindow (private_data->imlibdata->x.disp, private_data->window,
@@ -661,10 +712,8 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
     private_data->bg_image = new_img;
 
     XLOCK(private_data->imlibdata->x.disp);
-
     Imlib_destroy_image(private_data->imlibdata, old_img);
     Imlib_apply_image(private_data->imlibdata, new_img, private_data->window);
-
     XUNLOCK(private_data->imlibdata->x.disp);
 
     xitk_skin_unlock(skonfig);
@@ -674,19 +723,45 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
     {
       int x, y;
       int i = 0;
+      int dir;
       
       x = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name_ip);
       y = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name_ip);
+      dir = xitk_skin_get_direction(skonfig, private_data->skin_element_name_ip);
     
-      while(private_data->autodir_plugins[i] != NULL) {
-	
-	(void) xitk_set_widget_pos(private_data->autodir_plugins[i], x, y);
-	
-	y += xitk_get_widget_height(private_data->autodir_plugins[i]) + 1;
-	i++;
+      switch(dir) {
+      case DIRECTION_UP:
+      case DIRECTION_DOWN:
+	while(private_data->autodir_plugins[i] != NULL) {
+	  
+	  (void) xitk_set_widget_pos(private_data->autodir_plugins[i], x, y);
+	  
+	  if(dir == DIRECTION_DOWN)
+	    y += xitk_get_widget_height(private_data->autodir_plugins[i]) + 1;
+	  else
+	    y -= (xitk_get_widget_height(private_data->autodir_plugins[i]) + 1);
+	  
+	  i++;
+	}
+	break;
+      case DIRECTION_LEFT:
+      case DIRECTION_RIGHT:
+	while(private_data->autodir_plugins[i] != NULL) {
+	  
+	  (void) xitk_set_widget_pos(private_data->autodir_plugins[i], x, y);
+	  
+	  if(dir == DIRECTION_RIGHT)
+	    x += xitk_get_widget_width(private_data->autodir_plugins[i]) + 1;
+	  else
+	    x -= (xitk_get_widget_width(private_data->autodir_plugins[i]) + 1);
+	  
+	  i++;
+	}
+	break;
       }
-    }
 
+    }
+    
     xitk_paint_widget_list(private_data->widget_list);
     
   }
@@ -697,13 +772,13 @@ void xitk_mrlbrowser_change_skins(xitk_widget_t *w, xitk_skin_config_t *skonfig)
  */
 static void mrlbrowser_select_mrl(mrlbrowser_private_data_t *private_data,
 				  int j, int add_callback, int play_callback) {
-  mrl_t *ms = private_data->mc->filtered_mrls[j];
+  xine_mrl_t *ms = private_data->mc->filtered_mrls[j];
   char   buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
   
   memset(&buf, '\0', sizeof(buf));
   sprintf(buf, "%s", ms->mrl);
   
-  if((ms->type & mrl_file) && (ms->type & mrl_file_directory)) {
+  if((ms->type & XINE_MRL_TYPE_file) && (ms->type & XINE_MRL_TYPE_file_directory)) {
     char *filename = ms->mrl;
     
     if(ms->origin) {
@@ -743,17 +818,18 @@ static void mrlbrowser_select_mrl(mrlbrowser_private_data_t *private_data,
     
     {
       int num_mrls;
-      mrl_t **mtmp = xine_get_browse_mrls(private_data->xine, 
-					  private_data->last_mrl_source, 
-					  buf, &num_mrls);
-      
+      xine_mrl_t **mtmp = xine_get_browse_mrls(private_data->xine, 
+					       private_data->last_mrl_source, 
+					       buf, &num_mrls);
+
       mrlbrowser_duplicate_mrls(private_data, mtmp, num_mrls);
+
     }
     
     update_current_origin(private_data);
     mrlbrowser_create_enlighted_entries(private_data);
     xitk_browser_update_list(private_data->mrlb_list, 
-			     private_data->mc->mrls_disp, 
+			     (const char* const*)private_data->mc->mrls_disp, 
 			     private_data->mc->mrls_to_disp, 0);
     
   }
@@ -806,15 +882,16 @@ static void handle_dbl_click(xitk_widget_t *w, void *data, int selected) {
   XLOCK(private_data->imlibdata->x.disp);
   XPeekEvent(private_data->imlibdata->x.disp, &xev);
   XUNLOCK(private_data->imlibdata->x.disp);
-
+  
   xitk_get_key_modifier(&xev, &modifier);
-
+  
   if((modifier & MODIFIER_CTRL) && (modifier & MODIFIER_SHIFT))
     mrlbrowser_select_mrl(private_data, selected, 1, 1);
   else if(modifier & MODIFIER_CTRL)
     mrlbrowser_select_mrl(private_data, selected, 1, 0);
   else
     mrlbrowser_select_mrl(private_data, selected, 0, 1); 
+
 }
 
 /*
@@ -826,7 +903,7 @@ static void combo_filter_select(xitk_widget_t *w, void *data, int select) {
   private_data->filter_selected = select;
   mrlbrowser_create_enlighted_entries(private_data);
   xitk_browser_update_list(private_data->mrlb_list, 
-  			   private_data->mc->mrls_disp, 
+  			  (const char* const*) private_data->mc->mrls_disp, 
   			   private_data->mc->mrls_to_disp, 0);
 }
 
@@ -838,20 +915,6 @@ static void mrlbrowser_handle_event(XEvent *event, void *data) {
   
   switch(event->type) {
 
-  case ButtonPress: {
-    XButtonEvent *bevent = (XButtonEvent *) event;
-
-    XLOCK(private_data->imlibdata->x.disp);
-    XRaiseWindow(private_data->imlibdata->x.disp, private_data->window);
-    XUNLOCK(private_data->imlibdata->x.disp);
-
-    if (bevent->button == Button4)
-      xitk_browser_step_down((xitk_widget_t *)private_data->mrlb_list, NULL);
-    else if(bevent->button == Button5)
-      xitk_browser_step_up((xitk_widget_t *)private_data->mrlb_list, NULL);
-  }
-  break;
-
   case MappingNotify:
     XLOCK(private_data->imlibdata->x.disp);
     XRefreshKeyboardMapping((XMappingEvent *) event);
@@ -923,6 +986,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   xitk_widget_t              *mywidget;
   mrlbrowser_private_data_t  *private_data;
   xitk_labelbutton_widget_t   lb;
+  xitk_widget_t               *default_source = NULL;
   xitk_button_widget_t        pb;
   xitk_label_widget_t         lbl;
   xitk_combo_widget_t         cmb;
@@ -968,6 +1032,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
     XUNLOCK(mb->imlibdata->x.disp);
     return NULL;
   }
+  XUNLOCK(mb->imlibdata->x.disp);
 
   private_data->mc              = (mrl_contents_t *) xitk_xmalloc(sizeof(mrl_contents_t));
   private_data->mrls_num        = 0;
@@ -979,11 +1044,13 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   hint.height = private_data->bg_image->rgb_height;
   hint.flags  = PPosition | PSize;
 
+  XLOCK(mb->imlibdata->x.disp);
   screen = DefaultScreen(mb->imlibdata->x.disp);
 
   XAllocNamedColor(mb->imlibdata->x.disp, 
 		   Imlib_get_colormap(mb->imlibdata),
 		   "black", &black, &dummy);
+  XUNLOCK(mb->imlibdata->x.disp);
 
   attr.override_redirect = False;
   attr.background_pixel  = black.pixel;
@@ -998,12 +1065,11 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   attr.border_pixel      = 1;
   attr.colormap		 = Imlib_get_colormap(mb->imlibdata);
 
+  XLOCK(mb->imlibdata->x.disp);
   private_data->window   = XCreateWindow (mb->imlibdata->x.disp,
-					  DefaultRootWindow(mb->imlibdata->x.disp), 
-					  hint.x, hint.y, hint.width, 
-					  hint.height, 0, 
-					  mb->imlibdata->x.depth, 
-					  InputOutput, 
+					  mb->imlibdata->x.root, 
+					  hint.x, hint.y, hint.width, hint.height, 0, 
+					  mb->imlibdata->x.depth, InputOutput, 
 					  mb->imlibdata->x.visual,
 					  CWBackPixel | CWBorderPixel | CWColormap | CWOverrideRedirect,
 					  &attr);
@@ -1012,12 +1078,14 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
 			 None, NULL, 0, &hint);
 
   XSelectInput(mb->imlibdata->x.disp, private_data->window, INPUT_MOTION | KeymapStateMask);
+  XUNLOCK(mb->imlibdata->x.disp);
   
   /*
    * layer above most other things, like gnome panel
    * WIN_LAYER_ABOVE_DOCK  = 10
    *
    */
+  XLOCK(mb->imlibdata->x.disp);
   if(mb->layer_above) {
     XA_WIN_LAYER = XInternAtom(mb->imlibdata->x.disp, "_WIN_LAYER", False);
     
@@ -1031,12 +1099,16 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
    * wm, no border please
    */
   prop                 = XInternAtom(mb->imlibdata->x.disp, "_MOTIF_WM_HINTS", True);
+  XUNLOCK(mb->imlibdata->x.disp);
+
   mwmhints.flags       = MWM_HINTS_DECORATIONS;
   mwmhints.decorations = 0;
   
+  XLOCK(mb->imlibdata->x.disp);
   XChangeProperty(mb->imlibdata->x.disp, private_data->window, prop, prop, 32,
                   PropModeReplace, (unsigned char *) &mwmhints,
                   PROP_MWM_HINTS_ELEMENTS);						
+
   if(mb->window_trans != None)
     XSetTransientForHint (mb->imlibdata->x.disp, private_data->window, mb->window_trans);
 
@@ -1047,7 +1119,9 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
     xclasshint->res_class = mb->resource_class ? mb->resource_class : "xitk";
     XSetClassHint(mb->imlibdata->x.disp, private_data->window, xclasshint);
   }
+  XUNLOCK(mb->imlibdata->x.disp);
   
+  XLOCK(mb->imlibdata->x.disp);
   wm_hint = XAllocWMHints();
   if (wm_hint != NULL) {
     wm_hint->input         = True;
@@ -1056,7 +1130,9 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
     XSetWMHints(mb->imlibdata->x.disp, private_data->window, wm_hint);
     XFree(wm_hint);
   }
+  XUNLOCK(mb->imlibdata->x.disp);
   
+  XLOCK(mb->imlibdata->x.disp);
   gc = XCreateGC(mb->imlibdata->x.disp, private_data->window, 0, 0);
   
   Imlib_apply_image(mb->imlibdata, 
@@ -1077,7 +1153,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   lb.skin_element_name = mb->select.skin_element_name;
   xitk_list_append_content(private_data->widget_list->l,
 		   (w = xitk_labelbutton_create (private_data->widget_list, skonfig, &lb)));
-  w->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
+  w->type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
   xitk_set_widget_tips(w, _("Select current entry"));
   
   pb.skin_element_name = mb->play.skin_element_name;
@@ -1085,7 +1161,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   pb.userdata          = (void *)private_data;
   xitk_list_append_content(private_data->widget_list->l,
 		   (w = xitk_button_create (private_data->widget_list, skonfig, &pb)));
-  w->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
+  w->type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
   xitk_set_widget_tips(w, _("Play selected entry"));
 
   lb.button_type       = CLICK_BUTTON;
@@ -1096,7 +1172,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   lb.skin_element_name = mb->dismiss.skin_element_name;
   xitk_list_append_content(private_data->widget_list->l,
 		   (w = xitk_labelbutton_create (private_data->widget_list, skonfig, &lb)));
-  w->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
+  w->type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
   xitk_set_widget_tips(w, _("Close MRL browser window"));
   
   private_data->add_callback      = mb->select.callback;
@@ -1108,7 +1184,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   xitk_list_append_content (private_data->widget_list->l,
 		    (private_data->mrlb_list = 
 		     xitk_browser_create(private_data->widget_list, skonfig, &mb->browser)));
-  private_data->mrlb_list->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
+  private_data->mrlb_list->type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
 
   lbl.label             = "";
   lbl.skin_element_name = mb->origin.skin_element_name;
@@ -1118,7 +1194,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   xitk_list_append_content(private_data->widget_list->l,
 			  (private_data->widget_origin = 
 			   xitk_label_create(private_data->widget_list, skonfig, &lbl)));
-  private_data->widget_origin->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
+  private_data->widget_origin->type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
   
   memset(&private_data->current_origin, 0, strlen(private_data->current_origin));
   if(mb->origin.cur_origin)
@@ -1133,7 +1209,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
     lbl.callback          = NULL;
     xitk_list_append_content(private_data->widget_list->l, 
 			     (w = xitk_label_create (private_data->widget_list, skonfig, &lbl)));
-    w->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
+    w->type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
 
   }
 
@@ -1143,10 +1219,12 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   {
     int x, y;
     int i = 0;
+    int dir;
     
     private_data->skin_element_name_ip = strdup(mb->ip_name.button.skin_element_name);
     x = xitk_skin_get_coord_x(skonfig, mb->ip_name.button.skin_element_name);
     y = xitk_skin_get_coord_y(skonfig, mb->ip_name.button.skin_element_name);
+    dir = xitk_skin_get_direction(skonfig, mb->ip_name.button.skin_element_name);
     
     while(mb->ip_availables[i] != NULL) {
 
@@ -1156,36 +1234,45 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
       lb.state_callback = NULL;
       lb.userdata       = (void *)private_data;
       lb.skin_element_name = mb->ip_name.button.skin_element_name;
+
       xitk_list_append_content(private_data->widget_list->l,
 		       (private_data->autodir_plugins[i] = 
 			xitk_labelbutton_create (private_data->widget_list, skonfig, &lb)));
       xitk_set_widget_tips(private_data->autodir_plugins[i], 
-			   xine_get_input_plugin_description(mb->xine, mb->ip_availables[i]));
-
-      private_data->autodir_plugins[i]->widget_type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
+			   (char *) xine_get_input_plugin_description(mb->xine, mb->ip_availables[i]));
+      private_data->autodir_plugins[i]->type |= WIDGET_GROUP | WIDGET_GROUP_MRLBROWSER;
       
       (void) xitk_set_widget_pos(private_data->autodir_plugins[i], x, y);
 
-      y += xitk_get_widget_height(private_data->autodir_plugins[i]) + 1;
+      /* make "file" the default mrl source */
+      if( !strcasecmp(lb.label, "file") )
+        default_source = private_data->autodir_plugins[i];
+      
+      switch(dir) {
+      case DIRECTION_UP:
+      case DIRECTION_DOWN:
+	if(dir == DIRECTION_DOWN)
+	  y += xitk_get_widget_height(private_data->autodir_plugins[i]) + 1;
+	else
+	  y -= (xitk_get_widget_height(private_data->autodir_plugins[i]) + 1);
+	break;
+      case DIRECTION_LEFT:
+      case DIRECTION_RIGHT:
+	if(dir == DIRECTION_RIGHT)
+	  x += xitk_get_widget_width(private_data->autodir_plugins[i]) + 1;
+	else
+	  x -= (xitk_get_widget_width(private_data->autodir_plugins[i]) + 1);
+	break;
+      }
+
       i++;
     }
     if(i)
       private_data->autodir_plugins[i+1] = NULL;
   }
 
+  _duplicate_mrl_filters(private_data, mb->mrl_filters);
 
-  private_data->filters = (char **) xitk_xmalloc(sizeof(char *) * 
-						 (sizeof(mrl_filters) / sizeof(mrl_filter_t)));
-  {
-    int i;
-
-    for(i = 0; mrl_filters[i].name != NULL; i++)
-      private_data->filters[i] = strdup(mrl_filters[i].name);
-
-    private_data->filters[i] = NULL;
-    private_data->filter_selected = 0;
-  }
-  
   cmb.skin_element_name = mb->combo.skin_element_name;
   cmb.parent_wlist      = private_data->widget_list;
   cmb.entries           = private_data->filters;
@@ -1198,7 +1285,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
 
   private_data->visible        = 1;
   
-  mywidget->widget_list        = NULL;
+  mywidget->wl                 = NULL;
 
   mywidget->enable             = 1;
   mywidget->running            = 1;
@@ -1209,25 +1296,18 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   mywidget->y                  = mb->y;
   mywidget->width              = private_data->bg_image->width;
   mywidget->height             = private_data->bg_image->height;
-  mywidget->widget_type        = WIDGET_GROUP | WIDGET_GROUP_WIDGET | WIDGET_GROUP_MRLBROWSER;
-  mywidget->paint              = NULL;
-  mywidget->notify_click       = NULL;
-  mywidget->notify_focus       = NULL;
-  mywidget->notify_keyevent    = NULL;
-  mywidget->notify_inside      = NULL;
-  mywidget->notify_change_skin = NULL;
-  mywidget->notify_destroy     = notify_destroy;
-  mywidget->get_skin           = NULL;
-
+  mywidget->type               = WIDGET_GROUP | WIDGET_GROUP_WIDGET | WIDGET_GROUP_MRLBROWSER;
+  mywidget->event              = notify_event;
   mywidget->tips_timeout       = 0;
   mywidget->tips_string        = NULL;
 
   xitk_browser_update_list(private_data->mrlb_list, 
-			   private_data->mc->mrls_disp, 
+			   (const char* const*)private_data->mc->mrls_disp, 
 			   private_data->mrls_num, 0);
-  
+
   XLOCK (mb->imlibdata->x.disp);
-  XMapRaised(mb->imlibdata->x.disp, private_data->window); 
+  XRaiseWindow(mb->imlibdata->x.disp, private_data->window); 
+  XMapWindow(mb->imlibdata->x.disp, private_data->window); 
   XUNLOCK (mb->imlibdata->x.disp);
 
   private_data->widget_key = 
@@ -1236,8 +1316,20 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
 				mrlbrowser_handle_event,
 				NULL,
 				mb->dndcallback,
-				private_data->widget_list,
-				(void *) private_data);
+				private_data->widget_list,(void *) private_data);
+
+  while(!xitk_is_window_visible(mb->imlibdata->x.disp, private_data->window))
+    xitk_usec_sleep(5000);
+  
+  xitk_mrlbrowser_change_skins(mywidget, skonfig);
+  
+  XLOCK(mb->imlibdata->x.disp);
+  XSetInputFocus(mb->imlibdata->x.disp, private_data->window, RevertToParent, CurrentTime);
+  XUNLOCK(mb->imlibdata->x.disp);
+
+  if( default_source && !private_data->last_mrl_source ) {
+    mrlbrowser_grab_mrls(default_source, private_data);
+  }
 
   return mywidget;
 }
