@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skin.c,v 1.15 2002/06/06 11:30:31 f1rmb Exp $
+ * $Id: skin.c,v 1.23 2003/03/28 13:50:13 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -32,11 +32,8 @@
 #include <pthread.h>
 #include <errno.h>
 
-#include "skin.h"
 #include "_xitk.h"
 
-extern int errno;
-
 #undef DEBUG_SKIN
 
 /************************************************************************************
@@ -152,6 +149,7 @@ static void _nullify_me(xitk_skin_element_t *s) {
   s->color_click     = NULL;
   s->font            = NULL;
   s->browser_entries = -2;
+  s->direction       = DIRECTION_LEFT; /* Compatibility */
 }
 
 /*
@@ -294,9 +292,9 @@ static int skin_get_align_value(const char *val) {
     const char *str;
     int value;
   } aligns[] = {
-    { "left",   LABEL_ALIGN_LEFT   }, 
-    { "center", LABEL_ALIGN_CENTER }, 
-    { "right",  LABEL_ALIGN_RIGHT  },
+    { "left",   ALIGN_LEFT   }, 
+    { "center", ALIGN_CENTER }, 
+    { "right",  ALIGN_RIGHT  },
     { NULL,     0 }
   };
   int i;
@@ -308,7 +306,33 @@ static int skin_get_align_value(const char *val) {
       return aligns[i].value;
   }
 
-  return LABEL_ALIGN_CENTER;
+  return ALIGN_CENTER;
+}
+
+/*
+ * Return direction
+ */
+static int skin_get_direction(const char *val) {
+  static struct {
+    const char *str;
+    int         value;
+  } directions[] = {
+    { "left",   DIRECTION_LEFT   }, 
+    { "right",  DIRECTION_RIGHT  }, 
+    { "up",     DIRECTION_UP     },
+    { "down",   DIRECTION_DOWN   },
+    { NULL,     0 }
+  };
+  int   i;
+
+  assert(val != NULL);
+  
+  for(i = 0; directions[i].str != NULL; i++) {
+    if(!(strcasecmp(directions[i].str, val)))
+      return directions[i].value;
+  }
+
+  return DIRECTION_LEFT;
 }
 
 /*
@@ -400,9 +424,10 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
       skin_get_next_line(skonfig);
     }
     else if(!strncasecmp(skonfig->ln, "label", 5)) {
-      skonfig->celement->print = 1;
-      
-      skonfig->celement->align = LABEL_ALIGN_CENTER;
+      skonfig->celement->print           = 1;
+      skonfig->celement->animation_step  = 1;
+      skonfig->celement->animation_timer = xitk_get_timer_label_animation();
+      skonfig->celement->align           = ALIGN_CENTER;
 
       while(skin_end_section(skonfig) < 0) {
 	skin_get_next_line(skonfig);
@@ -445,10 +470,18 @@ static void skin_parse_subsection(xitk_skin_config_t *skonfig) {
 	  skin_set_pos_to_value(&p);
 	  skonfig->celement->print = skin_get_bool_value(p);
 	}
+	else if(!strncasecmp(skonfig->ln, "timer", 5)) {
+	  skin_set_pos_to_value(&p);
+	  skonfig->celement->animation_timer = strtol(p, &p, 10);
+	}
 	else if(!strncasecmp(skonfig->ln, "font", 4)) {
 	  skin_set_pos_to_value(&p);
 	  skonfig->celement->font = strdup(p);
 	}
+	else if(!strncasecmp(skonfig->ln, "step", 4)) {
+	  skin_set_pos_to_value(&p);
+	  skonfig->celement->animation_step = strtol(p, &p, 10);
+	}
 
       }
       skin_get_next_line(skonfig);
@@ -509,7 +542,11 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 	  }
 	  else {
 
-	    if(!strncasecmp(skonfig->ln, "visible", 7)) {
+	    if(!strncasecmp(skonfig->ln, "direction", 9)) {
+	      skin_set_pos_to_value(&p);
+	      s->direction = skin_get_direction(p);
+	    }
+	    else if(!strncasecmp(skonfig->ln, "visible", 7)) {
 	      skin_set_pos_to_value(&p);
 	      s->visible = skin_get_bool_value(p);
 	    }
@@ -546,6 +583,10 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 	    skonfig->load_command = _get_expanded_command(skonfig, p);
 	    return;
 	  }
+	  else if(!strncasecmp(section, "animation", 9)) {
+	    skonfig->animation = strdup(p);
+	    return;
+	  }
 	  else if(!strncasecmp(section, "version", 7)) {
 	    skonfig->version = strtol(p, &p, 10);
 	    return;
@@ -571,7 +612,7 @@ static void skin_parse_section(xitk_skin_config_t *skonfig) {
 	    return;
 	  }
 	  else {
-	    XITK_WARNING("%s(): wrong section entry found: '%s'\n", __FUNCTION__, section);
+	    XITK_WARNING("wrong section entry found: '%s'\n", section);
 	    return;
 	  }
 	}
@@ -602,6 +643,7 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
     printf("     uload cmd '%s'\n", skonfig->unload_command);
     printf("     URL       '%s'\n", skonfig->url);
     printf("     logo      '%s'\n", skonfig->logo);
+    printf("     animation '%s'\n", skonfig->animation);
 
     while(s) {
       printf("Section '%s'\n", s->section);
@@ -609,6 +651,7 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
       printf("  visible     = %d\n", s->visible);
       printf("  X           = %d\n", s->x);
       printf("  Y           = %d\n", s->y);
+      printf("  direction   = %d\n", s->direction);
       printf("  pixmap      = '%s'\n", s->pixmap);
 
       if(s->slider_type) {
@@ -620,6 +663,7 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
 	printf("  browser entries = %d\n", s->browset_entries);
 
       printf("  animation   = %d\n", s->animation);
+      printf("  step        = %d\n", s->animation_step);
       printf("  print       = %d\n", s->print);
       printf("  static      = %d\n", s->staticity);
       printf("  length      = %d\n", s->length);
@@ -639,8 +683,10 @@ static void check_skonfig(xitk_skin_config_t *skonfig) {
       printf("  visible     = %d\n", s->visible);
       printf("  X           = %d\n", s->x);
       printf("  Y           = %d\n", s->y);
+      printf("  direction   = %d\n", s->direction);
       printf("  pixmap      = '%s'\n", s->pixmap);
       printf("  animation   = %d\n", s->animation);
+      printf("  step        = %d\n", s->animation_step);
       printf("  print       = %d\n", s->print);
       printf("  static      = %d\n", s->staticity);
       printf("  length      = %d\n", s->length);
@@ -683,12 +729,13 @@ xitk_skin_config_t *xitk_skin_init_config(void) {
   xitk_skin_config_t *skonfig;
   
   if((skonfig = (xitk_skin_config_t *) xitk_xmalloc(sizeof(xitk_skin_config_t))) == NULL) {
-    XITK_DIE("%s(): xitk_xmalloc() failed: %s\n", __FUNCTION__, strerror(errno));
+    XITK_DIE("xitk_xmalloc() failed: %s\n", strerror(errno));
   }
   skonfig->version = -1;
   skonfig->first = skonfig->last = skonfig->celement = NULL;
   skonfig->name = skonfig->author = skonfig->date = skonfig->url = 
-    skonfig->load_command = skonfig->unload_command = skonfig->logo = NULL;
+    skonfig->load_command = skonfig->unload_command = skonfig->logo = 
+    skonfig->animation = NULL;
   skonfig->skinfile = skonfig->path = NULL;
 
   skonfig->ln = skonfig->buf;
@@ -731,6 +778,7 @@ void xitk_skin_free_config(xitk_skin_config_t *skonfig) {
   XITK_FREE(skonfig->date);
   XITK_FREE(skonfig->url);
   XITK_FREE(skonfig->logo);
+  XITK_FREE(skonfig->animation);
   XITK_FREE(skonfig->path);
   XITK_FREE(skonfig->load_command);
   XITK_FREE(skonfig->unload_command);
@@ -829,6 +877,20 @@ int xitk_skin_check_version(xitk_skin_config_t *skonfig, int min_version) {
   return -1;
 }
 
+/*
+ *
+ */
+int xitk_skin_get_direction(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+  
+  assert(skonfig);
+  
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->direction;
+
+  return DIRECTION_LEFT; /* Compatibility */
+}
+
 /*
  *
  */
@@ -955,6 +1017,31 @@ int xitk_skin_get_label_animation(xitk_skin_config_t *skonfig, const char *str)
   return 0;
 }
 
+/*
+ *
+ */
+int xitk_skin_get_label_animation_step(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+
+  assert(skonfig);
+
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->animation_step;
+
+  return 1;
+}
+
+unsigned long xitk_skin_get_label_animation_timer(xitk_skin_config_t *skonfig, const char *str) {
+  xitk_skin_element_t *s;
+  
+  assert(skonfig);
+  
+  if((s = skin_lookup_section(skonfig, str)) != NULL)
+    return s->animation_timer;
+  
+  return 0;
+}
+
 /*
  *
  */
@@ -1008,7 +1095,7 @@ int xitk_skin_get_label_alignment(xitk_skin_config_t *skonfig, const char *str)
   if((s = skin_lookup_section(skonfig, str)) != NULL)
     return s->align;
   
-  return LABEL_ALIGN_CENTER;
+  return ALIGN_CENTER;
 }
 
 /*
@@ -1082,6 +1169,15 @@ int xitk_skin_get_slider_radius(xitk_skin_config_t *skonfig, const char *str) {
   return 0;
 }
 
+/*
+ *
+ */
+char *xitk_skin_get_animation(xitk_skin_config_t *skonfig) {
+  assert(skonfig);
+  
+  return skonfig->animation;
+}
+
 /*
  *
  */
