@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: slider.c,v 1.42 2002/07/17 20:29:32 f1rmb Exp $
+ * $Id: slider.c,v 1.58 2003/04/08 21:49:39 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -27,11 +27,6 @@
 #include <stdio.h>
 #include <math.h>
 
-#include "Imlib-light/Imlib.h"
-#include "image.h"
-#include "widget.h"
-#include "slider.h"
-#include "widget_types.h"
 #include "_xitk.h"
 
 #ifndef M_PI
@@ -209,10 +204,10 @@ static void slider_update(xitk_widget_t *w, int x, int y) {
 /*
  *
  */
-static void notify_destroy(xitk_widget_t *w, void *data) {
+static void notify_destroy(xitk_widget_t *w) {
   slider_private_data_t *private_data;
 
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     private_data = (slider_private_data_t *) w->private_data;
 
     XITK_FREE(private_data->skin_element_name);
@@ -228,7 +223,7 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
 static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     
     private_data = (slider_private_data_t *) w->private_data;
     
@@ -249,7 +244,7 @@ static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
 static int notify_inside(xitk_widget_t *w, int x, int y) {
   slider_private_data_t *private_data = (slider_private_data_t *) w->private_data;
 
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     
     private_data = (slider_private_data_t *) w->private_data;
     
@@ -261,7 +256,8 @@ static int notify_inside(xitk_widget_t *w, int x, int y) {
       else
 	skin = private_data->bg_skin;
 
-      return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, w, skin->mask, x, y);
+      if(skin->mask)
+	return xitk_is_cursor_out_mask(private_data->imlibdata->x.disp, w, skin->mask->pixmap, x, y);
     }
     else
       return 0;
@@ -273,14 +269,14 @@ static int notify_inside(xitk_widget_t *w, int x, int y) {
 /*
  * Draw widget
  */
-static void paint_slider(xitk_widget_t *w, Window win, GC gc) {
+static void paint_slider(xitk_widget_t *w) {
   int                     button_width, button_height;	
   slider_private_data_t  *private_data;
   GC                      bgc, pgc;
   xitk_image_t           *bg;
   xitk_image_t           *paddle;
   
-  if(w && (((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER) && (w->visible == 1))) {
+  if(w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER) && (w->visible == 1))) {
     int    x, y, srcx1, srcx2, destx1, srcy1, srcy2, desty1;
     int    xcenter, ycenter;
     int    paddle_width;
@@ -293,134 +289,147 @@ static void paint_slider(xitk_widget_t *w, Window win, GC gc) {
 
     x = y = srcx1 = srcx2 = destx1 = srcy1 = srcy2 = desty1 = 0;
         
-    XLOCK (private_data->imlibdata->x.disp);
-
-    bgc = XCreateGC(private_data->imlibdata->x.disp, bg->image, None, None);
-    XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, bgc);
-    pgc = XCreateGC(private_data->imlibdata->x.disp, paddle->image, None, None);
-    XCopyGC(private_data->imlibdata->x.disp, gc, (1 << GCLastBit) - 1, pgc);
+    XLOCK(private_data->imlibdata->x.disp);
+    bgc = XCreateGC(private_data->imlibdata->x.disp, bg->image->pixmap, None, None);
+    XCopyGC(private_data->imlibdata->x.disp, w->wl->gc, (1 << GCLastBit) - 1, bgc);
+    XUNLOCK(private_data->imlibdata->x.disp);
       
     if(bg->mask) {
+      XLOCK(private_data->imlibdata->x.disp);
       XSetClipOrigin(private_data->imlibdata->x.disp, bgc, w->x, w->y);
-      XSetClipMask(private_data->imlibdata->x.disp, bgc, bg->mask);
+      XSetClipMask(private_data->imlibdata->x.disp, bgc, bg->mask->pixmap);
+      XUNLOCK(private_data->imlibdata->x.disp);
     }
     
-    XCopyArea(private_data->imlibdata->x.disp, bg->image, win, bgc, 0, 0,
+    XLOCK(private_data->imlibdata->x.disp);
+    XCopyArea(private_data->imlibdata->x.disp, bg->image->pixmap, w->wl->win, bgc, 0, 0,
 	      bg->width, bg->height, w->x, w->y);
       
     XUNLOCK(private_data->imlibdata->x.disp);
 
-    if(private_data->sType == XITK_RSLIDER) {
-      
-      button_width = private_data->bg_skin->width;
-      button_height = private_data->bg_skin->height;
+    if(w->enable == WIDGET_ENABLE) {
 
-      xcenter       = (bg->width /2) + w->x;
-      ycenter       = (bg->width /2) + w->y;
-      paddle_width  = paddle->width / 3;
-      paddle_height = paddle->height;
-      angle         = private_data->angle;
-      
-      if(angle < M_PI / -2)
-	angle = angle + M_PI * 2;
-      
-      x = (int) (0.5 + xcenter + private_data->radius * cos(angle)) - (paddle_width / 2);
-      y = (int) (0.5 + ycenter - private_data->radius * sin(angle)) - (paddle_height / 2);
-
-      srcx1 = 0;
-      
-      if((private_data->focus == FOCUS_RECEIVED) || (private_data->focus == FOCUS_MOUSE_IN)) {
-	srcx1 += paddle_width;
-	if(private_data->bClicked)
+      XLOCK(private_data->imlibdata->x.disp);
+      pgc = XCreateGC(private_data->imlibdata->x.disp, paddle->image->pixmap, None, None);
+      XCopyGC(private_data->imlibdata->x.disp, w->wl->gc, (1 << GCLastBit) - 1, pgc);
+      XUNLOCK(private_data->imlibdata->x.disp);
+    
+      if(private_data->sType == XITK_RSLIDER) {
+	
+	button_width = private_data->bg_skin->width;
+	button_height = private_data->bg_skin->height;
+	
+	xcenter       = (bg->width /2) + w->x;
+	ycenter       = (bg->width /2) + w->y;
+	paddle_width  = paddle->width / 3;
+	paddle_height = paddle->height;
+	angle         = private_data->angle;
+	
+	if(angle < M_PI / -2)
+	  angle = angle + M_PI * 2;
+	
+	x = (int) (0.5 + xcenter + private_data->radius * cos(angle)) - (paddle_width / 2);
+	y = (int) (0.5 + ycenter - private_data->radius * sin(angle)) - (paddle_height / 2);
+	
+	srcx1 = 0;
+	
+	if((private_data->focus == FOCUS_RECEIVED) || (private_data->focus == FOCUS_MOUSE_IN)) {
 	  srcx1 += paddle_width;
-      }
-      
-      srcy1 = 0;
-      srcx2 = paddle_width;
-      srcy2 = paddle_height;
-      destx1 = x;
-      desty1 = y;
-
-    }
-    else {
-      int   dir_area;
-      float tmp;
-
-      button_width = private_data->button_width;
-      button_height = paddle->height;
-      
-      dir_area = (float)(private_data->sType == XITK_HSLIDER) ? 
-	w->width - button_width : w->height - button_height;
-	
-      tmp = (dir_area * .01) * (private_data->percentage * 100.0);
-      
-      if(private_data->sType == XITK_HSLIDER) {
-	x = rint(w->x + tmp);
-	y = w->y;
-      }
-      else if(private_data->sType == XITK_VSLIDER) {
-	x = w->x;
-	y = rint(w->y + private_data->bg_skin->height - button_height - tmp);
-      }
-      
-      if(private_data->paddle_cover_bg == 1) {
-	int pixpos;
-	int direction;
-	
-	x = w->x;
-	y = w->y;
-	
-	direction = (private_data->sType == XITK_HSLIDER) ? 
-	  private_data->bg_skin->width : private_data->bg_skin->height;
-	
-	pixpos = (int)private_data->value / 
-	  ((private_data->upper - private_data->lower) / direction);
-	
-	if(private_data->sType == XITK_VSLIDER) {
-	  pixpos = button_height - pixpos;
-	  srcx1  = 0;
-	  srcy1  = pixpos;
-	  srcx2  = button_width;
-	  srcy2  = paddle->height - pixpos;
-	  destx1 = w->x;
-	  desty1 = w->y + pixpos;
+	  if(private_data->bClicked)
+	    srcx1 += paddle_width;
 	}
-	else if(private_data->sType == XITK_HSLIDER) {
-	  srcx1  = 0;
-	  srcy1  = 0;
-	  srcx2  = pixpos;
-	  srcy2  = paddle->height;
-	  destx1 = w->x;
-	  desty1 = w->y;
-	}
-      }
-      else {
-	srcy1  = 0;
-	srcx2  = button_width;
-	srcy2  = paddle->height;
+	
+	srcy1 = 0;
+	srcx2 = paddle_width;
+	srcy2 = paddle_height;
 	destx1 = x;
 	desty1 = y;
+	
+      }
+      else {
+	int   dir_area;
+	float tmp;
+	
+	button_width = private_data->button_width;
+	button_height = paddle->height;
+	
+	dir_area = (float)(private_data->sType == XITK_HSLIDER) ? 
+	  w->width - button_width : w->height - button_height;
+	
+	tmp = (dir_area * .01) * (private_data->percentage * 100.0);
+	
+	if(private_data->sType == XITK_HSLIDER) {
+	  x = rint(w->x + tmp);
+	  y = w->y;
+	}
+	else if(private_data->sType == XITK_VSLIDER) {
+	  x = w->x;
+	  y = rint(w->y + private_data->bg_skin->height - button_height - tmp);
+	}
+	
+	if(private_data->paddle_cover_bg == 1) {
+	  int pixpos;
+	  int direction;
+	  
+	  x = w->x;
+	  y = w->y;
+	  
+	  direction = (private_data->sType == XITK_HSLIDER) ? 
+	    private_data->bg_skin->width : private_data->bg_skin->height;
+	  
+	  pixpos = (int)private_data->value / 
+	    ((private_data->upper - private_data->lower) / direction);
+	  
+	  if(private_data->sType == XITK_VSLIDER) {
+	    pixpos = button_height - pixpos;
+	    srcx1  = 0;
+	    srcy1  = pixpos;
+	    srcx2  = button_width;
+	    srcy2  = paddle->height - pixpos;
+	    destx1 = w->x;
+	    desty1 = w->y + pixpos;
+	  }
+	  else if(private_data->sType == XITK_HSLIDER) {
+	    srcx1  = 0;
+	    srcy1  = 0;
+	    srcx2  = pixpos;
+	    srcy2  = paddle->height;
+	    destx1 = w->x;
+	    desty1 = w->y;
+	  }
+	}
+	else {
+	  srcy1  = 0;
+	  srcx2  = button_width;
+	  srcy2  = paddle->height;
+	  destx1 = x;
+	  desty1 = y;
+	}
+	
+	if((private_data->focus == FOCUS_RECEIVED) || (private_data->focus == FOCUS_MOUSE_IN)) {
+	  if(private_data->bClicked)
+	    srcx1 = 2*button_width;
+	  else
+	    srcx1 = button_width;
+	}
+	
       }
       
-      if((private_data->focus == FOCUS_RECEIVED) || (private_data->focus == FOCUS_MOUSE_IN)) {
-	if(private_data->bClicked)
-	  srcx1 = 2*button_width;
-	else
-	  srcx1 = button_width;
+      if(paddle->mask) {
+	XLOCK(private_data->imlibdata->x.disp);
+	XSetClipOrigin(private_data->imlibdata->x.disp, pgc, x, y);
+	XSetClipMask(private_data->imlibdata->x.disp, pgc, paddle->mask->pixmap);
+	XUNLOCK(private_data->imlibdata->x.disp);
       }
-
+      
+      XLOCK(private_data->imlibdata->x.disp);
+      XCopyArea(private_data->imlibdata->x.disp, paddle->image->pixmap, w->wl->win, pgc,
+		srcx1, srcy1, srcx2, srcy2, destx1, desty1);
+      XFreeGC(private_data->imlibdata->x.disp, pgc);
+      XUNLOCK(private_data->imlibdata->x.disp);
     }
     
     XLOCK(private_data->imlibdata->x.disp);
-    if(paddle->mask) {
-      XSetClipOrigin(private_data->imlibdata->x.disp, pgc, x, y);
-      XSetClipMask(private_data->imlibdata->x.disp, pgc, paddle->mask);
-    }
-    
-    XCopyArea(private_data->imlibdata->x.disp, paddle->image, win, pgc,
-	      srcx1, srcy1, srcx2, srcy2, destx1, desty1);
-    
-    XFreeGC(private_data->imlibdata->x.disp, pgc);
     XFreeGC(private_data->imlibdata->x.disp, bgc);
     XUNLOCK(private_data->imlibdata->x.disp);
   }
@@ -430,11 +439,10 @@ static void paint_slider(xitk_widget_t *w, Window win, GC gc) {
 /*
  *
  */
-static void notify_change_skin(xitk_widget_list_t *wl, 
-			       xitk_widget_t *w, xitk_skin_config_t *skonfig) {
+static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     
     private_data = (slider_private_data_t *) w->private_data;
     
@@ -442,10 +450,10 @@ static void notify_change_skin(xitk_widget_list_t *wl,
 
       xitk_skin_lock(skonfig);
 
-      XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->paddle_skin);
+      xitk_image_free_image(private_data->imlibdata, &private_data->paddle_skin);
       private_data->paddle_skin     = xitk_image_load_image(private_data->imlibdata, xitk_skin_get_slider_skin_filename(skonfig, private_data->skin_element_name));
       private_data->button_width    = private_data->paddle_skin->width / 3;
-      XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->bg_skin);
+      xitk_image_free_image(private_data->imlibdata, &private_data->bg_skin);
       private_data->bg_skin         = xitk_image_load_image(private_data->imlibdata, xitk_skin_get_skin_filename(skonfig, private_data->skin_element_name));
       private_data->sType = xitk_skin_get_slider_type(skonfig, private_data->skin_element_name);
       private_data->paddle_cover_bg = 0;
@@ -478,69 +486,45 @@ static void notify_change_skin(xitk_widget_list_t *wl,
 /*
  * Got click
  */
-static int notify_click_slider(xitk_widget_list_t *wl, xitk_widget_t *w, int bUp, int x, int y) {
+static int notify_click_slider(xitk_widget_t *w, int button, int bUp, int x, int y) {
   slider_private_data_t *private_data;
-    
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
-    
-    private_data = (slider_private_data_t *) w->private_data;    
-    
-    slider_update(w, (x - w->x), (y - w->y));
-    
-    private_data->bClicked = !bUp;
-    
-    paint_slider(w, wl->win, wl->gc);
-    
-    if((bUp == 0) && (private_data->focus == FOCUS_RECEIVED)) {
-      XEvent sliderevent;
-      int new_value = (int) private_data->value;
-      int old_value = !new_value;
+  int                    ret = 0;
+
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+    if(button == Button1) {
+      private_data = (slider_private_data_t *) w->private_data;    
       
-      /*
-       * Exec motion callback function (if available)
-       */
-
-      if(private_data->motion_callback) {
-	private_data->motion_callback(private_data->sWidget,
-				      private_data->motion_userdata,
-				      (int) private_data->value);
-      }
-
-      /*
-       * Loop of death ;-)
-       */
-      do {
-
-	XNextEvent(private_data->imlibdata->x.disp, &sliderevent) ;
-
-	switch(sliderevent.type) {
+      if(private_data->focus == FOCUS_RECEIVED) {
+	int old_value = (int) private_data->value;
+	
+	slider_update(w, (x - w->x), (y - w->y));
+	
+	if(private_data->bClicked == bUp)
+	  private_data->bClicked = !bUp;
+	
+	if(bUp == 0) {
 	  
-	case MotionNotify: {
-
-	  while(XCheckMaskEvent (private_data->imlibdata->x.disp, ButtonMotionMask,
-				 &sliderevent));
-
-	  slider_update(w, (sliderevent.xbutton.x - w->x), (sliderevent.xbutton.y - w->y));	
-	  new_value = (int) private_data->value;
-
-	  if(new_value != old_value) {
-	    old_value = new_value;
-	    paint_slider(w, wl->win, wl->gc);
+	  if(old_value != ((int) private_data->value)) {
+	    slider_update_value(w, private_data->value);
+	    paint_slider(w);
 	  }
 	  
- 	  if(private_data->motion_callback) {
-	    private_data->motion_callback(private_data->sWidget,
-					  private_data->motion_userdata,
-					  (int) private_data->value);
+	  /*
+	   * Exec motion callback function (if available)
+	   */
+	  if(old_value != ((int) private_data->value)) {
+	    if(private_data->motion_callback)
+	      private_data->motion_callback(private_data->sWidget,
+					    private_data->motion_userdata,
+					    (int) private_data->value);
 	  }
-
+	  
 	}
-	break;
-
-	case ButtonRelease: {
+	else if(bUp == 1) {
 	  private_data->bClicked = 0;
 	  
-	  xitk_slider_set_pos(wl, w, private_data->value);
+	  slider_update_value(w, private_data->value);
+	  paint_slider(w);
 	  
 	  if(private_data->callback) {
 	    private_data->callback(private_data->sWidget,
@@ -548,46 +532,76 @@ static int notify_click_slider(xitk_widget_list_t *wl, xitk_widget_t *w, int bUp
 				   (int) private_data->value);
 	  }
 	}
-	break;
-
-	default:
-	  xitk_xevent_notify (&sliderevent);
- 	  break;
-	}
-	
-      } while(sliderevent.type != ButtonRelease); 
+      }
+      ret = 1;
     }
-
   }
 
-  return 1;
+  return ret;
 }
 
 /*
  * Got focus
  */
-static int notify_focus_slider(xitk_widget_list_t *wl, xitk_widget_t *w, int focus) {
+static int notify_focus_slider(xitk_widget_t *w, int focus) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     private_data = (slider_private_data_t *) w->private_data;
-    private_data->focus = focus;
+    if(!private_data->bClicked)
+      private_data->focus = focus;
   }
 
   return 1;
 }
 
+static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_result_t *result) {
+  int retval = 0;
+
+  switch(event->type) {
+  case WIDGET_EVENT_PAINT:
+    paint_slider(w);
+    break;
+  case WIDGET_EVENT_CLICK:
+    result->value = notify_click_slider(w, event->button, 
+					event->button_pressed, event->x, event->y);
+    retval = 1;
+    break;
+  case WIDGET_EVENT_FOCUS:
+    notify_focus_slider(w, event->focus);
+    break;
+  case WIDGET_EVENT_INSIDE:
+    result->value = notify_inside(w, event->x, event->y);
+    retval = 1;
+    break;
+  case WIDGET_EVENT_CHANGE_SKIN:
+    notify_change_skin(w, event->skonfig);
+    break;
+  case WIDGET_EVENT_DESTROY:
+    notify_destroy(w);
+    break;
+  case WIDGET_EVENT_GET_SKIN:
+    if(result) {
+      result->image = get_skin(w, event->skin_layer);
+      retval = 1;
+    }
+    break;
+  }
+  
+  return retval;
+}
+
 /*
  * Increment position
  */
-void xitk_slider_make_step(xitk_widget_list_t *wl, xitk_widget_t *w) {
+void xitk_slider_make_step(xitk_widget_t *w) {
   slider_private_data_t *private_data;
 
-  if (w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     private_data = (slider_private_data_t *) w->private_data;
     if(!private_data->bClicked) {
       if(((xitk_slider_get_pos(w)) + private_data->step) <= (xitk_slider_get_max(w)))
-	xitk_slider_set_pos(wl, w, xitk_slider_get_pos(w) + private_data->step);
+	xitk_slider_set_pos(w, xitk_slider_get_pos(w) + private_data->step);
     }
   } 
 
@@ -596,14 +610,14 @@ void xitk_slider_make_step(xitk_widget_list_t *wl, xitk_widget_t *w) {
 /*
  * Decrement position
  */
-void xitk_slider_make_backstep(xitk_widget_list_t *wl, xitk_widget_t *w) {
+void xitk_slider_make_backstep(xitk_widget_t *w) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     private_data = (slider_private_data_t *) w->private_data;
     if(!private_data->bClicked) {
       if(((xitk_slider_get_pos(w)) - private_data->step) >= (xitk_slider_get_min(w)))
-	xitk_slider_set_pos(wl, w, xitk_slider_get_pos(w) - private_data->step);
+	xitk_slider_set_pos(w, xitk_slider_get_pos(w) - private_data->step);
     }
     
   } 
@@ -616,7 +630,7 @@ void xitk_slider_make_backstep(xitk_widget_list_t *wl, xitk_widget_t *w) {
 void xitk_slider_set_min(xitk_widget_t *w, int min) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     private_data = (slider_private_data_t *) w->private_data;
     slider_update_minmax(w, (float)((min == private_data->upper) 
 				    ? min - 1 : min), private_data->upper);
@@ -629,7 +643,7 @@ void xitk_slider_set_min(xitk_widget_t *w, int min) {
 int xitk_slider_get_min(xitk_widget_t *w) {
   slider_private_data_t *private_data;
   
-  if((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER) {
+  if((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER) {
     private_data = (slider_private_data_t *) w->private_data;
     return (int) private_data->lower;
   } 
@@ -643,7 +657,7 @@ int xitk_slider_get_min(xitk_widget_t *w) {
 int xitk_slider_get_max(xitk_widget_t *w) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     private_data = (slider_private_data_t *) w->private_data;
     return (int) private_data->upper;
   } 
@@ -657,7 +671,7 @@ int xitk_slider_get_max(xitk_widget_t *w) {
 void xitk_slider_set_max(xitk_widget_t *w, int max) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     private_data = (slider_private_data_t *) w->private_data;
     slider_update_minmax(w, private_data->lower, (float)((max == private_data->lower) 
 							 ? max + 1 : max));
@@ -667,28 +681,28 @@ void xitk_slider_set_max(xitk_widget_t *w, int max) {
 /*
  * Set pos to 0 and redraw the widget.
  */
-void xitk_slider_reset(xitk_widget_list_t *wl, xitk_widget_t *w) {
+void xitk_slider_reset(xitk_widget_t *w) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     private_data = (slider_private_data_t *) w->private_data;
     slider_update_value(w, 0.0);
     private_data->bClicked = 0;
-    paint_slider(w, wl->win, wl->gc);
+    paint_slider(w);
   }
 }
 
 /*
  * Set pos to max and redraw the widget.
  */
-void xitk_slider_set_to_max(xitk_widget_list_t *wl, xitk_widget_t *w) {
+void xitk_slider_set_to_max(xitk_widget_t *w) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     private_data = (slider_private_data_t *) w->private_data;
     if(!private_data->bClicked) {
       slider_update_value(w, private_data->upper);
-      paint_slider(w, wl->win, wl->gc);
+      paint_slider(w);
     }
   }
 }
@@ -699,7 +713,7 @@ void xitk_slider_set_to_max(xitk_widget_list_t *wl, xitk_widget_t *w) {
 int xitk_slider_get_pos(xitk_widget_t *w) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     private_data = (slider_private_data_t *) w->private_data;
     return (int) private_data->value;
   } 
@@ -710,10 +724,10 @@ int xitk_slider_get_pos(xitk_widget_t *w) {
 /*
  * Set position.
  */
-void xitk_slider_set_pos(xitk_widget_list_t *wl, xitk_widget_t *w, int pos) {
+void xitk_slider_set_pos(xitk_widget_t *w, int pos) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     float value = (float) pos;
     
     private_data = (slider_private_data_t *) w->private_data;
@@ -721,7 +735,7 @@ void xitk_slider_set_pos(xitk_widget_list_t *wl, xitk_widget_t *w, int pos) {
     if(!private_data->bClicked) {
       if(value >= private_data->lower && value <= private_data->upper) {
 	slider_update_value(w, value);
-	paint_slider(w, wl->win, wl->gc);
+	paint_slider(w);
       }
     }
     
@@ -734,7 +748,7 @@ void xitk_slider_set_pos(xitk_widget_list_t *wl, xitk_widget_t *w, int pos) {
 void xitk_slider_callback_exec(xitk_widget_t *w) {
   slider_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
     private_data = (slider_private_data_t *) w->private_data;
 
     if(private_data->callback) {
@@ -810,7 +824,7 @@ static xitk_widget_t *_xitk_slider_create(xitk_widget_list_t *wl,
 
   mywidget->private_data                 = private_data;
 
-  mywidget->widget_list                  = wl;
+  mywidget->wl                           = wl;
 
   mywidget->enable                       = enable;
   mywidget->running                      = 1;
@@ -821,16 +835,8 @@ static xitk_widget_t *_xitk_slider_create(xitk_widget_list_t *wl,
   mywidget->y                            = y;
   mywidget->width                        = private_data->bg_skin->width;
   mywidget->height                       = private_data->bg_skin->height;
-  mywidget->widget_type                  = WIDGET_TYPE_SLIDER;
-  mywidget->paint                        = paint_slider;
-  mywidget->notify_click                 = notify_click_slider;
-  mywidget->notify_focus                 = notify_focus_slider;
-  mywidget->notify_keyevent              = NULL;
-  mywidget->notify_inside                = notify_inside;
-  mywidget->notify_change_skin           = (skin_element_name == NULL) ? NULL : notify_change_skin;
-  mywidget->notify_destroy               = notify_destroy;
-  mywidget->get_skin                     = get_skin;
-
+  mywidget->type                         = WIDGET_TYPE_SLIDER | WIDGET_FOCUSABLE | WIDGET_CLICKABLE | WIDGET_KEYABLE;
+  mywidget->event                        = notify_event;
   mywidget->tips_timeout                 = 0;
   mywidget->tips_string                  = NULL;
 
