@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: widget.c,v 1.30 2002/07/17 20:29:32 f1rmb Exp $
+ * $Id: widget.c,v 1.53 2003/03/27 20:52:09 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -35,13 +35,7 @@
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
 
-#include "widget.h"
-#include "list.h"
 #include "_xitk.h"
-#include "combo.h"
-#include "tips.h"
-
-extern int errno;
 
 static xitk_color_names_t xitk_color_names[] = {
   { 255,  250,  250,  "snow" },
@@ -704,31 +698,33 @@ static xitk_color_names_t xitk_color_names[] = {
 
 #if 0
 void dump_widget_type(xitk_widget_t *w) {
-  if(w->widget_type & WIDGET_GROUP) {
+  if(w->type & WIDGET_GROUP) {
     printf("WIDGET_TYPE_GROUP: ");
-    if(w->widget_type & WIDGET_GROUP_WIDGET)
+    if(w->type & WIDGET_GROUP_WIDGET)
       printf("[THE_WIDGET] ");
-    if((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER)
+    if((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER)
       printf("WIDGET_TYPE_BROWSER | ");
-    if((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_FILEBROWSER)
+    if((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_FILEBROWSER)
       printf("WIDGET_TYPE_FILEBROWSER | ");
-    if((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER)
+    if((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MRLBROWSER)
       printf("WIDGET_TYPE_MRLBROWSER | ");
-    if((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO)
+    if((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO)
       printf("WIDGET_TYPE_COMBO | ");
-    if((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_TABS)
+    if((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_TABS)
       printf("WIDGET_TYPE_TABS | ");
-    if((w->widget_type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX)
+    if((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX)
       printf("WIDGET_TYPE_INTBOX | ");
+    if((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_MENU)
+      printf("WIDGET_TYPE_MENU | ");
   }
 
-  if((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_BUTTON)      printf("WIDGET_TYPE_BUTTON ");
-  if((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON) printf("WIDGET_TYPE_LABELBUTTON ");
-  if((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)      printf("WIDGET_TYPE_SLIDER ");
-  if((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)       printf("WIDGET_TYPE_LABEL ");
-  if((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)    printf("WIDGET_TYPE_CHECKBOX ");
-  if((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_IMAGE)       printf("WIDGET_TYPE_IMAGE ");
-  if((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)   printf("WIDGET_TYPE_INPUTTEXT ");
+  if((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_BUTTON)      printf("WIDGET_TYPE_BUTTON ");
+  if((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABELBUTTON) printf("WIDGET_TYPE_LABELBUTTON ");
+  if((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)      printf("WIDGET_TYPE_SLIDER ");
+  if((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)       printf("WIDGET_TYPE_LABEL ");
+  if((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)    printf("WIDGET_TYPE_CHECKBOX ");
+  if((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_IMAGE)       printf("WIDGET_TYPE_IMAGE ");
+  if((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT)   printf("WIDGET_TYPE_INPUTTEXT ");
   printf("\n");
 }
 #endif
@@ -743,13 +739,11 @@ void *xitk_xmalloc(size_t size) {
   if(!size)
     size++;
   
-  if((ptrmalloc = malloc(size)) == NULL) {
-    XITK_WARNING("%s: malloc() failed: %s.\n", __FUNCTION__, strerror(errno));
+  if((ptrmalloc = calloc(1, size)) == NULL) {
+    XITK_WARNING("calloc() failed: %s.\n", strerror(errno));
     return NULL;
   }
 
-  memset(ptrmalloc, 0, size);
-
   return ptrmalloc;
 }
 
@@ -758,18 +752,22 @@ void *xitk_xmalloc(size_t size) {
  * the specifier widget list.
  */
 void xitk_change_skins_widget_list(xitk_widget_list_t *wl, xitk_skin_config_t *skonfig) {
-  xitk_widget_t *mywidget;
+  xitk_widget_t   *mywidget;
+  widget_event_t   event;
 
   if(!wl) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return;
   }
+
+  event.type    = WIDGET_EVENT_CHANGE_SKIN;
+  event.skonfig = skonfig;
+
   mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
   while (mywidget && wl && wl->l && wl->win && wl->gc && skonfig) {
 
-    if(mywidget->notify_change_skin)
-      (mywidget->notify_change_skin) (wl, mywidget, skonfig);
-    
+    (void) mywidget->event(mywidget, &event, NULL);
+
     mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l); 
   }
 }
@@ -778,22 +776,26 @@ void xitk_change_skins_widget_list(xitk_widget_list_t *wl, xitk_skin_config_t *s
  * (re)paint the specified widget list.
  */
 int xitk_paint_widget_list (xitk_widget_list_t *wl) {
-  xitk_widget_t *mywidget;
+  xitk_widget_t   *mywidget;
+  widget_event_t   event;
 
   mywidget = (xitk_widget_t *) xitk_list_first_content (wl->l);
+
   while (mywidget && wl && wl->l && wl->win && wl->gc) {
     
     if((mywidget->enable != WIDGET_ENABLE) && (mywidget->have_focus != FOCUS_LOST)) {
-      if(mywidget->notify_focus)
-	(void) (mywidget->notify_focus)(wl, mywidget, FOCUS_LOST);
+
+      event.type = WIDGET_EVENT_FOCUS;
+      event.focus = FOCUS_LOST;
+      (void) mywidget->event(mywidget, &event, NULL);
       
-      if(mywidget->paint)
-	(mywidget->paint) (mywidget, wl->win, wl->gc);
+      event.type = WIDGET_EVENT_PAINT;
+      (void) mywidget->event(mywidget, &event, NULL);
       
     }
     else {
-      if(mywidget->paint)
-	(mywidget->paint) (mywidget, wl->win, wl->gc);
+      event.type = WIDGET_EVENT_PAINT;
+      (void) mywidget->event(mywidget, &event, NULL);
     }
     
     mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l); 
@@ -809,11 +811,11 @@ int xitk_is_cursor_out_mask(Display *display, xitk_widget_t *w, Pixmap mask, int
   Pixel p;
   
   if(!display) {
-    XITK_WARNING("%s(): display was unset.\n", __FUNCTION__);
+    XITK_WARNING("display was unset.\n");
     return 0;
   }
   if(!w) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return 0;
   }
 
@@ -824,11 +826,9 @@ int xitk_is_cursor_out_mask(Display *display, xitk_widget_t *w, Pixmap mask, int
     if((yy = (y - w->y)) == w->height) yy--;
     
     XLOCK(display);
-    
     xi = XGetImage(display, mask, xx, yy, 1, 1, AllPlanes, ZPixmap);
     p = XGetPixel(xi, 0, 0);
     XDestroyImage(xi);
-    
     XUNLOCK(display);
 
     return (int) p;
@@ -844,18 +844,22 @@ int xitk_is_inside_widget (xitk_widget_t *widget, int x, int y) {
   int inside = 0;
   
   if(!widget) {
-    XITK_WARNING("%s(): widget was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget was NULL.\n");
     return 0;
   }
 
   if(((x >= widget->x) && (x <= (widget->x + widget->width))) && 
      ((y >= widget->y) && (y <= (widget->y + widget->height)))) {
-    
-    inside = 1;
-    
-    if(widget->notify_inside) 
-      inside = widget->notify_inside(widget, x, y);
+    widget_event_t        event;
+    widget_event_result_t result;
 
+    inside = 1;
+
+    event.type = WIDGET_EVENT_INSIDE;
+    event.x    = x;
+    event.y    = y;
+    if(widget->event(widget, &event, &result))
+      inside = result.value;
   }
 
   return inside;
@@ -868,7 +872,7 @@ xitk_widget_t *xitk_get_widget_at (xitk_widget_list_t *wl, int x, int y) {
   xitk_widget_t *mywidget;
 
   if(!wl) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return 0;
   }
 
@@ -887,14 +891,33 @@ xitk_widget_t *xitk_get_widget_at (xitk_widget_list_t *wl, int x, int y) {
  * Call notify_focus (with FOCUS_MOUSE_[IN|OUT] as focus state), 
  * function in right widget (the one who get, and the one who lose focus).
  */
-void xitk_motion_notify_widget_list (xitk_widget_list_t *wl, int x, int y) { 
+void xitk_motion_notify_widget_list(xitk_widget_list_t *wl, int x, int y, unsigned int state) { 
   xitk_widget_t *mywidget;
+  widget_event_t event;
   
   if(!wl) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return;
   }
-  
+ 
+  /* Slider still pressed, mouse pointer outside widget */
+  if(wl->widget_pressed && wl->widget_focused && (wl->widget_pressed == wl->widget_focused) &&
+     ((wl->widget_pressed->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_SLIDER)) {
+    widget_event_result_t   result;
+    
+    //    printf("slider already clicked -> send event\n");
+
+    if(state & Button1Mask) {
+      event.type           = WIDGET_EVENT_CLICK;
+      event.x              = x;
+      event.y              = y;
+      event.button_pressed = LBUTTON_DOWN;
+      event.button         = Button1;
+      (void) wl->widget_focused->event(wl->widget_focused, &event, &result);
+    }
+    return;
+  }
+
   mywidget = xitk_get_widget_at (wl, x, y);
   
   if (mywidget != wl->widget_under_mouse) {
@@ -904,13 +927,16 @@ void xitk_motion_notify_widget_list (xitk_widget_list_t *wl, int x, int y) {
       /* Kill (hide) tips */
       xitk_tips_tips_kill(wl->widget_under_mouse);
       
-      if (wl->widget_under_mouse->notify_focus && 
-	  wl->widget_under_mouse->enable == WIDGET_ENABLE) {
-	(void) (wl->widget_under_mouse->notify_focus)(wl, wl->widget_under_mouse, FOCUS_MOUSE_OUT);
+      if((wl->widget_under_mouse->type & WIDGET_FOCUSABLE) && 
+	 wl->widget_under_mouse->enable == WIDGET_ENABLE) {
+	
+	event.type  = WIDGET_EVENT_FOCUS;
+	event.focus = FOCUS_MOUSE_OUT;
+	(void) wl->widget_under_mouse->event(wl->widget_under_mouse, &event, NULL);
+
+      event.type = WIDGET_EVENT_PAINT;
+      (void) wl->widget_under_mouse->event(wl->widget_under_mouse, &event, NULL);
       }
-      
-      if(wl->widget_under_mouse->paint)
-	(wl->widget_under_mouse->paint) (wl->widget_under_mouse, wl->win, wl->gc);
     }
     
     wl->widget_under_mouse = mywidget;
@@ -921,15 +947,31 @@ void xitk_motion_notify_widget_list (xitk_widget_list_t *wl, int x, int y) {
       dump_widget_type(mywidget);
 #endif
       
-      xitk_tips_create(mywidget, wl);
+      xitk_tips_create(mywidget);
       
-      if (mywidget->notify_focus && mywidget->enable == WIDGET_ENABLE)
-	(void) (mywidget->notify_focus) (wl, mywidget, FOCUS_MOUSE_IN);
+      if((mywidget->type & WIDGET_FOCUSABLE) && mywidget->enable == WIDGET_ENABLE) {
+	event.type  = WIDGET_EVENT_FOCUS;
+	event.focus = FOCUS_MOUSE_IN;
+	(void) mywidget->event(mywidget, &event, NULL);
+
+	event.type  = WIDGET_EVENT_PAINT;
+	(void) mywidget->event(mywidget, &event, NULL);
+      }
+    }
+  }
+  else {
+    if(mywidget && (mywidget->type & WIDGET_TYPE_SLIDER)) {
+      widget_event_result_t   result;
       
-      if(mywidget->paint)
-	  (mywidget->paint) (mywidget, wl->win, wl->gc);
+      if(state & Button1Mask) {
+	event.type           = WIDGET_EVENT_CLICK;
+	event.x              = x;
+	event.y              = y;
+	event.button_pressed = LBUTTON_DOWN;
+	event.button         = Button1;
+	(void) mywidget->event(mywidget, &event, &result);
+      }
     }
-    
   }
 }
 
@@ -937,15 +979,17 @@ void xitk_motion_notify_widget_list (xitk_widget_list_t *wl, int x, int y) {
  * Call notify_focus (with FOCUS_[RECEIVED|LOST] as focus state), 
  * then call notify_click function, if exist, of widget at coords x/y.
  */
-int xitk_click_notify_widget_list (xitk_widget_list_t *wl, int x, int y, int bUp) {
-  int            bRepaint = 0;
-  xitk_widget_t  *mywidget;
+int xitk_click_notify_widget_list (xitk_widget_list_t *wl, int x, int y, int button, int bUp) {
+  int                    bRepaint = 0;
+  xitk_widget_t         *mywidget, *menu = NULL;
+  widget_event_t         event;
+  widget_event_result_t  result;
 
   if(!wl) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return 0;
   }
-
+  
   mywidget = xitk_get_widget_at (wl, x, y);
 
   if(mywidget != wl->widget_focused) {
@@ -954,37 +998,66 @@ int xitk_click_notify_widget_list (xitk_widget_list_t *wl, int x, int y, int bUp
       /* Kill (hide) tips */
       xitk_tips_tips_kill(wl->widget_focused);
       
-      if(wl->widget_focused->notify_focus && wl->widget_focused->enable == WIDGET_ENABLE) {
-	(void) (wl->widget_focused->notify_focus) (wl, wl->widget_focused, FOCUS_LOST);
-	wl->widget_focused->have_focus = FOCUS_LOST;
-      }
+      if((wl->widget_focused->type & WIDGET_FOCUSABLE) &&
+	 wl->widget_focused->enable == WIDGET_ENABLE) {
+	
+	if(wl->widget_focused && (wl->widget_focused->type & WIDGET_GROUP_MENU))
+	  menu = xitk_menu_get_menu(wl->widget_focused);
 
-      if(wl->widget_focused->paint)
-	(wl->widget_focused->paint) (wl->widget_focused, wl->win, wl->gc);
+	if((wl->widget_focused->type & WIDGET_GROUP_COMBO)) {
+	  if(((wl->widget_focused->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)
+	     || ((wl->widget_focused->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
+
+	    if((!(mywidget && 
+		  ((((mywidget->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL) || 
+		    ((mywidget->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_CHECKBOX)) &&
+		   (xitk_combo_is_same_parent(wl->widget_focused, mywidget) == 1)))) ||
+	       (!mywidget)) {
+
+	      if(xitk_checkbox_get_state(wl->widget_focused))
+		xitk_combo_rollunroll(wl->widget_focused);
+	      
+	    }
+	  }
+	}
+	else {
+	  event.type  = WIDGET_EVENT_FOCUS;
+	  event.focus = FOCUS_LOST;
+	  (void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
+	  wl->widget_focused->have_focus = FOCUS_LOST;
+	}
+      }
+      
+      event.type = WIDGET_EVENT_PAINT;
+      (void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
     }
     
-
+    
     wl->widget_focused = mywidget;
     
     if (mywidget) {
       
-      if (mywidget->notify_focus && mywidget->enable == WIDGET_ENABLE) {
-	(void) (mywidget->notify_focus) (wl, mywidget, FOCUS_RECEIVED);
+      if ((mywidget->type & WIDGET_FOCUSABLE) && mywidget->enable == WIDGET_ENABLE) {
+	event.type = WIDGET_EVENT_FOCUS;
+	event.focus = FOCUS_RECEIVED;
+	(void) mywidget->event(mywidget, &event, NULL);
 	mywidget->have_focus = FOCUS_RECEIVED;
       }
       else
 	wl->widget_focused = NULL;
       
-      if(mywidget->paint)
-	(mywidget->paint) (mywidget, wl->win, wl->gc);
+      event.type = WIDGET_EVENT_PAINT;
+      (void) mywidget->event(mywidget, &event, NULL);
     }
     
   }
   else {
     if(wl->widget_under_mouse && mywidget && (wl->widget_under_mouse == mywidget)) {
       
-      if (mywidget->notify_focus && mywidget->enable == WIDGET_ENABLE) {
-	(void) (mywidget->notify_focus) (wl, mywidget, FOCUS_RECEIVED);
+      if ((mywidget->type & WIDGET_FOCUSABLE) && mywidget->enable == WIDGET_ENABLE) {
+	event.type  = WIDGET_EVENT_FOCUS;
+	event.focus = FOCUS_RECEIVED;
+	(void) mywidget->event(mywidget, &event, NULL);
 	mywidget->have_focus = FOCUS_RECEIVED;
       }
     }
@@ -992,28 +1065,47 @@ int xitk_click_notify_widget_list (xitk_widget_list_t *wl, int x, int y, int bUp
   }
   
   if (!bUp) {
-    
+
     wl->widget_pressed = mywidget;
     
     if (mywidget) {
-      
+      widget_event_result_t result;
+
       /* Kill (hide) tips */
       xitk_tips_tips_kill(mywidget);
       
-      if (mywidget->notify_click && mywidget->enable == WIDGET_ENABLE && mywidget->running)
-	bRepaint |= (mywidget->notify_click) (wl, mywidget, LBUTTON_DOWN, x, y);
-      
+      if((mywidget->type & WIDGET_CLICKABLE) && 
+	 mywidget->enable == WIDGET_ENABLE && mywidget->running) {
+	event.type           = WIDGET_EVENT_CLICK;
+	event.x              = x;
+	event.y              = y;
+	event.button_pressed = LBUTTON_DOWN;
+	event.button         = button;
+
+	if(mywidget->event(mywidget, &event, &result))
+	  bRepaint |= result.value;
+      }      
     }
-  } 
+  }
   else {
-    if (wl->widget_pressed) {
-      if (wl->widget_pressed->notify_click 
-	  && wl->widget_pressed->enable == WIDGET_ENABLE && wl->widget_pressed->running)
-	bRepaint |= (wl->widget_pressed->notify_click) (wl, wl->widget_pressed, LBUTTON_UP, x, y);
-      
+    if(wl->widget_pressed) {
+      if((wl->widget_pressed->type & WIDGET_CLICKABLE) &&
+	 (wl->widget_pressed->enable == WIDGET_ENABLE) && wl->widget_pressed->running) {
+	event.type           = WIDGET_EVENT_CLICK;
+	event.x              = x;
+	event.y              = y;
+	event.button_pressed = LBUTTON_UP;
+	event.button         = button;
+
+	if(wl->widget_pressed->event(wl->widget_pressed, &event, &result))
+	  bRepaint |= result.value;
+      }
     }
   }
   
+  if((wl->widget_focused == NULL) && menu)
+    wl->widget_focused = menu;
+
   return((bRepaint == 1));
 }
 
@@ -1025,7 +1117,7 @@ static xitk_widget_t *get_next_focusable_widget(xitk_widget_list_t *wl, int back
   int found;
   
   if(!wl) {
-    XITK_WARNING("%s(): widget list is NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list is NULL.\n");
     return NULL;
   }
   
@@ -1054,7 +1146,7 @@ static xitk_widget_t *get_next_focusable_widget(xitk_widget_list_t *wl, int back
 	  else
 	    widget = (xitk_widget_t *) xitk_list_first_content (wl->l);
 	  
-	  if(widget && (widget->notify_focus) && (widget->enable) && (widget->visible))
+	  if(widget && (widget->type & WIDGET_FOCUSABLE) && (widget->enable) && (widget->visible))
 	    found = 1;
 	  
 	} while(!found);
@@ -1062,11 +1154,11 @@ static xitk_widget_t *get_next_focusable_widget(xitk_widget_list_t *wl, int back
 	return widget;
       }
       else
-	if((widget->notify_focus) && (widget->enable) && (widget->visible))
+	if((widget->type & WIDGET_FOCUSABLE) && (widget->enable) && (widget->visible))
 	  found = 1;
       
     }
-    else if((widget->notify_focus) && (widget->enable) && (widget->visible))
+    else if((widget->type & WIDGET_FOCUSABLE) && (widget->enable) && (widget->visible))
       found = 1;
     
   } while(!found);
@@ -1078,10 +1170,11 @@ static xitk_widget_t *get_next_focusable_widget(xitk_widget_list_t *wl, int back
  *
  */
 void xitk_set_focus_to_next_widget(xitk_widget_list_t *wl, int backward) {
-  xitk_widget_t *first_widget, *widget;
+  xitk_widget_t   *first_widget, *widget;
+  widget_event_t   event;
 
   if(!wl) {
-    XITK_WARNING("%s(): widget list is NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list is NULL.\n");
     return;
   }
 
@@ -1095,7 +1188,7 @@ void xitk_set_focus_to_next_widget(xitk_widget_list_t *wl, int backward) {
     /* There is no widget focused yet */
     if(!wl->widget_focused) {
       
-      if((!first_widget->notify_focus) || (!first_widget->enable) || (!first_widget->visible)) {
+      if((!(first_widget->type & WIDGET_FOCUSABLE)) || (!first_widget->enable) || (!first_widget->visible)) {
 	first_widget = get_next_focusable_widget(wl, backward);
       }
       
@@ -1106,26 +1199,32 @@ void xitk_set_focus_to_next_widget(xitk_widget_list_t *wl, int backward) {
       /* Kill (hide) tips */
       xitk_tips_tips_kill(wl->widget_focused);
 
-      if (wl->widget_focused->notify_focus && wl->widget_focused->enable == WIDGET_ENABLE) {
-
-	(void) (wl->widget_focused->notify_focus) (wl, wl->widget_focused, FOCUS_RECEIVED);
+      if ((wl->widget_focused->type & WIDGET_FOCUSABLE) &&
+	  (wl->widget_focused->enable == WIDGET_ENABLE)) {
+	event.type  = WIDGET_EVENT_FOCUS;
+	event.focus = FOCUS_RECEIVED;
+	(void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
 	wl->widget_focused->have_focus = FOCUS_RECEIVED;
 	
-	if(wl->widget_focused->paint)
-	  (wl->widget_focused->paint) (wl->widget_focused, wl->win, wl->gc);
+	event.type = WIDGET_EVENT_PAINT;
+	(void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
       }
       
-      if (wl->widget_focused->notify_click
-      	  && wl->widget_focused->enable == WIDGET_ENABLE && wl->widget_focused->running) {
+      if (wl->widget_focused->enable == WIDGET_ENABLE && wl->widget_focused->running) {
       	
-      	if((wl->widget_focused->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT) {
-      	  (void) (wl->widget_focused->notify_click) (wl, wl->widget_focused, LBUTTON_DOWN, 
-      						     wl->widget_focused->x + 1, 
-      						     wl->widget_focused->y + 1);
-	  
-      	  if(wl->widget_focused->paint)
-      	    (wl->widget_focused->paint) (wl->widget_focused, wl->win, wl->gc);
-	  
+      	if((wl->widget_focused->type & WIDGET_CLICKABLE) &&
+	   (wl->widget_focused->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT) {
+	  widget_event_result_t  result;
+
+	  event.type           = WIDGET_EVENT_CLICK;
+	  event.x              = wl->widget_focused->x + 1;
+	  event.y              = wl->widget_focused->y + 1;
+	  event.button_pressed = LBUTTON_DOWN;
+	  event.button         = AnyButton;
+      	  (void) wl->widget_focused->event(wl->widget_focused, &event, &result);
+
+	  event.type = WIDGET_EVENT_PAINT;
+	  (void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
       	}
 	
       }
@@ -1136,13 +1235,17 @@ void xitk_set_focus_to_next_widget(xitk_widget_list_t *wl, int backward) {
     if(widget == wl->widget_focused) {
       xitk_widget_t *next_widget;
       
-      if (wl->widget_focused->notify_focus && wl->widget_focused->enable == WIDGET_ENABLE) {
+      if ((wl->widget_focused->type & WIDGET_FOCUSABLE) &&
+	  (wl->widget_focused->enable == WIDGET_ENABLE)) {
 	
+	event.type  = WIDGET_EVENT_FOCUS;
+	event.focus = FOCUS_LOST;
+
 	wl->widget_focused->have_focus = FOCUS_LOST;
-	(void) (wl->widget_focused->notify_focus) (wl, wl->widget_focused, FOCUS_LOST);
+	(void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
 	
-	if(wl->widget_focused->paint)
-	  (wl->widget_focused->paint) (wl->widget_focused, wl->win, wl->gc);
+	event.type = WIDGET_EVENT_PAINT;
+	(void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
 	
       }
       
@@ -1161,17 +1264,97 @@ void xitk_set_focus_to_next_widget(xitk_widget_list_t *wl, int backward) {
 
 }
 
+/*
+ *
+ */
+void xitk_set_focus_to_widget(xitk_widget_t *w) {
+  xitk_widget_t       *widget;
+  widget_event_t       event;
+  xitk_widget_list_t  *wl = w->wl;
+  
+  if(!wl) {
+    XITK_WARNING("widget list is NULL.\n");
+    return;
+  }
+
+  widget = (xitk_widget_t *) xitk_list_first_content (wl->l);
+  
+  while (widget && wl && wl->l && wl->win && wl->gc && (widget != w))
+    widget = (xitk_widget_t *) xitk_list_next_content(wl->l);
+  
+  if(widget) {
+    
+    if(wl->widget_focused) {
+      
+      /* Kill (hide) tips */
+      xitk_tips_tips_kill(wl->widget_focused);
+      
+      if ((wl->widget_focused->type & WIDGET_FOCUSABLE) && 
+	  (wl->widget_focused->enable == WIDGET_ENABLE)) {
+	event.type = WIDGET_EVENT_FOCUS;
+	event.focus = FOCUS_LOST;
+	wl->widget_focused->have_focus = FOCUS_LOST;
+	(void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
+	
+	event.type = WIDGET_EVENT_PAINT;
+	(void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
+      }
+    }
+    
+    wl->widget_focused = widget;
+    
+    if ((wl->widget_focused->type & WIDGET_FOCUSABLE) &&
+	(wl->widget_focused->enable == WIDGET_ENABLE)) {
+      event.type = WIDGET_EVENT_FOCUS;
+      event.focus = FOCUS_RECEIVED;
+      (void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
+      wl->widget_focused->have_focus = FOCUS_RECEIVED;
+
+      event.type = WIDGET_EVENT_PAINT;
+      (void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
+    }
+    
+    if (wl->widget_focused->enable == WIDGET_ENABLE && wl->widget_focused->running) {
+
+      if(wl->widget_focused->type & WIDGET_CLICKABLE)
+	wl->widget_under_mouse = wl->widget_focused;
+
+      if((wl->widget_focused->type & WIDGET_CLICKABLE) &&
+	 (wl->widget_focused->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_INPUTTEXT) {
+	widget_event_result_t  result;
+	
+	event.type           = WIDGET_EVENT_CLICK;
+	event.x              = wl->widget_focused->x + 1;
+	event.y              = wl->widget_focused->y + 1;
+	event.button_pressed = LBUTTON_DOWN;
+	event.button         = AnyButton;
+	(void) wl->widget_focused->event(wl->widget_focused, &event, &result);
+	
+	event.type = WIDGET_EVENT_PAINT;
+	(void) wl->widget_focused->event(wl->widget_focused, &event, NULL);
+      }
+      
+    }
+  }
+  else
+    XITK_WARNING("widget not found in list.\n");
+}
+
 /*
  * Call notify_keyevent, if exist, of specified plugin. This pass an X11 XEvent.
  */
-void xitk_send_key_event(xitk_widget_list_t *wl, xitk_widget_t *w, XEvent *xev) {
-  
+void xitk_send_key_event(xitk_widget_t *w, XEvent *xev) {
+  widget_event_t event;
+
   if(!w) {
     XITK_WARNING("widget is NULL\n");
     return;
   }
-  if(w->notify_keyevent)
-    w->notify_keyevent(wl, w, xev);
+
+  event.type   = WIDGET_EVENT_KEY_EVENT;
+  event.xevent = xev;
+  
+  (void) w->event(w, &event, NULL);
 }
 
 /*
@@ -1253,6 +1436,16 @@ int xitk_get_widget_pos(xitk_widget_t *w, int *x, int *y) {
   return 1;
 }
 
+uint32_t xitk_get_widget_type(xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return 0;
+  }
+
+  return w->type;
+}
+
 /*
  * Return 1 if widget is enabled.
  */
@@ -1286,54 +1479,68 @@ void xitk_enable_widget(xitk_widget_t *w) {
     XITK_WARNING("widget is NULL\n");
     return;
   }
+
   w->enable = WIDGET_ENABLE;
+  if(w->type & WIDGET_GROUP) {
+    widget_event_t  event;
+
+    event.type = WIDGET_EVENT_ENABLE;
+    (void) w->event(w, &event, NULL);
+  }
 }
 
 /*
  * Disable a widget.
  */
 void xitk_disable_widget(xitk_widget_t *w) {
+  widget_event_t  event;
 
   if(!w) {
     XITK_WARNING("widget is NULL\n");
     return;
   }
   
-
-  if(w->widget_list != NULL) {
+  if(w->wl != NULL) {
     
-    if((w->widget_list->widget_under_mouse != NULL) && (w == w->widget_list->widget_under_mouse)) {
+    if((w->wl->widget_under_mouse != NULL) && (w == w->wl->widget_under_mouse)) {
       /* Kill (hide) tips */
       xitk_tips_tips_kill(w);
     }
     
-    if((w->enable == WIDGET_ENABLE) && (w->have_focus != FOCUS_LOST)) {
-      
-      if(w->notify_focus)
-	(void) (w->notify_focus)(w->widget_list, w, FOCUS_LOST);
-      
-      if(w->paint)
-	(w->paint) (w, w->widget_list->win, w->widget_list->gc);
+    if((w->type & WIDGET_FOCUSABLE) && 
+       (w->enable == WIDGET_ENABLE) && (w->have_focus != FOCUS_LOST)) {
+
+      event.type  = WIDGET_EVENT_FOCUS;
+      event.focus = FOCUS_LOST;
+      (void) w->event(w, &event, NULL);
       
+      event.type = WIDGET_EVENT_PAINT;
+      (void) w->event(w, &event, NULL);
     }
   }
   
   w->enable = !WIDGET_ENABLE;
+  if(w->type & WIDGET_GROUP) {
+    event.type = WIDGET_EVENT_ENABLE;
+    (void) w->event(w, &event, NULL);
+  }
 }
 
 /*
  *
  */
 void xitk_free_widget(xitk_widget_t *w) {
-  
+  widget_event_t event;
   if(!w) {
     XITK_WARNING("widget is NULL\n");
     return;
   }
   
-  if(w->notify_destroy)
-    w->notify_destroy(w, NULL);
+  event.type = WIDGET_EVENT_DESTROY;
+  (void) w->event(w, &event, NULL);
   
+  xitk_tips_tips_kill(w);
+  XITK_FREE(w->tips_string);
   XITK_FREE(w);
   w = NULL;
   
@@ -1342,14 +1549,14 @@ void xitk_free_widget(xitk_widget_t *w) {
 /*
  * Destroy a widget.
  */
-void xitk_destroy_widget(xitk_widget_list_t *wl, xitk_widget_t *w) {
+void xitk_destroy_widget(xitk_widget_t *w) {
 
   if(!w) {
     XITK_WARNING("widget is NULL\n");
     return;
   }
 
-  xitk_hide_widget(wl, w);
+  xitk_hide_widget(w);
   xitk_stop_widget(w);
   xitk_disable_widget(w);
 
@@ -1363,7 +1570,7 @@ void xitk_destroy_widgets(xitk_widget_list_t *wl) {
   xitk_widget_t *mywidget;
   
   if(!wl) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return;
   }
   
@@ -1371,7 +1578,7 @@ void xitk_destroy_widgets(xitk_widget_list_t *wl) {
 
   while(mywidget) {
     
-    xitk_destroy_widget(wl, mywidget);
+    xitk_destroy_widget(mywidget);
     
     mywidget = (xitk_widget_t *) xitk_list_prev_content (wl->l);
   }
@@ -1393,7 +1600,7 @@ xitk_color_names_t *xitk_get_color_name(char *color) {
   xitk_color_names_t *pcn = NULL;
 
   if(color == NULL) {
-    XITK_WARNING("%s(): color was NULL.\n", __FUNCTION__);
+    XITK_WARNING("color was NULL.\n");
     return NULL;
   }
 
@@ -1497,7 +1704,7 @@ void xitk_stop_widgets(xitk_widget_list_t *wl) {
   xitk_widget_t *mywidget;
   
   if(!wl) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return;
   }
     
@@ -1514,7 +1721,7 @@ void xitk_stop_widgets(xitk_widget_list_t *wl) {
 /*
  * Show a widget.
  */
-void xitk_show_widget(xitk_widget_list_t *wl, xitk_widget_t *w) {
+void xitk_show_widget(xitk_widget_t *w) {
 
   if(!w) {
     XITK_WARNING("widget is NULL\n");
@@ -1522,10 +1729,12 @@ void xitk_show_widget(xitk_widget_list_t *wl, xitk_widget_t *w) {
   }
 
   if(w->visible == 0) {
+    widget_event_t  event;
+    
     w->visible = 1;
     
-    if(w->paint)
-      w->paint(w, wl->win, wl->gc);
+    event.type = WIDGET_EVENT_PAINT;
+    (void) w->event(w, &event, NULL);
   }
 }
 
@@ -1536,7 +1745,7 @@ void xitk_show_widgets(xitk_widget_list_t *wl) {
   xitk_widget_t *mywidget;
   
   if(!wl) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return;
   }
     
@@ -1547,7 +1756,7 @@ void xitk_show_widgets(xitk_widget_list_t *wl) {
     if(mywidget->visible == -1) 
       mywidget->visible = 0;
     else
-      xitk_show_widget(wl, mywidget);
+      xitk_show_widget(mywidget);
     
     mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l);
   }
@@ -1556,7 +1765,7 @@ void xitk_show_widgets(xitk_widget_list_t *wl) {
 /*
  * Hide a widget.
  */
-void xitk_hide_widget(xitk_widget_list_t *wl, xitk_widget_t *w) {
+void xitk_hide_widget(xitk_widget_t *w) {
 
   if(!w) {
     XITK_WARNING("widget is NULL\n");
@@ -1564,10 +1773,14 @@ void xitk_hide_widget(xitk_widget_list_t *wl, xitk_widget_t *w) {
   }
 
   if(w->visible == 1) {
-    w->visible = 0;
+    widget_event_t event;
     
-    if(w->paint)
-      w->paint(w, wl->win, wl->gc);
+    w->visible = 0;
+
+    xitk_tips_tips_kill(w);
+    
+    event.type = WIDGET_EVENT_PAINT;
+    (void) w->event(w, &event, NULL);
   }
 }
 
@@ -1578,7 +1791,7 @@ void xitk_hide_widgets(xitk_widget_list_t *wl) {
   xitk_widget_t *mywidget;
   
   if(!wl) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return;
   }
     
@@ -1589,7 +1802,7 @@ void xitk_hide_widgets(xitk_widget_list_t *wl) {
     if(mywidget->visible == 0)
       mywidget->visible = -1;
     else
-      xitk_hide_widget(wl, mywidget);
+      xitk_hide_widget(mywidget);
     
     mywidget = (xitk_widget_t *) xitk_list_next_content (wl->l);
   }
@@ -1599,32 +1812,44 @@ void xitk_hide_widgets(xitk_widget_list_t *wl) {
  *
  */
 xitk_image_t *xitk_get_widget_foreground_skin(xitk_widget_t *w) {
+  widget_event_t         event;
+  widget_event_result_t  result;
+  xitk_image_t          *image = NULL;
 
   if(!w) {
     XITK_WARNING("widget is NULL\n");
     return NULL;
   }
+  
+  event.type = WIDGET_EVENT_GET_SKIN;
+  event.skin_layer = FOREGROUND_SKIN;
+  
+  if(w->event(w, &event, &result))
+    image = result.image;
 
-  if(w->get_skin)
-    return w->get_skin(w, FOREGROUND_SKIN);
-
-  return NULL;
+  return image;
 }
 
 /*
  *
  */
 xitk_image_t *xitk_get_widget_background_skin(xitk_widget_t *w) {
+  widget_event_t         event;
+  widget_event_result_t  result;
+  xitk_image_t          *image = NULL;
 
   if(!w) {
     XITK_WARNING("widget is NULL\n");
     return NULL;
   }
 
-  if(w->get_skin)
-    return w->get_skin(w, BACKGROUND_SKIN);
+  event.type = WIDGET_EVENT_GET_SKIN;
+  event.skin_layer = BACKGROUND_SKIN;
+  
+  if(w->event(w, &event, &result))
+    image = result.image;
 
-  return NULL;
+  return image;
 }
 
 /*
@@ -1701,7 +1926,7 @@ void xitk_disable_widgets_tips(xitk_widget_list_t *wl) {
   xitk_widget_t *mywidget;
   
   if(!wl) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return;
   }
   
@@ -1722,7 +1947,7 @@ void xitk_enable_widgets_tips(xitk_widget_list_t *wl) {
   xitk_widget_t *mywidget;
   
   if(!wl) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return;
   }
   
@@ -1744,7 +1969,7 @@ void xitk_set_widgets_tips_timeout(xitk_widget_list_t *wl, unsigned long timeout
   xitk_widget_t *mywidget;
   
   if(!wl) {
-    XITK_WARNING("%s(): widget list was NULL.\n", __FUNCTION__);
+    XITK_WARNING("widget list was NULL.\n");
     return;
   }
 
@@ -1771,3 +1996,150 @@ void xitk_set_widget_tips_timeout(xitk_widget_t *w, unsigned long timeout) {
 
   xitk_tips_set_timeout(w, timeout);
 }
+
+int xitk_is_mouse_over_widget(Display *display, Window window, xitk_widget_t *w) {
+  Bool            ret;
+  Window          root_window, child_window;
+  int             root_x, root_y, win_x, win_y;
+  unsigned int    mask;
+  int             retval = 0;
+  
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return 0;
+  }
+  if(window == None) {
+    XITK_WARNING("window is None\n");
+    return 0;
+  }
+  
+  XLOCK(display);
+  ret = XQueryPointer(display, window, 
+		      &root_window, &child_window, &root_x, &root_y, &win_x, &win_y, &mask);
+  XUNLOCK(display);
+  
+  if((ret == False) ||
+     ((child_window == None) && (win_x == 0) && (win_y == 0))) {
+    retval = 0;
+  }
+  else {
+    
+    if(((win_x >= w->x) && (win_x <= (w->x + w->width))) && 
+       ((win_y >= w->y) && (win_y <= (w->y + w->height)))) {
+      retval = 1;
+    }
+    
+  }
+  
+  return retval;
+}
+
+int xitk_get_mouse_coords(Display *display, Window window, int *x, int *y, int *rx, int *ry) {
+  Bool            ret;
+  Window          root_window, child_window;
+  int             root_x, root_y, win_x, win_y;
+  unsigned int    mask;
+  int             retval = 0;
+
+  if(window == None) {
+    XITK_WARNING("window is None\n");
+    return 0;
+  }
+  
+  XLOCK(display);
+  ret = XQueryPointer(display, window, 
+		      &root_window, &child_window, &root_x, &root_y, &win_x, &win_y, &mask);
+  XUNLOCK(display);
+  
+  if((ret == False) ||
+     ((child_window == None) && (win_x == 0) && (win_y == 0))) {
+    retval = 0;
+  }
+  else {
+
+    if(x)
+      *x = win_x;
+    if(y)
+      *y = win_y;
+    if(rx)
+      *rx = root_x;
+    if(ry)
+      *ry = root_y;
+    
+    retval = 1;
+  }
+  
+  return retval;
+}
+
+int xitk_widget_list_set(xitk_widget_list_t *wl, int param, void *data) {
+
+  if(!wl) {
+    XITK_WARNING("List is NULL\n");
+    return 0;
+  }
+
+  switch(param) {
+  case WIDGET_LIST_GC:
+    wl->gc = (GC) data;
+    break;
+  case WIDGET_LIST_WINDOW:
+    wl->win = (Window) data;
+    break;
+  case WIDGET_LIST_LIST:
+    wl->l = (xitk_list_t *) data;
+    break;
+  default:
+    XITK_WARNING("Unknown param %d\n", param);
+    return 0;
+    break;
+  }
+  return 1;
+}
+
+void *xitk_widget_list_get(xitk_widget_list_t *wl, int param) {
+  void *data = NULL;
+
+  if(!wl) {
+    XITK_WARNING("List is NULL\n");
+    return NULL;
+  }
+
+  switch(param) {
+  case WIDGET_LIST_GC:
+    if(wl->gc)
+      data = (void *) wl->gc;
+    else
+      XITK_WARNING("widget list GC unset\n");
+    break;
+  case WIDGET_LIST_WINDOW:
+    if(wl->win)
+      data = (void *) wl->win;
+    else
+      XITK_WARNING("widget list window unset\n");
+    break;
+  case WIDGET_LIST_LIST:
+    if(wl->l)
+      data = (void *) wl->l;
+    else
+      XITK_WARNING("widget list list unset\n");
+    break;
+  default:
+    XITK_WARNING("Unknown param %d\n", param);
+    break;
+  }
+  
+  return data;
+}
+
+void xitk_widget_keyable(xitk_widget_t *w, int keyable) {
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+  
+  if(keyable)
+    w->type |= WIDGET_KEYABLE;
+  else
+    w->type &= ~WIDGET_KEYABLE;
+}
